@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bf_lib.c - non-GUI general functions
  *
- * Copyright (C) 2000 Olivier Sessink
+ * Copyright (C) 2000-2004 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,93 @@
 #define DIRSTR "/"
 #define DIRCHR '/'
 #endif
+/**
+ * get_filename_on_disk_encoding
+ *
+ * if gnome_vfs is defined, this function will also escape local paths
+ * to make sure we can open files with a # in their name
+ */
+gchar *get_filename_on_disk_encoding(const gchar *utf8filename) {
+	if (utf8filename) {
+		GError *gerror=NULL;
+		gsize b_written;
+		gchar *ondiskencoding = g_filename_from_utf8(utf8filename,-1, NULL,&b_written,&gerror);
+		if (gerror) {
+			g_print(_("Bluefish has trouble reading the filenames. Try to set the environment variable G_BROKEN_FILENAMES=1\n"));
+			ondiskencoding = g_strdup(utf8filename);
+		}
+#ifdef HAVE_GNOME_VFS
+		/* convert local path's */
+		if (ondiskencoding[0] == '/') {
+			gchar *tmp = gnome_vfs_escape_path_string(ondiskencoding);
+			g_free(ondiskencoding);
+			ondiskencoding = tmp;
+		}
+#endif /* HAVE_GNOME_VFS */
+		return ondiskencoding;
+	}
+	return NULL;
+}
+
+gchar *get_utf8filename_from_on_disk_encoding(const gchar *encodedname) {
+	if (encodedname) {
+		GError *gerror=NULL;
+		gsize b_written;
+		gchar *ondiskencoding = g_filename_to_utf8(encodedname,-1, NULL,&b_written,&gerror);
+		if (gerror) {
+			g_print(_("Bluefish has trouble reading the filenames. Try to set the environment variable G_BROKEN_FILENAMES=1\n"));
+			ondiskencoding = g_strdup(encodedname);
+		}
+		DEBUG_MSG("get_utf8filename_from_on_disk_encoding, utf8filename=%s\n",ondiskencoding);
+		return ondiskencoding;
+	}
+	return NULL;
+}
+
+gboolean string_is_color(const gchar *color) {
+	GdkColor gcolor;
+	return gdk_color_parse(color, &gcolor);
+}
+
+static void fill_rwx(short unsigned int bits, char *chars) {
+	chars[0] = (bits & S_IRUSR) ? 'r' : '-';
+	chars[1] = (bits & S_IWUSR) ? 'w' : '-';
+	chars[2] = (bits & S_IXUSR) ? 'x' : '-';
+}
+static void fill_setid(short unsigned int bits, char *chars) {
+#ifdef S_ISUID
+	if (bits & S_ISUID) {
+		/* Set-uid, but not executable by owner.  */
+		if (chars[3] != 'x') chars[3] = 'S';
+		else chars[3] = 's';
+	}
+#endif
+#ifdef S_ISGID
+	if (bits & S_ISGID) {
+		/* Set-gid, but not executable by group.  */
+		if (chars[6] != 'x') chars[6] = 'S';
+		else chars[6] = 's';
+	}
+#endif
+#ifdef S_ISVTX
+	if (bits & S_ISVTX) {
+		/* Sticky, but not executable by others.  */
+		if (chars[9] != 'x') chars[9] = 'T';
+		else chars[9] = 't';
+	}
+#endif
+}
+gchar *filemode_to_string(mode_t statmode) {
+	gchar *str = g_malloc0(10);
+ 	/* following code "adapted" from GNU filemode.c program */
+	fill_rwx((statmode & 0700) << 0, &str[0]);
+	fill_rwx((statmode & 0070) << 3, &str[3]);
+	fill_rwx((statmode & 0007) << 6, &str[6]);
+	fill_setid(statmode, str);
+	return str;
+}
+
+
 /**
  * return_root_with_protocol:
  * @url: #const gchar* with the url 
@@ -128,10 +215,15 @@ gboolean file_copy(gchar *source, gchar *dest) {
 	GnomeVFSFileSize bytes_read, bytes_written;
 	guint buffer[BYTES_TO_PROCESS];
 	GnomeVFSResult result;
+	gchar *OnDiEn_source, *OnDiEn_dest;
+	OnDiEn_source = get_filename_on_disk_encoding(source);
+	OnDiEn_dest = get_filename_on_disk_encoding(dest);
 	
-	result = gnome_vfs_open(&read_handle, source, GNOME_VFS_OPEN_READ);
+	result = gnome_vfs_open(&read_handle, OnDiEn_source, GNOME_VFS_OPEN_READ);
+	g_free(OnDiEn_source);
 	if (result != GNOME_VFS_OK) return FALSE;
-	result = gnome_vfs_create(&write_handle, dest, GNOME_VFS_OPEN_WRITE, FALSE, 0644);
+	result = gnome_vfs_create(&write_handle, OnDiEn_dest, GNOME_VFS_OPEN_WRITE, FALSE, 0644);
+	g_free(OnDiEn_dest);
 	if (result != GNOME_VFS_OK) {
 		gnome_vfs_close(read_handle);
 		return FALSE;
@@ -159,12 +251,17 @@ gboolean file_copy(gchar *source, gchar *dest) {
 #endif
 	int c;
 	FILE *in, *out;
-	
-	in = fopen(source, "r");
+	gchar *OnDiEn_source, *OnDiEn_dest;
+	OnDiEn_source = get_filename_on_disk_encoding(source);
+	OnDiEn_dest = get_filename_on_disk_encoding(dest);
+
+	in = fopen(OnDiEn_source, "r");
+	g_free(OnDiEn_source);
 	if (!in) {
 		return FALSE;
 	}
-	out = fopen(dest, "w");
+	out = fopen(OnDiEn_dest, "w");
+	g_free(OnDiEn_dest);
 	if (!out) {
 		fclose(in);
 		return FALSE;
@@ -230,8 +327,9 @@ gint find_common_prefixlen_in_stringlist(GList *stringlist) {
  **/
 gboolean append_string_to_file(gchar *filename, gchar *string) {
 	FILE *out;
-
-	out = fopen(filename, "a");
+	gchar *ondiskencoding = get_filename_on_disk_encoding(filename);
+	out = fopen(ondiskencoding, "a");
+	g_free(ondiskencoding);
 	if (!out) {
 		DEBUG_MSG("append_to_file, could not open file %s for append\n", filename);
 		return FALSE;
@@ -448,6 +546,7 @@ static Tconvert_table standardescapetable [] = {
 };
 gchar *unescape_string(const gchar *original, gboolean escape_colon) {
 	gchar *string, *tmp=NULL;
+	DEBUG_MSG("unescape_string, started\n");
 	if (!escape_colon) {
 		tmp = standardescapetable[9].my_char;
 		standardescapetable[9].my_char = NULL;
@@ -460,6 +559,7 @@ gchar *unescape_string(const gchar *original, gboolean escape_colon) {
 }
 gchar *escape_string(const gchar *original, gboolean escape_colon) {
 	gchar *string, *tmp=NULL;
+	DEBUG_MSG("escape_string, started\n");
 	if (!escape_colon) {
 		tmp = standardescapetable[9].my_char;
 		standardescapetable[9].my_char = NULL;
@@ -471,17 +571,19 @@ gchar *escape_string(const gchar *original, gboolean escape_colon) {
 	return string;
 }
 Tconvert_table *new_convert_table(gint size, gboolean fill_standardescape) {
-	gint realsize = (fill_standardescape) ? size + 10 : size +1;
-	Tconvert_table * tct = g_new(Tconvert_table, realsize);
-	DEBUG_MSG("new_convert_table, size=%d, realsize=%d\n",size,realsize);
+	gint realsize = (fill_standardescape) ? size + 10 : size;
+	Tconvert_table * tct = g_new(Tconvert_table, realsize+1);
+	DEBUG_MSG("new_convert_table, size=%d, realsize=%d,alloced=%d\n",size,realsize,realsize+1);
 	if (fill_standardescape) {
 		gint i;
 		for (i=size;i<realsize;i++) {
 			tct[i].my_int = standardescapetable[i-size].my_int;
 			tct[i].my_char = g_strdup(standardescapetable[i-size].my_char);
 		}
+		DEBUG_MSG("new_convert_table, setting tct[%d] (i) to NULL\n",i);
 		tct[i].my_char = NULL;
 	} else {
+		DEBUG_MSG("new_convert_table, setting tct[%d] (size) to NULL\n",size);
 		tct[size].my_char = NULL;
 	}
 	return tct;
@@ -500,12 +602,30 @@ void free_convert_table(Tconvert_table *tct) {
 /* byte offset to UTF8 character offset functions */
 /**************************************************/
 
-#define UTF8_OFFSET_CACHE_SIZE 32
+/*
+html files usually have enough cache at size 4
+large php files, a cache of 
+	10 resulted in 160% of the buffer to be parsed
+	12 resulted in 152% of the buffer to be parsed
+	14 resulted in 152% of the buffer to be parsed
+	16 resulted in 152% of the buffer to be parsed
+so we keep it at 10 for the moment
+*/
+#define UTF8_OFFSET_CACHE_SIZE 10
+/* #define UTF8_BYTECHARDEBUG */
 
 typedef struct {
 	gchar *last_buf;
+	/* the two arrays must be grouped and in this order, because they are moved back 
+	one position in ONE memmove() call */
 	guint  last_byteoffset[UTF8_OFFSET_CACHE_SIZE];
 	guint  last_charoffset[UTF8_OFFSET_CACHE_SIZE];
+#ifdef UTF8_BYTECHARDEBUG
+	guint numcalls_since_reset;
+	unsigned long long int numbytes_parsed;
+	guint numcalls_cached_since_reset;
+	unsigned long long int numbytes_cached_parsed;
+#endif
 } Tutf8_offset_cache;
 
 static Tutf8_offset_cache utf8_offset_cache;
@@ -527,6 +647,10 @@ static Tutf8_offset_cache utf8_offset_cache;
 __inline__ 
 #endif
 void utf8_offset_cache_reset() {
+#ifdef UTF8_BYTECHARDEBUG
+	g_print("UTF8_BYTECHARDEBUG: called %d times for total %llu bytes\n",utf8_offset_cache.numcalls_since_reset,utf8_offset_cache.numbytes_parsed);
+	g_print("UTF8_BYTECHARDEBUG: cache HIT %d times, reduced to %llu bytes, cache size %d\n",utf8_offset_cache.numcalls_cached_since_reset,utf8_offset_cache.numbytes_cached_parsed,UTF8_OFFSET_CACHE_SIZE);
+#endif
 	memset(&utf8_offset_cache, 0, sizeof(Tutf8_offset_cache));
 }
 /**
@@ -561,8 +685,16 @@ guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 	
 	if (i > 0) {
 		retval = g_utf8_pointer_to_offset(string+utf8_offset_cache.last_byteoffset[i], string+byteoffset)+utf8_offset_cache.last_charoffset[i];
+#ifdef UTF8_BYTECHARDEBUG
+		utf8_offset_cache.numbytes_parsed += (byteoffset - utf8_offset_cache.last_byteoffset[i]);
+		utf8_offset_cache.numbytes_cached_parsed += (byteoffset - utf8_offset_cache.last_byteoffset[i]);
+		utf8_offset_cache.numcalls_cached_since_reset++;
+#endif
 	} else {
 		retval = g_utf8_pointer_to_offset(string, string+byteoffset);
+#ifdef UTF8_BYTECHARDEBUG
+		utf8_offset_cache.numbytes_parsed += byteoffset;
+#endif
 	}
 	if (i == (UTF8_OFFSET_CACHE_SIZE-1)) {
 		/* add this new calculation to the cache */
@@ -572,6 +704,9 @@ guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 		utf8_offset_cache.last_byteoffset[UTF8_OFFSET_CACHE_SIZE-1] = byteoffset;
 		utf8_offset_cache.last_charoffset[UTF8_OFFSET_CACHE_SIZE-1] = retval;
 	}
+#ifdef UTF8_BYTECHARDEBUG
+	utf8_offset_cache.numcalls_since_reset++;
+#endif
 	return retval;
 }
 
@@ -684,30 +819,6 @@ guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 	return newstring;
 }*/
 
-/**
- * change_dir:
- * @filename: a gchar * with the path (may be a filename!!) to change to
- *
- * this function will get the directory name from the filename
- * and chdir() to that directory
- * 
- * Return value: gboolean, returning what chdir() returned
- **/
-gboolean change_dir(gchar * filename)
-{
-	gchar *tmpstring;
-	gboolean returncode;
-
-	if (!filename)
-		return FALSE;
-	tmpstring = g_path_get_dirname(filename);
-	DEBUG_MSG("change_dir, filename = %s, tmpstring = %s in %p\n", filename, tmpstring, tmpstring);
-	returncode = chdir(tmpstring) == 0 ? TRUE : FALSE;
-	g_free(tmpstring);
-	DEBUG_MSG("change_dir, after gfree(tmpstring) \n");
-	return returncode;
-}
-
 /**
  * strip_any_whitespace:
  * @string: a gchar * to strip
@@ -925,7 +1036,12 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	g_free(eff_link_to_filepath);
 	return returnstring;
 }
-
+#ifdef HAVE_GNOME_VFS
+#define STRIP_FILE_URI
+#endif
+#ifdef HAVE_ATLEAST_GTK_2_4
+#define STRIP_FILE_URI
+#endif
 /**
  * create_full_path:
  * @filename: a gchar * with the (relative or not) filename
@@ -935,19 +1051,34 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
  * else it will use basedir if available, else the current dir
  * to add to the filename to form the full path
  *
+ * for URL's it will simply return a strdup(), except for file:// URL's, 
+ * there the file:// bit is stripped and 
+ * IF YOU HAVE GNOME_VFS any %XX sequenves are converted
+ * so if you DON'T have gnome_vfs, you should not feed file:// uri's!!
+ *
  * it does use most_efficient_filename() to remote unwanted dir/../ entries
  *
  * Return value: a newly allocated gchar * with the full path
  **/
-gchar *create_full_path(gchar * filename, gchar *basedir) {
+gchar *create_full_path(const gchar * filename, const gchar *basedir) {
 	gchar *absolute_filename;
 	gchar *tmpcdir;
 
+	if (!filename) return NULL;
 	DEBUG_MSG("create_full_path, filename=%s, basedir=%s\n", filename, basedir);
-#ifdef HAVE_GNOME_VFS
-	if (strchr(filename, ':') != NULL) {
+#ifdef STRIP_FILE_URI
+	if (strchr(filename, ':') != NULL) { /* it is an URI!! */
 		DEBUG_MSG("create_full_path, %s is an URI\n",filename);
-		return g_strdup(filename);
+		if (strncmp(filename, "file://", 7)==0) {
+#ifdef HAVE_GNOME_VFS
+			return gnome_vfs_get_local_path_from_uri(filename);
+#else
+			/* THIS IS A BUG, IF YOU DON'T HAVE GNOME_VFS BUT YOU DO HAVE 
+			GTK-2.4 A %21 OR SOMETHING LIKE THAT IS NOW NOT CONVERTED !!!!!!!!! */
+			return g_strdup(filename+7); /* file:// URI's are never relative paths */
+#endif
+		}
+		return g_strdup(filename); /* cannot do this on remote paths */
 	}
 #endif /* HAVE_GNOME_VFS */
 	if (g_path_is_absolute(filename)) {
@@ -1006,6 +1137,7 @@ gchar *path_get_dirname_with_ending_slash(const gchar *filename) {
 		return NULL;
 	}
 }
+
 /**
  * file_exists_and_readable:
  * @filename: a #const gchar * with a file path
@@ -1019,6 +1151,8 @@ gchar *path_get_dirname_with_ending_slash(const gchar *filename) {
  * Return value: gboolean, TRUE if readable, else FALSE
  **/
 gboolean file_exists_and_readable(const gchar * filename) {
+	gchar *ondiskencoding;
+	gboolean retval=TRUE;
 #ifdef DEVELOPMENT
 	g_assert(filename);
 #endif
@@ -1027,30 +1161,29 @@ gboolean file_exists_and_readable(const gchar * filename) {
 		return FALSE;
 	}
 	DEBUG_MSG("file_exists_and_readable, filename(%p)=\"%s\", strlen(filename)=%d\n", filename, filename, strlen(filename));
+	ondiskencoding = get_filename_on_disk_encoding(filename);
+	DEBUG_MSG("file_exists_and_readable, ondiskencoding='%s'\n",ondiskencoding);
 #ifndef WIN32
-
 #ifdef HAVE_GNOME_VFS
 	{
-		gboolean result;
-		GnomeVFSURI* uri = gnome_vfs_uri_new(filename);
-		result = gnome_vfs_uri_exists(uri);
+		GnomeVFSURI* uri;
+		uri = gnome_vfs_uri_new(ondiskencoding);
+		retval = gnome_vfs_uri_exists(uri);
+		DEBUG_MSG("gnome_vfs_uri has path %s\n",gnome_vfs_uri_get_path(uri));
 		gnome_vfs_uri_unref(uri);
-		DEBUG_MSG("file_exists_and_readable, return %d for %s\n",result,filename);
-		return result;
+		DEBUG_MSG("file_exists_and_readable, return %d for %s\n",retval,filename);
 	}
 #else /* HAVE_GNOME_VFS */
 	{
 		struct stat naamstat;
-		if ((stat(filename, &naamstat) == -1) && (errno == ENOENT)) {
-			return FALSE;
-		} else {
-			return TRUE;
-		}
+		errno = 0;
+		retval = ((stat(ondiskencoding, &naamstat) == 0) && (errno == 0));
+		DEBUG_MSG("file_exists_and_readable, retval=%d (ernno=%d) for %s\n",retval,errno,ondiskencoding);
 	}
 #endif /* HAVE_GNOME_VFS */
-#else /* WIN32 */
-	return TRUE;
+	g_free(ondiskencoding);
 #endif /* WIN32 */
+	return retval;
 }
 /**
  * return_first_existing_filename:
@@ -1283,3 +1416,23 @@ void wordcount(gchar *text, guint *chars, guint *lines, guint *words)
 	/* We start counting line numbers from 1 */
 	if(*chars > 0) (*lines)++;
 }
+
+GSList *gslist_from_glist(GList *src) {
+	GSList *target=NULL;
+	GList *tmplist = g_list_first(src);
+	while (tmplist) {
+		target = g_slist_append(target, tmplist->data);
+		tmplist = g_list_next(tmplist);
+	}
+	return target;
+}
+
+GList *glist_from_gslist(GSList *src) {
+	GList *target=NULL;
+	GSList *tmplist = src;
+	while (tmplist) {
+		target = g_list_append(target, tmplist->data);
+		tmplist = g_slist_next(tmplist);
+	}
+	return target;
+}
