@@ -30,6 +30,9 @@
 #ifdef HAVE_GNOME_VFS
 #include <libgnomevfs/gnome-vfs.h>
 #endif
+#ifdef HAVE_ATLEAST_GNOMEUI_2_6
+#include <libgnomeui/libgnomeui.h>
+#endif
 
 #ifdef ENABLE_NLS
 #include <locale.h>
@@ -139,19 +142,28 @@ int main(int argc, char *argv[])
 	DEBUG_MSG("set bindtextdomain for %s to %s\n",PACKAGE,LOCALEDIR);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);                                                    
-#endif    
+#endif
+#ifdef HAVE_ATLEAST_GNOMEUI_2_6
+	gnome_init(PACKAGE, VERSION, argc, argv);
+#else
 	gtk_init(&argc, &argv);
+#endif /* HAVE_ATLEAST_GNOMEUI_2_6
+ */
 #ifdef HAVE_GNOME_VFS
 	DEBUG_MSG("main, we have gnome_vfs, so we init it\n");
 	gnome_vfs_init();
-#ifdef HAVE_ATLEAST_GNOMEVFS_2_5
+#ifdef HAVE_ATLEAST_GNOMEUI_2_6
+	gnome_authentication_manager_init();
+#else
+#ifdef HAVE_ATLEAST_GNOMEVFS_2_6
 	set_authen_callbacks();
-#endif
-#endif
+#endif /* HAVE_ATLEAST_GNOMEVFS_2_6 */
+#endif /* HAVE_ATLEAST_GNOME_2_6 */
+#endif /* HAVE_GNOME_VFS */
 	main_v = g_new0(Tmain, 1);
 	main_v->session = g_new0(Tsessionvars,1);
 	DEBUG_MSG("main, main_v is at %p\n", main_v);
-	
+
 	rcfile_check_directory();
 	rcfile_parse_main();
 	
@@ -184,7 +196,7 @@ int main(int argc, char *argv[])
 #ifndef NOSPLASH
 	splash_screen_set_label(_("parsing custom menu file..."));
 #endif /* #ifndef NOSPLASH */
-	rcfile_parse_custom_menu();
+	rcfile_parse_custom_menu(FALSE,FALSE);
 	main_v->tooltips = gtk_tooltips_new();
 	fref_init();
 	bmark_init();
@@ -244,6 +256,7 @@ int main(int argc, char *argv[])
 	/* do the cleanup */
 	msg_queue_cleanup();
 #endif /* WITH_MSG_QUEUE */
+	DEBUG_MSG("Bluefish: exiting cleanly\n");
 	return 0;
 }
 
@@ -256,14 +269,24 @@ void bluefish_exit_request() {
 	tmplist = return_allwindows_documentlist();
 	tmpb = (tmplist && test_docs_modified(tmplist));
 	g_list_free(tmplist);
-	if (tmpb) {
-		tmplist = g_list_first(main_v->bfwinlist);
-		while (tmplist) {
-			file_close_all_cb(NULL, BFWIN(tmplist->data));
-			tmplist = g_list_next(tmplist);
+	tmplist = g_list_first(main_v->bfwinlist);
+	while (tmplist) {
+		/* if there is a project, we anyway want to save & close the project */
+		if (BFWIN(tmplist->data)->project) {
+			if (!project_save_and_close(BFWIN(tmplist->data))) {
+				/* cancelled or error! */
+				DEBUG_MSG("bluefish_exit_request, project_save_and_close returned FALSE\n");
+				return;
+			}
 		}
-		/* if we still have modified documents we don't do a thing,
-		 if we don't have them we can quit */
+		if (tmpb) {
+			file_close_all_cb(NULL, BFWIN(tmplist->data));
+		}
+		tmplist = g_list_next(tmplist);
+	}
+	/* if we still have modified documents we don't do a thing,
+	 if we don't have them we can quit */
+	if (tmpb) {
 		tmplist = return_allwindows_documentlist();
 		tmpb = (tmplist && test_docs_modified(tmplist));
 		g_list_free(tmplist);
@@ -305,5 +328,6 @@ void bluefish_exit_request() {
 	/* do the cleanup */
 	msg_queue_cleanup();
 #endif /* WITH_MSG_QUEUE */
+	DEBUG_MSG("Bluefish: exiting cleanly\n");
 	exit(0);
 }
