@@ -2,7 +2,7 @@
  * bluefish.h - global prototypes
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
- * Copyright (C) 1999-2002 Olivier Sessink
+ * Copyright (C) 1999-2004 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,22 +38,22 @@
 #define DEBUG_MSG(args...)
  /**/
 #else/* notdef __GNUC__ */
-extern void g_none(...);
+extern void g_none(gchar *first, ...);
 #define DEBUG_MSG g_none
 #endif /* __GNUC__ */
 #endif /* DEBUG */
 
-#ifdef ENABLE_NLS                                                               
-                                                                                
-#include <libintl.h>                                                            
-#define _(String) gettext (String)                                              
-#define N_(String) (String)                                                     
-                                                                                
+#ifdef ENABLE_NLS
+
+#include <libintl.h>
+#define _(String) gettext (String)
+#define N_(String) (String)
+
 #else                                                                           
-                                                                                
-#define _(String)(String)                                                       
-#define N_(String)(String)                                                      
-                                                                                
+
+#define _(String)(String)
+#define N_(String)(String)
+
 #endif    
 
 
@@ -65,6 +65,7 @@ extern void g_none(...);
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <pcre.h>
 
 #ifdef HAVE_GNOME_VFS
 #include <libgnomevfs/gnome-vfs.h>
@@ -102,6 +103,7 @@ typedef struct {
 	gchar *update_chars;
 	GList *highlightlist;
 	gboolean editable; /* this a type that can be edited by Bluefish */
+	gint autoclosingtag; /* 0=off, 1=xml mode, 2=html mode */
 	gchar *content_regex; /* a regex pattern to test the filetype using the content */
 } Tfiletype;
 
@@ -112,7 +114,7 @@ typedef struct {
 #define DOCUMENT(var) ((Tdocument *)(var))
 
 typedef struct {
-	gchar *filename; /* or NULL if no filename known */
+	gchar *filename; /* this is the UTF-8 encoded filename, before you use it on disk you need convert to disk-encoding! */
 	gchar *encoding;
 	gint modified;
 /*	time_t mtime; */ /* from stat() */
@@ -128,8 +130,10 @@ typedef struct {
 	unre_t unre;
 	GtkWidget *view;
 	GtkWidget *tab_label;
+	GtkWidget *tab_eventbox;
 	GtkWidget *tab_menu;
 	GtkTextBuffer *buffer;
+	gpointer paste_operation;
 	gint last_rbutton_event; /* index of last 3rd button click */
 	Tfiletype *hl; /* filetype & highlighting set to use for this document */
 	gint need_highlighting; /* if you open 10+ documents you don't need immediate highlighting, just set this var, and notebook_switch() will trigger the actual highlighting when needed */
@@ -137,8 +141,11 @@ typedef struct {
 	gboolean wrapstate; /* does this document use wrap?*/
 	gboolean linenumberstate; /* does this document use linenumbers? */
 	gboolean overwrite_mode; /* is document in overwrite mode */
+	gboolean autoclosingtag; /* does the document use autoclosing of tags */
 	gpointer floatingview; /* a 2nd textview widget that has its own window */
 	gpointer bfwin;
+	GtkTreeIter *bmark_parent; /* if NULL this document doesn't have bookmarks, if 
+									it does have bookmarks they are children of this GtkTreeIter */
 } Tdocument;
 
 typedef struct {
@@ -150,6 +157,7 @@ typedef struct {
 	gint filebrowser_show_hidden_files;
 	gint filebrowser_show_backup_files;
 	gint filebrowser_two_pane_view; /* have one or two panes in the filebrowser */
+	gint filebrowser_focus_follow; /* have the directory of the current document in focus */
 	gchar *filebrowser_unknown_icon;
 	gchar *filebrowser_dir_icon;
 	gchar *editor_font_string;		/* editor font */
@@ -161,7 +169,6 @@ typedef struct {
 	GList *cust_menu; 		/* DEPRECATED entries in the custom menu */
 	GList *cmenu_insert; /* custom menu inserts */
 	GList *cmenu_replace; /* custom menu replaces */
-	GList *quickbar_items; /* items in the quickbar toolbar */
 	gint highlight_num_lines_count; /* number of lines to highlight in continous highlighting */	
 	gint defaulthighlight;		/* highlight documents by default */
 	GList *filetypes; /* filetypes for highlighting and filtering */
@@ -172,9 +179,7 @@ typedef struct {
 	gint transient_htdialogs;  /* set html dialogs transient ro the main window */
 	gint restore_dimensions; /* use the dimensions as used the previous run */
 	gint left_panel_width; 	/* width of filelist */
-	gint two_pane_filebrowser_height; /* position of the pane separater on the two paned file browser */
-	gint main_window_h;			/* main window height */
-	gint main_window_w;			/* main window width */
+	gint left_panel_left; /* 1 = left, 0 = right */
 	gint max_recent_files;	/* length of Open Recent list */
 	gint max_dir_history;	/* length of directory history */
 	gint backup_file; 			/* wheather to use a backup file */
@@ -202,8 +207,8 @@ typedef struct {
 	gint ext_commands_in_submenu;
 	gint ext_outputbox_in_submenu;
 	GList *reference_files; /* all reference files */
-	gint fref_ldoubleclick_action; /* left doubleclick in the function reference */
-	gint fref_info_type; /* type of info shown in a small function reference window */
+	gint bookmarks_default_store; /* 0= temporary by default, 1= permanent by default */
+	gint bookmarks_filename_mode; /* 0=FULLPATH, 1=DIR FROM BASE 2=BASENAME */
 	gint document_tabposition;
 	gint leftpanel_tabposition;
 	gchar *default_basedir;
@@ -211,43 +216,45 @@ typedef struct {
 #ifdef HAVE_LIBASPELL
 	gchar *spell_default_lang;
 #endif /* HAVE_LIBASPELL */
-	gint cont_highlight_full; 	/* if you want to highlight the full text or just the line */
-	gint lasttime_cust_menu; /* the last time the defaultfile was checked for new entries */
-	gint lasttime_highlighting; /* see above */
-	gint lasttime_filetypes; /* see above */
-	gint lasttime_encodings; /* see above */
 	/* not yet in use */
 	gchar *image_editor_cline; 	/* image editor commandline */
-	gint full_p;				/* use </p> */
-	gint full_li;				/* use </li> */
-	gint allow_css;				/* CSS allowed */
 	gint allow_dep;				/* allow <FONT>... */
 	gint format_by_context; 	/* use <strong> instead of <b>, <emphasis instead of <i> etc. (W3C reccomendation) */
 	gint xhtml;					/* write <br /> */
 	gint allow_ruby;			/* allow <ruby> */
-	gint allow_h4;				/* allow <Q>... */
-	gint allow_frames;			/* allow <FRAME> */
 	gint force_dtd;				/* write <!DOCTYPE...> */
 	gint dtd_url;				/* URL in DTD */
 	gint xml_start;				/* <?XML...> */
 	gint lowercase_tags;		/* use lowercase tags */
 	gint word_wrap;				/* use wordwrap */
 	gint autoindent;			/* autoindent code */
-	gint fontset;				/* load fontset (0 = load font) */
-	gint force_def_style;  /* force white editor background */
 	gint drop_at_drop_pos; 	/* drop at drop position instead of cursor position */
 	gint link_management; 	/* perform link management */
 	gint cont_highlight_update;	/* update the syntax highlighting continuous */
-	gchar *html_ver;
 	/* key conversion */
-	gint conv_ctrl_enter;		/* convert control-enter key press */
-	gchar *ctrl_enter_text;		/* inserted text */
-	gint conv_shift_enter;		/* convert shift-enter key press */
-	gchar *shift_enter_text;	/* inserted text */
-	gint conv_special_char;		/* convert ctrl-'<','>','&' */
 	gint open_in_running_bluefish; /* open commandline documents in already running session*/
+#ifdef HAVE_GNOME_VFS
+	gint server_zope_compat;        /* add 'document_src' to uri when reading remote files */
+#endif
 } Tproperties;
 
+/* the Tglobalsession contains all settings that can change 
+over every time you run Bluefish, so things that *need* to be
+saved after every run! */
+typedef struct {
+	GList *quickbar_items; /* items in the quickbar toolbar */	
+	gint main_window_h;			/* main window height */
+	gint main_window_w;			/* main window width */
+	gint two_pane_filebrowser_height; /* position of the pane separater on the two paned file browser */
+	gint fref_ldoubleclick_action; /* left doubleclick in the function reference */
+	gint fref_info_type; /* type of info shown in a small function reference window */
+	gint lasttime_cust_menu; /* the last time the defaultfile was checked for new entries */
+	gint lasttime_highlighting; /* see above */
+	gint lasttime_filetypes; /* see above */
+	gint lasttime_encodings; /* see above */
+	GList *recent_projects;
+} Tglobalsession;
+
 typedef struct {
 	GList *classlist;
 	GList *colorlist;
@@ -259,6 +266,11 @@ typedef struct {
 	GList *positionlist; /* is this used ?? */
 	GList *searchlist; /* used in snr2 */
 	GList *replacelist; /* used in snr2 */
+	GList *bmarks;
+	GList *recent_files;
+	GList *recent_dirs;
+	gchar *opendir;
+	gchar *savedir;
 } Tsessionvars;
 
 typedef struct {
@@ -269,17 +281,17 @@ typedef struct {
 	gchar *webdir;
 	gchar *template;
 	gpointer editor;
-	GList *recentfiles;
 	gint view_main_toolbar;
 	gint view_left_panel;
 	gint view_custom_menu;
 	gint view_html_toolbar;
 	gint word_wrap;
 	Tsessionvars *session;
+	GtkTreeStore *bookmarkstore; /* project bookmarks */
 } Tproject;
 
 typedef struct {
-	Tsessionvars *session;
+	Tsessionvars *session; /* points to the global session, or to the project session */
 	Tdocument *current_document; /* one object out of the documentlist, the current visible document */
 	GList *documentlist; /* document.c and others: all Tdocument objects */
 	Tdocument *last_activated_doc;
@@ -325,27 +337,33 @@ typedef struct {
 	gpointer filebrowser;
 	gpointer snr2;
 	gpointer fref;
-#ifdef BOOKMARKS	
 	gpointer bmark;
-#endif /* BOOKMARKS */	
+	GtkTreeStore *bookmarkstore; /* this is a link to project->bookmarkstore OR main_v->bookmarkstore
+											  and it is only here for convenience !!!! */
+	GHashTable *bmark_files;     /* no way, I have to have list of file iters. Other way I 
+	                                cannot properly load bmarks for closed files */
 } Tbfwin;
 
 typedef struct {
-	Tproperties props;
+	Tproperties props; /* preferences */
+	Tglobalsession globses; /* global session */
 	GList *filetypelist; /* highlighting.c: a list of all filetypes with their icons and highlighting sets */
 	GList *bfwinlist;
 	GList *recent_directories; /* a stringlist with the most recently used directories */
+	Tsessionvars *session; /* holds all session variables for non-project windows */
 	gpointer filebrowserconfig;
 	gpointer frefdata;
-#ifdef BOOKMARKS	
 	gpointer bmarkdata;
-#endif /* BOOKMARKS */	
+	GtkTreeStore *bookmarkstore; /* the global bookmarks from the global session */
 	gint num_untitled_documents;
+	GtkTooltips *tooltips;
+	guint16 lastkp_hardware_keycode; /* for the autoclosing, we need to know the last pressed key, in the key release callback, */
+	guint lastkp_keyval;             /* this is different if the modifier key is not pressed anymore during the key-release */
+	pcre *autoclosingtag_regc; /* the regular expression to check for a valid tag in tag autoclosing*/
 } Tmain;
 
 extern Tmain *main_v;
 
 /* public functions from bluefish.c */
 void bluefish_exit_request(void);
-void free_session(Tsessionvars *session);
 #endif /* __BLUEFISH_H_ */
