@@ -26,6 +26,6 @@
    pointer to a string whose uniqueness is quaranteed for the next
    five calls to this function.  (That is, the return strings are
    local static buffers (five of them) used in a round-robin style.  */
-extern char *cap (const char *);
+char *cap(const char *);
 
-#endif	/* __CAP_H_ */
+#endif							/* __CAP_H_ */
