@@ -17,26 +17,65 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/*#define DEBUG*/
+/* #define DEBUG */
 
+/* ******* NEW FILEBROWSER DESIGN ********
+I'm thinking about a new design for the filebrowser, the 
+code is too complicated right now. For now I'll just write some thoughts, so 
+after we have the next stable releae we can implement it.
+
+we'll drop the one-pane-view, we'll only have the files and directories separate
+
+the public API:
+--------------
+GtkWidget *fb2_init(Tbfwin *bfwin);
+void fb2_cleanup(Tbfwin *bfwin);
+void fb2_set_basedir(Tbfwin *bfwin, gchar *basedir);
+void fb2_focus_document(Tbfwin *bfwin, Tdocument *doc);
+--------------
+
+in the treemodel for the directories, we'll have two columns. A visible column (with the name), 
+and a column with the full path,  so if a click on an item is done, it is very easy to see which 
+full path corresponds to that item.
+
+to make 'focus document' very easy, each document can have a GtkTreeIter 
+pointing to the directory where the file located. If it does not have the treeiter, the 
+document has not been focused before (so it is not required to have a treeiter for 
+every document).
+
+the most difficult thing to code now is when we for example open a new file, we'll have to 
+find which directory item corresponds to that filename, and if it is not yet there, we'll 
+have to find where we should add it. Same for a document that has not been focused before.
+A possibility to do this is to have a hashtable with TreeIters to each directory that is already 
+in the tree. If you then want to open some directory, you check the full path in the hashtable, 
+if not exists you remove the last directory component, check again etc. until you have found a
+position in the tree where you can add things.
+
+USER INTERFACE NOTES:
+
+each directory should be expandable by default (so have a dummy item) unless we know there are 
+no subdirectories (and not the other way around like it is right now)
+
+*/
 #include <gtk/gtk.h>
-#include <sys/types.h> /* stat() getuid */
-#include <sys/stat.h> /* stat() */
-#include <unistd.h> /* stat() getuid */
-#include <string.h> /*strchr() */
-#include <stdio.h> /* rename() */	
-#include <stdlib.h> /* atoi() */
+#include <sys/types.h>	/* stat() getuid */
+#include <sys/stat.h>	/* stat() */
+#include <unistd.h>		/* stat() getuid */
+#include <string.h>		/*strchr() */
+#include <stdio.h>		/* rename() */	
+#include <stdlib.h>		/* atoi() */
 
 #include "bluefish.h"
 
-#include "document.h"
-#include "gtk_easy.h" /* *_dialog(), flush_queue() */
-#include "bf_lib.h"
-#include "stringlist.h" /* count_array() */
 #include "filebrowser.h"
-#include "gui.h" /* statusbar_message() */
-#include "image.h" /* image_insert_from_filename() */
-#include "menu.h" /* translation */
+#include "bf_lib.h"
+#include "document.h"
+#include "gtk_easy.h"	/* *_dialog(), flush_queue() */
+#include "gui.h"			/* statusbar_message() */
+#include "image.h" 		/* image_insert_from_filename() */
+#include "menu.h" 		/* translation */
+#include "project.h" 	/* project_open_from_file() */
+#include "stringlist.h" /* count_array() */
 
 /*#define DEBUG_SORTING
 #define DEBUG_FILTER
@@ -93,6 +132,7 @@ typedef struct {
 	GtkListStore *store2;
 	GtkWidget *dirmenu;
 	GtkWidget *showfulltree;
+	GtkWidget *focus_follow;
 	GList *dirmenu_entries;
 	gchar *last_opened_dir; /* SHOULD end on a '/' */
 	gboolean last_popup_on_dir;
@@ -116,6 +156,7 @@ typedef struct {
 
 /* some functions need to be referenced before they are declared*/
 static void row_expanded_lcb(GtkTreeView *tree,GtkTreeIter *iter,GtkTreePath *path,Tfilebrowser *filebrowser);
+static void populate_dir_history(Tfilebrowser *filebrowser,gboolean firsttime, gchar *activedir);
 
 #ifdef DEBUG
 void DEBUG_DUMP_TREE_PATH(GtkTreePath *path) {
@@ -132,7 +173,6 @@ void DEBUG_DUMP_TREE_PATH(GtkTreePath *path) {
  /**/
 #endif /* DEBUG */
 
-
 /*
  returns TRUE if there is no basedir, or if the file is indeed in the basedir, else returns FALSE
 */
@@ -144,17 +184,17 @@ static gboolean path_in_basedir(Tfilebrowser *filebrowser, const gchar *path) {
 }
 
 
-/* MODIFIES THE ITER POINTER TO BY PARENT !!!!!!!! */
-static gboolean get_iter_by_filename_from_parent(GtkTreeStore *store, GtkTreeIter *parent, gchar *filename) {
+/* MODIFIES THE ITER POINTER TO BY 'found' !!!!!!!! */
+static gboolean get_iter_by_filename_from_parent(GtkTreeStore *store, GtkTreeIter *parent, GtkTreeIter *found, gchar *filename) {
 	gboolean valid;
 	GtkTreeIter iter;
-	iter = *parent;
 	valid = gtk_tree_model_iter_children(GTK_TREE_MODEL(store),&iter,parent);
 	while (valid) {
 		gchar *name;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, FILENAME_COLUMN, &name, -1);
+		DEBUG_MSG("get_iter_by_filename_from_parent, comparing '%s' and '%s'\n",name,filename);
 		if (strcmp(name, filename)==0) {
-			*parent = iter;
+			*found = iter;
 			DEBUG_MSG("get_iter_by_filename_from_parent, found!!\n");
 			g_free(name);
 			return TRUE;
@@ -171,7 +211,11 @@ static gchar *return_filename_from_path(Tfilebrowser *filebrowser, GtkTreeModel
 	gchar *retval = NULL, *tmp;
 	gboolean showfulltree = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(filebrowser->showfulltree));
 	GtkTreePath *path = gtk_tree_path_copy(thispath);
-
+	DEBUG_MSG("return_filename_from_path, ");
+	DEBUG_DUMP_TREE_PATH(path);
+	if (!path) {
+		return NULL;
+	}
 	while (valid) {
 		gchar *name = NULL, *tmpstr;
 		GtkTreeIter iter;
@@ -291,8 +335,10 @@ static GList *return_dir_entries(Tfilebrowser *filebrowser,const gchar *dirname)
 	GnomeVFSResult result;
 	GnomeVFSDirectoryHandle *handle=NULL;
 	GnomeVFSFileInfo *gvfi=NULL;
-	DEBUG_MSG("return_dir_entries, started for %s\n",dirname);
-	result = gnome_vfs_directory_open(&handle,dirname,GNOME_VFS_FILE_INFO_DEFAULT|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	gchar *ondiskname = get_filename_on_disk_encoding(dirname);
+	DEBUG_MSG("return_dir_entries, started for dirname %s\n",dirname);
+	result = gnome_vfs_directory_open(&handle,ondiskname,GNOME_VFS_FILE_INFO_DEFAULT|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	g_free(ondiskname);
 	if (result != GNOME_VFS_OK) {
 		return NULL;
 	}
@@ -303,7 +349,9 @@ static GList *return_dir_entries(Tfilebrowser *filebrowser,const gchar *dirname)
 		if (strcmp(gvfi->name,".")!=0 && strcmp(gvfi->name,"..")!=0) {
 			entry = g_new(Tdir_entry,1);
 			entry->icon = NULL;
-			entry->name = g_strdup(gvfi->name);
+			DEBUG_MSG("return_dir_entries, found file '%s'\n",gvfi->name);
+			entry->name = get_utf8filename_from_on_disk_encoding(gvfi->name);
+			DEBUG_MSG("return_dir_entries, entry->name='%s'\n",entry->name);
 			if (gvfi->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 				entry->type = TYPE_DIR;
 			} else {
@@ -332,16 +380,14 @@ static GList *return_dir_entries(Tfilebrowser *filebrowser,gchar *dirname) {
 	GList *tmplist=NULL;
 	Tdir_entry *entry;
 	const gchar *name;
-	
-	dir = g_dir_open(dirname, 0, NULL);
+	gchar *ode_dirname = get_filename_on_disk_encoding(dirname);
+	dir = g_dir_open(ode_dirname, 0, NULL);
 	while ((name = g_dir_read_name(dir))) {
 		gchar *fullpath;
-
 		entry = g_new(Tdir_entry,1);
 		entry->icon = NULL;
-		entry->name = g_strdup(name);
-		
-		fullpath = g_strconcat(dirname, "/", name, NULL);
+		entry->name = get_utf8filename_from_on_disk_encoding(name);
+		fullpath = g_strconcat(ode_dirname, "/", entry->name, NULL);
 		stat(fullpath, &entry->stat);
 		g_free(fullpath);
 		if (S_ISDIR(entry->stat.st_mode)) {
@@ -360,6 +406,7 @@ static GList *return_dir_entries(Tfilebrowser *filebrowser,gchar *dirname) {
 		}
 	}
 	g_dir_close(dir);
+	g_free(ode_dirname);
 	return tmplist;
 }
 #endif /* HAVE_GNOME_VFS */
@@ -395,17 +442,20 @@ static GtkTreePath *return_path_from_filename(Tfilebrowser *filebrowser,gchar *t
 			while (!found && gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
 				gchar *found_root;
 				gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, FILENAME_COLUMN, &found_root, -1);
-				DEBUG_MSG("return_path_from_filename, searchinf for the root, comparing %s and %s\n",root,found_root);
+				DEBUG_MSG("return_path_from_filename, searching for the root, comparing %s and %s\n",root,found_root);
 				if (strcmp(found_root, root)==0) {
 					/* we found the root for this protocol/server */
 					parent = iter;
 					prevlen = strlen(root);
+					found = TRUE;
+					DEBUG_MSG("return_path_from_filename, we found '%s'!\n",found_root);
 				}
 				g_free(found_root);
 			}
-			if (strcmp(root, this_filename)==0) {
+			if (found && strcmp(root, this_filename)==0) {
 				/* we are looking for the root!! */
 				g_free(root);
+				DEBUG_MSG("return_path_from_filename, 'this_filename' and 'root' are equal! return the GtkTreePath for 'found_root'\n");
 				return gtk_tree_model_get_path(GTK_TREE_MODEL(store),&parent);
 			}
 			g_free(root);
@@ -458,6 +508,7 @@ static GtkTreePath *return_path_from_filename(Tfilebrowser *filebrowser,gchar *t
 		curlen = strlen(p);
 		tmpstr = g_strndup(&filepath[prevlen], (totlen - curlen - prevlen));
 		/* now search for this dirname */
+		DEBUG_MSG("return_path_from_filename, searching for '%s'\n",tmpstr);
 		valid = gtk_tree_model_iter_children(GTK_TREE_MODEL(store),&iter,&parent);
 		while (valid) {
 			gchar *found_name;
@@ -634,6 +685,7 @@ static void refresh_dir_by_path_and_filename(Tfilebrowser *filebrowser, GtkTreeP
 #endif
 	if (filebrowser->last_opened_dir) g_free(filebrowser->last_opened_dir);
 	filebrowser->last_opened_dir = ending_slash(filename);
+	populate_dir_history(filebrowser,FALSE, filebrowser->last_opened_dir);
 	direntrylist = return_dir_entries(filebrowser,filename);
 	{
 		GtkTreePath *path = gtk_tree_path_copy(thispath);		
@@ -739,7 +791,9 @@ static GtkTreePath *build_tree_from_path(Tfilebrowser *filebrowser, const gchar
 	/* first build path from root to here */
 	{
 		gchar *tmpstr, *p;
-		gint totlen, curlen, prevlen=1;
+		gint totlen, curlen, prevlen=0;
+		/* if we're working on an URL, prevlen should be 0, else it should be 1 */
+		if (filepath[0] == '/') prevlen = 1;
 		
 		if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(filebrowser->store), &iter)) {
 			if (!showfulltree && filebrowser->basedir) {
@@ -766,7 +820,11 @@ static GtkTreePath *build_tree_from_path(Tfilebrowser *filebrowser, const gchar
 		} else {
 			gchar *root = return_root_with_protocol(filepath);
 			if (root && root[0] != '/') {
-				iter = add_tree_item(NULL, filebrowser, root, TYPE_DIR, NULL);
+				DEBUG_MSG("finding iter for '%s' in the root of the tree\n",root);
+				if (!get_iter_by_filename_from_parent(filebrowser->store, NULL, &iter, root)) {
+					DEBUG_MSG("not found --> adding '%s'\n",root);
+					iter = add_tree_item(NULL, filebrowser, root, TYPE_DIR, NULL);
+				}
 				prevlen = strlen(root);
 			}
 			g_free(root);
@@ -781,7 +839,7 @@ static GtkTreePath *build_tree_from_path(Tfilebrowser *filebrowser, const gchar
 	#ifdef DEBUG_ADDING_TO_TREE
 			DEBUG_MSG("build_tree_from_path, tmpstr='%s'\n", tmpstr);
 	#endif
-			if (!get_iter_by_filename_from_parent(filebrowser->store, &iter, tmpstr)) {
+			if (!get_iter_by_filename_from_parent(filebrowser->store, &iter, &iter, tmpstr)) {
 				iter = add_tree_item(&iter, filebrowser, tmpstr, TYPE_DIR, NULL);
 			}
 			g_free(tmpstr);
@@ -799,13 +857,16 @@ static GtkTreePath *build_tree_from_path(Tfilebrowser *filebrowser, const gchar
 		if (filebrowser->last_opened_dir) g_free(filebrowser->last_opened_dir);
 		DEBUG_MSG("build_tree_from_path, freed last_opened_dir\n");
 		filebrowser->last_opened_dir = ending_slash(dirname);
+		populate_dir_history(filebrowser,FALSE, filebrowser->last_opened_dir);
 		DEBUG_MSG("build_tree_from_path, after ending slash\n");
-		
+
+		/* we need to add an empty item to each dir so it can be expanded */		
 		direntrylist = return_dir_entries(filebrowser,dirname);
 		DEBUG_MSG("build_tree_from_path, after return_dir_entries\n");
 		tmplist = g_list_first(direntrylist);
 		while (tmplist) {
 			Tdir_entry *entry = (Tdir_entry *)tmplist->data;
+			DEBUG_MSG("build_tree_from_path, entry=%s\n",entry->name);
 			if (entry->type == TYPE_DIR) {
 				GtkTreeIter tmp;
 				tmp = add_tree_item(&iter, filebrowser, entry->name, TYPE_DIR, NULL);
@@ -823,6 +884,10 @@ static GtkTreePath *build_tree_from_path(Tfilebrowser *filebrowser, const gchar
 		DEBUG_MSG("build_tree_from_path, freed dirname\n");
 		free_dir_entries(direntrylist);
 		DEBUG_MSG("build_tree_from_path, cleaned direntrylist\n");
+		/* if the toplevel is an empty directory, it needs a '.' item as well */
+		if (!gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store), &iter)) {
+			add_tree_item(&iter, filebrowser, ".", TYPE_FILE, NULL);
+		}
 	}
 	return gtk_tree_model_get_path(GTK_TREE_MODEL(filebrowser->store),&iter);
 }
@@ -832,7 +897,7 @@ if the GtkTreePath is already known, one should call refresh_dir_by_path_and_fil
 instead of this function
 */
 void filebrowser_refresh_dir(Tfilebrowser *filebrowser, gchar *dir) {
-	if (filebrowser->tree) {
+	if (filebrowser->tree && dir) {
 		/* get the path for this dir */
 		GtkTreePath *path = return_path_from_filename(filebrowser, dir);
 		DEBUG_DUMP_TREE_PATH(path);
@@ -898,9 +963,11 @@ static GtkTreePath *filebrowser_get_path_from_selection(GtkTreeModel *model, Gtk
 	return NULL;
 }
 
-/* is_directory is only meaningful if you have a two paned view and you want the directory name */
+/* is_directory is only meaningful if you have a two paned view and you want the directory name 
+*/
 static gchar *get_selected_filename(Tfilebrowser *filebrowser, gboolean is_directory) {
 	GtkTreePath *path;
+	gchar *retval=NULL;
 	if (filebrowser->store2 && !is_directory) {
 		path = filebrowser_get_path_from_selection(GTK_TREE_MODEL(filebrowser->store2),GTK_TREE_VIEW(filebrowser->tree2),NULL);
 		if (path) {
@@ -911,17 +978,17 @@ static gchar *get_selected_filename(Tfilebrowser *filebrowser, gboolean is_direc
 			tmp2 = g_strconcat(filebrowser->last_opened_dir, name, NULL);
 			gtk_tree_path_free(path);
 			g_free(name);
-			return tmp2;
+			retval = tmp2;
 		}
 	} else {
 		path = filebrowser_get_path_from_selection(GTK_TREE_MODEL(filebrowser->store),GTK_TREE_VIEW(filebrowser->tree),NULL);
 		if (path) {
 			gchar *filename = return_filename_from_path(filebrowser,GTK_TREE_MODEL(filebrowser->store), path);
 			gtk_tree_path_free(path);
-			return filename;
+			retval = filename;
 		}
 	}
-	return NULL;
+	return retval;
 }
 
 /**
@@ -948,101 +1015,24 @@ void filebrowser_open_dir(Tbfwin *bfwin,const gchar *dirarg) {
 		path = return_path_from_filename(filebrowser, dir);
 		DEBUG_MSG("filebrowser_open_dir, called for '%s', dir is '%s'\n", dirarg, dir);
 		DEBUG_DUMP_TREE_PATH(path);
-		if (path) {
-			DEBUG_MSG("jump_to_dir, it exists in tree, refreshing\n");
+		if (!path) {
+			DEBUG_MSG("filebrowser_open_dir, '%s' does NOT exist in the tree, building..\n", dir);
+			path = build_tree_from_path(filebrowser, dir);
+		} else {
+			DEBUG_MSG("filebrowser_open_dir, it exists in tree, refreshing\n");
 			refresh_dir_by_path_and_filename(filebrowser, path, dir);
-			DEBUG_MSG("jump_to_dir, now scroll to the path\n");
+		}
+		if (path) {
+			DEBUG_MSG("filebrowser_open_dir, now scroll to the path\n");
 			filebrowser_expand_to_root(filebrowser,path);
 			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(filebrowser->tree),path,0,TRUE,0.5,0.5);
+			gtk_tree_selection_select_path(gtk_tree_view_get_selection(GTK_TREE_VIEW(filebrowser->tree)),path);
 			gtk_tree_path_free(path);
-		} else {
-			DEBUG_MSG("jump_to_dir, it does NOT exist in the tree, building..\n");
-			path = build_tree_from_path(filebrowser, dir);
-/*			path = return_path_from_filename(GTK_TREE_STORE(filebrowser->store), dir);*/
-/*			gtk_tree_view_expand_row(GTK_TREE_VIEW(filebrowser->tree),path,FALSE);*/
-			if (path) {
-				filebrowser_expand_to_root(filebrowser,path);
-				gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(filebrowser->tree),path,0,TRUE,0.5,1.0);
-				gtk_tree_path_free(path);
-			}
 		}
 		g_free(dir);
 	}
 }
 
-static void dirmenu_activate_lcb(GtkMenuItem *widget, Tfilebrowser *filebrowser) {
-	const gchar *dir = gtk_label_get_text(GTK_LABEL(GTK_BIN(widget)->child));
-	if (dir) {
-		DEBUG_MSG("dirmenu_activate_lcb, dir=%s\n", dir);
-		filebrowser_open_dir(filebrowser->bfwin,dir);
-	}
-}
-
-static void populate_dir_history(Tfilebrowser *filebrowser,gboolean firsttime) {
-	GtkWidget *menu;
-	gchar *tmpchar, *tmpdir, *tmp;
-	GList *tmplist;
-	GList *new_history_list=NULL;
-	DEBUG_MSG("populate_dir_history, empty\n");
-
-	if (!firsttime) {
-		gtk_option_menu_remove_menu(GTK_OPTION_MENU(filebrowser->dirmenu));
-		free_stringlist(filebrowser->dirmenu_entries);
-		filebrowser->dirmenu_entries = NULL;
-	
-		menu = gtk_menu_new();
-		gtk_option_menu_set_menu(GTK_OPTION_MENU(filebrowser->dirmenu), menu);
-		gtk_widget_show(menu);
-	} else {
-		filebrowser->dirmenu_entries = NULL;
-		menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(filebrowser->dirmenu));
-		gtk_widget_show(menu);
-	}
-
-	tmp = g_get_current_dir();
-	tmpdir = g_strconcat(tmp, DIRSTR, NULL);
-	g_free(tmp);
-	
-	new_history_list = add_to_stringlist(new_history_list, tmpdir);
-	while ((tmpchar = strrchr(tmpdir, DIRCHR))) {
-		tmpchar++;
-		*tmpchar = '\0';
-		DEBUG_MSG("populate_dir_history, adding %s\n", tmpdir);
-		new_history_list = add_to_stringlist(new_history_list, tmpdir);
-		tmpchar--;
-		*tmpchar = '\0';
-	}
-	g_free(tmpdir);
-	
-	tmpdir = g_strconcat(g_get_home_dir(), DIRSTR, NULL);
-	new_history_list = add_to_stringlist(new_history_list, tmpdir);
-	g_free(tmpdir);
-
-	tmplist = g_list_last(main_v->recent_directories);
-	while (tmplist) {
-		new_history_list = add_to_stringlist(new_history_list, (gchar *)tmplist->data);
-		tmplist = g_list_previous(tmplist);
-	}
-
-	/* make the actual option menu */
-	tmplist = g_list_first(new_history_list);
-	while (tmplist) {
-		GtkWidget *menuitem;
-		gchar *dirname = ending_slash((gchar *)tmplist->data);
-		DEBUG_MSG("populate_dir_history, adding %s to menu\n", dirname);
-		menuitem = gtk_menu_item_new_with_label(dirname);
-		g_signal_connect(G_OBJECT(menuitem),"activate",G_CALLBACK(dirmenu_activate_lcb),filebrowser);
-		g_free(dirname);
-		gtk_menu_append(GTK_MENU(menu), menuitem);
-		gtk_widget_show(menuitem);
-		g_free(tmplist->data);
-		tmplist = g_list_next(tmplist);
-	} 
-	g_list_free(new_history_list);
-
-	gtk_option_menu_set_history(GTK_OPTION_MENU(filebrowser->dirmenu), 0);
-}
-
 typedef struct {
 	GtkWidget *win;
 	GtkWidget *entry;
@@ -1075,13 +1065,16 @@ static void create_file_or_dir_ok_clicked_lcb(GtkWidget *widget, Tcfod *ws) {
 				if (ws->is_file) {
 					doc_new_with_new_file(ws->filebrowser->bfwin,newname);
 				} else {
+					gchar *ondiskencoding;
+					ondiskencoding = get_filename_on_disk_encoding(newname);
 #ifdef HAVE_GNOME_VFS
-					gnome_vfs_make_directory(newname,0755);
+					gnome_vfs_make_directory(ondiskencoding,0755);
 #else
-					if(mkdir(newname, 0755)== -1) {
+					if(mkdir(ondiskencoding, 0755)== -1) {
 /*						error_dialog(_("Error creating directory"),strerror(errno));*/
 					}
 #endif
+					g_free(ondiskencoding);
 				}
 			}
 			g_free(newname);
@@ -1101,6 +1094,8 @@ static void create_file_or_dir_win(Tfilebrowser *filebrowser, gint is_file) {
 	GtkTreeIter iter;
 	gchar *basedir = NULL;
 	selection = filebrowser_get_path_from_selection(GTK_TREE_MODEL(filebrowser->store),GTK_TREE_VIEW(filebrowser->tree),&iter);
+	DEBUG_MSG("create_file_or_dir_win, selection ");
+	DEBUG_DUMP_TREE_PATH(selection);
 	if (filebrowser->store2 && !selection) {
 		/* there is no selection, we'll use the current last used dir */
 		if (filebrowser->last_opened_dir) {
@@ -1111,6 +1106,7 @@ static void create_file_or_dir_win(Tfilebrowser *filebrowser, gint is_file) {
 		gchar *tmp;
 		if (filebrowser->store2 || gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store), &iter)) {
 			/* the selection does point to a directory, not a file */
+			DEBUG_MSG("create_file_or_dir_win, is a directory: filebrowser->store2=%p, tree_model_iter_has_child=%d\n",filebrowser->store2 ,gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store), &iter));
 		} else {
 			/* the selection points to a file, get the parent */
 			GtkTreeIter parent;
@@ -1174,6 +1170,7 @@ static void row_expanded_lcb(GtkTreeView *tree,GtkTreeIter *iter,GtkTreePath *pa
 	DEBUG_MSG("row_expanded_lcb, finished\n");
 }
 
+/* needs the UTF8 encoded filename */
 static void handle_activate_on_file(Tfilebrowser *filebrowser, gchar *filename) {
 	Tfiletype *ft = get_filetype_by_filename_and_content(filename, NULL);
 	DEBUG_MSG("handle_activate_on_file, file %s has type %p\n",filename, ft);
@@ -1183,6 +1180,8 @@ static void handle_activate_on_file(Tfilebrowser *filebrowser, gchar *filename)
 		gchar *relfilename = create_relative_link_to(filebrowser->bfwin->current_document->filename, filename);
 		image_insert_from_filename(filebrowser->bfwin,relfilename);
 		g_free(relfilename);
+	} else if (strcmp(ft->type, "bfproject") == 0) {
+		project_open_from_file(filebrowser->bfwin, filename);
 	} else {
 		DEBUG_MSG("handle_activate_on_file, file %s is not-editable, do something special now?\n",filename);
 	}
@@ -1243,20 +1242,25 @@ static void filebrowser_rpopup_rename(Tfilebrowser *filebrowser) {
 			/* If an error occurs, doc_save takes care of notifying the user.
 			 * Currently, nothing is done here.
 			 */	
-			doc_save(tmpdoc, 1, 1);
+			doc_save(tmpdoc, TRUE, TRUE, FALSE);
 		} else {
 			/* Promt user, "File/Move To"-style. */
-			newfilename = ask_new_filename(filebrowser->bfwin,oldfilename, 1);
+			newfilename = ask_new_filename(filebrowser->bfwin, oldfilename, oldfilename, TRUE);
 			if (newfilename) {
-	#ifdef HAVE_GNOME_VFS
-				if (gnome_vfs_move(oldfilename,newfilename,TRUE) != GNOME_VFS_OK) {
+				gchar *old_OnDiEn, *new_OnDiEn; /* OnDiskEncoding */
+				old_OnDiEn = get_filename_on_disk_encoding(oldfilename);
+				new_OnDiEn = get_filename_on_disk_encoding(newfilename);
+#ifdef HAVE_GNOME_VFS
+				if (gnome_vfs_move(old_OnDiEn,new_OnDiEn,TRUE) != GNOME_VFS_OK) {
 					errmessage = g_strconcat(_("Could not rename\n"), oldfilename, NULL);
 				}
-	#else
-				if(rename(oldfilename, newfilename) != 0) {
+#else
+				if(rename(old_OnDiEn, new_OnDiEn) != 0) {
 					errmessage = g_strconcat(_("Could not rename\n"), oldfilename, NULL);
 				}
-	#endif
+#endif
+				g_free(old_OnDiEn);
+				g_free(new_OnDiEn);
 			}
 		} /* if(oldfilename is open) */
 	
@@ -1295,24 +1299,24 @@ static void filebrowser_rpopup_delete(Tfilebrowser *filebrowser) {
 		gchar *buttons[] = {GTK_STOCK_CANCEL, GTK_STOCK_DELETE, NULL};
 		gchar *label;
 		gint retval;
-		label = g_strdup_printf(_("Are you sure you want to\ndelete \"%s\" ?"), filename);
-		retval = multi_query_dialog(filebrowser->bfwin->main_window,label, _("The file will be permanently deleted."), 0, 0, buttons);
+		label = g_strdup_printf(_("Are you sure you want to delete \"%s\" ?"), filename);
+		retval = multi_query_dialog(filebrowser->bfwin->main_window,label, _("If you delete this file, it will be permanently lost."), 0, 0, buttons);
 		g_free(label);
 		if (retval == 1) {
 			gchar *errmessage = NULL;
-			gchar *tmp, *dir;
+			gchar *tmp, *dir, *ondiskenc;
 			DEBUG_MSG("file_list_rpopup_file_delete %s\n", filename);
-
+			ondiskenc = get_filename_on_disk_encoding(filename);
 #ifdef HAVE_GNOME_VFS
-			if ( gnome_vfs_unlink(filename) != GNOME_VFS_OK) {
+			if (gnome_vfs_unlink(ondiskenc) != GNOME_VFS_OK) {
 				errmessage = g_strconcat(_("Could not delete \n"), filename, NULL);
 			}
 #else
-			if( unlink(filename) != 0) {
+			if( unlink(ondiskenc) != 0) {
 				errmessage = g_strconcat(_("Could not delete \n"), filename, NULL);
 			}
 #endif
-
+			g_free(ondiskenc);
 			if (errmessage) {
 				error_dialog(filebrowser->bfwin->main_window,errmessage, NULL);
 				g_free(errmessage);
@@ -1334,12 +1338,12 @@ static void filebrowser_rpopup_delete(Tfilebrowser *filebrowser) {
 }
 
 static void filebrowser_rpopup_refresh(Tfilebrowser *filebrowser) {
-	GtkTreePath *path;
+	GtkTreePath *path1;
 	GtkTreeIter iter;
-	path = filebrowser_get_path_from_selection(GTK_TREE_MODEL(filebrowser->store),GTK_TREE_VIEW(filebrowser->tree),&iter);
-	if (path) {
+	path1 = filebrowser_get_path_from_selection(GTK_TREE_MODEL(filebrowser->store),GTK_TREE_VIEW(filebrowser->tree),&iter);
+	if (path1) {
 		gchar *tmp, *dir;
-		GtkTreePath *path;
+		GtkTreePath *path2;
 		if (filebrowser->store2 || gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store), &iter)) {
 			DEBUG_MSG("create_file_or_dir_win, a dir is selected\n");
 		} else {
@@ -1348,18 +1352,17 @@ static void filebrowser_rpopup_refresh(Tfilebrowser *filebrowser) {
 			gtk_tree_model_iter_parent(GTK_TREE_MODEL(filebrowser->store), &parent, &iter);
 			iter = parent;
 		}
-		path = gtk_tree_model_get_path(GTK_TREE_MODEL(filebrowser->store),&iter);
-		tmp = return_filename_from_path(filebrowser,GTK_TREE_MODEL(filebrowser->store),path);
-		DEBUG_MSG("filebrowser_rpopup_refresh_lcb, return_filename_from_path returned %s\n", tmp);
+		path2 = gtk_tree_model_get_path(GTK_TREE_MODEL(filebrowser->store),&iter);
+		tmp = return_filename_from_path(filebrowser,GTK_TREE_MODEL(filebrowser->store),path2);
+		DEBUG_MSG("filebrowser_rpopup_refresh_lcb, return_filename_from_path returned %s for path %p\n", tmp, path2);
 		dir = ending_slash(tmp);
 		DEBUG_MSG("filebrowser_rpopup_refresh_lcb, ending_slash returned %s\n", dir);
-		refresh_dir_by_path_and_filename(filebrowser, path, dir);
-		/* filebrowser_refresh_dir(filebrowser,dir); */
+		refresh_dir_by_path_and_filename(filebrowser, path2, dir);
 		g_free(tmp);
 		g_free(dir);
-		gtk_tree_path_free(path);
+		gtk_tree_path_free(path2);
+		gtk_tree_path_free(path1);
 	}
-	gtk_tree_path_free(path);
 }
 
 static void filebrowser_rpopup_action_lcb(Tfilebrowser *filebrowser,guint callback_action, GtkWidget *widget) {
@@ -1367,7 +1370,7 @@ static void filebrowser_rpopup_action_lcb(Tfilebrowser *filebrowser,guint callba
 	case 1: {
 		gchar *filename = get_selected_filename(filebrowser, FALSE);
 		if (filename) {
-		        GdkRectangle r;
+			GdkRectangle r;
 			gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(filebrowser->tree),&r);
 			handle_activate_on_file(filebrowser, filename);
 			g_free(filename);
@@ -1436,10 +1439,20 @@ static void filebrowser_rpopup_filter_toggled_lcb(GtkWidget *widget, Tfilebrowse
 	}
 }
 
+static void filebrowser_rpopup_shf_toggled_lcb(GtkWidget *widget, Tfilebrowser *filebrowser) {
+	main_v->props.filebrowser_show_hidden_files = GTK_CHECK_MENU_ITEM(widget)->active;
+	filebrowser_rpopup_refresh(filebrowser);
+}
+
+static void filebrowser_rpopup_sbf_toggled_lcb(GtkWidget *widget, Tfilebrowser *filebrowser) {
+	main_v->props.filebrowser_show_backup_files = GTK_CHECK_MENU_ITEM(widget)->active;
+	filebrowser_rpopup_refresh(filebrowser);
+}
+
 static GtkItemFactoryEntry filebrowser_dirmenu_entries[] = {
 #ifdef EXTERNAL_GREP
 #ifdef EXTERNAL_FIND	
-	{ N_("/Open _Advanced"),			NULL,	filebrowser_rpopup_action_lcb,		7,	"<Item>" },
+	{ N_("/Open _Advanced..."),			NULL,	filebrowser_rpopup_action_lcb,		7,	"<Item>" },
 #endif
 #endif
 	
@@ -1450,6 +1463,7 @@ static GtkItemFactoryEntry filebrowser_dirmenu_entries[] = {
 static GtkItemFactoryEntry filebrowser_bothmenu_entries[] = {
 	{ N_("/New _File"),			NULL,	filebrowser_rpopup_action_lcb,	4,	"<Item>" },
 	{ N_("/_New Directory"),	NULL,	filebrowser_rpopup_action_lcb,		5,	"<Item>" },
+	{ "/sep",	NULL,	NULL,		0,	"<Separator>" }
 };
 
 static GtkItemFactoryEntry filebrowser_filemenu_entries[] = {
@@ -1476,7 +1490,17 @@ static GtkWidget *filebrowser_rpopup_create_menu(Tfilebrowser *filebrowser, gboo
 	}
 	gtk_item_factory_create_items(menumaker, sizeof(filebrowser_bothmenu_entries)/sizeof(GtkItemFactoryEntry), filebrowser_bothmenu_entries, filebrowser);
 	menu = gtk_item_factory_get_widget(menumaker, "<Filebrowser>");
-		
+	/* set toggle options */
+	menu_item = gtk_check_menu_item_new_with_label(_("Show hidden files"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item), main_v->props.filebrowser_show_hidden_files);
+	g_signal_connect(GTK_OBJECT(menu_item), "toggled", G_CALLBACK(filebrowser_rpopup_shf_toggled_lcb), filebrowser);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+	
+	menu_item = gtk_check_menu_item_new_with_label(_("Show backup files"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item), main_v->props.filebrowser_show_backup_files);
+	g_signal_connect(GTK_OBJECT(menu_item), "toggled", G_CALLBACK(filebrowser_rpopup_sbf_toggled_lcb), filebrowser);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+	
 	/* Add filter submenu */
 	menu_item = gtk_menu_item_new_with_label(_("Filter"));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
@@ -1518,6 +1542,7 @@ static gboolean filebrowser_button_press_lcb(GtkWidget *widget, GdkEventButton *
 			gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(filebrowser->tree), event->x, event->y, &path, NULL, NULL, NULL);
 			if (path) {
 				gtk_tree_model_get_iter(GTK_TREE_MODEL(filebrowser->store),&iter,path);
+				gtk_tree_path_free(path);
 				if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store),&iter)) {
 					/* it is a directory */
 					menu = filebrowser_rpopup_create_menu(filebrowser, TRUE);
@@ -1564,21 +1589,19 @@ static gboolean filebrowser_button_press_lcb(GtkWidget *widget, GdkEventButton *
 						g_free(tmp);
 						g_free(dir);
 					}
-	/*				gtk_tree_path_free(path);
-					return FALSE;*/
 				} else if (event->type == GDK_2BUTTON_PRESS) {
-				        GdkRectangle r;
+					GdkRectangle r;
 					gchar *filename = return_filename_from_path(filebrowser,GTK_TREE_MODEL(filebrowser->store),path);
 					DEBUG_MSG("filebrowser_button_press_lcb, doubleclick!! open the file %s\n", filename);
 					gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(filebrowser->tree),&r);
 					handle_activate_on_file(filebrowser,filename);
 					g_free(filename);
 					gtk_tree_view_scroll_to_point(GTK_TREE_VIEW(filebrowser->tree),-1,r.y);
+					gtk_tree_path_free (path);
 					return TRUE;
 				}
 			}
 			gtk_tree_path_free(path);
-/*			return TRUE;*/
 		}
 	}
 	return FALSE; /* pass the event on */
@@ -1763,10 +1786,94 @@ static void filebrowser_two_pane_notify_position_lcb(GObject *object,GParamSpec
 	gint position;
 	g_object_get(object, pspec->name, &position, NULL);
 	if (main_v->props.restore_dimensions) {
-		main_v->props.two_pane_filebrowser_height = position;
+		main_v->globses.two_pane_filebrowser_height = position;
 	}
 }
 
+static void dirmenu_activate_lcb(GtkMenuItem *widget, Tfilebrowser *filebrowser) {
+	const gchar *dir = gtk_label_get_text(GTK_LABEL(GTK_BIN(widget)->child));
+	if (dir) {
+		DEBUG_MSG("dirmenu_activate_lcb, dir=%s\n", dir);
+		filebrowser_open_dir(filebrowser->bfwin,dir);
+	}
+}
+
+static void populate_dir_history(Tfilebrowser *filebrowser,gboolean firsttime, gchar *activedir) {
+	GtkWidget *menu;
+	gchar *tmpchar, *tmpdir;
+	GList *tmplist;
+	GList *new_history_list=NULL;
+/*	if (!firsttime) { */
+		DEBUG_MSG("populate_dir_history,remove menu, free stringlist etc.\n ");
+		gtk_option_menu_remove_menu(GTK_OPTION_MENU(filebrowser->dirmenu));
+		free_stringlist(filebrowser->dirmenu_entries);
+		filebrowser->dirmenu_entries = NULL;
+	
+		menu = gtk_menu_new();
+		gtk_option_menu_set_menu(GTK_OPTION_MENU(filebrowser->dirmenu), menu);
+		gtk_widget_show(menu);
+/*	} else {
+		DEBUG_MSG("populate_dir_history, first time, get the menu\n");
+		filebrowser->dirmenu_entries = NULL;
+		menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(filebrowser->dirmenu));
+		gtk_widget_show(menu);
+	}*/
+
+	if (activedir) {
+		gint rootlen=0;
+		gchar *root;
+		DEBUG_MSG("populate_dir_history, activedir=%s\n",activedir);
+		tmpdir = ending_slash(activedir);
+		
+		new_history_list = add_to_stringlist(new_history_list, tmpdir);
+		root = return_root_with_protocol(tmpdir);
+		if (root) {
+			rootlen = strlen(root)-1;
+			g_free(root);
+		}
+		while ((tmpchar = strrchr(tmpdir, DIRCHR)) && (tmpchar - tmpdir) >= rootlen) {
+			tmpchar++;
+			*tmpchar = '\0';
+			DEBUG_MSG("populate_dir_history, adding %s\n", tmpdir);
+			new_history_list = add_to_stringlist(new_history_list, tmpdir);
+			tmpchar--;
+			*tmpchar = '\0';
+		}
+		g_free(tmpdir);
+	}
+	tmpdir = g_strconcat(g_get_home_dir(), DIRSTR, NULL);
+	new_history_list = add_to_stringlist(new_history_list, tmpdir);
+	g_free(tmpdir);
+
+	tmplist = g_list_last(main_v->recent_directories);
+	while (tmplist) {
+		new_history_list = add_to_stringlist(new_history_list, (gchar *)tmplist->data);
+		tmplist = g_list_previous(tmplist);
+	}
+
+	/* make the actual option menu */
+	tmplist = g_list_first(new_history_list);
+	while (tmplist) {
+		GtkWidget *menuitem;
+		gchar *dirname = ending_slash((gchar *)tmplist->data);
+		DEBUG_MSG("populate_dir_history, adding %s to menu\n", dirname);
+		menuitem = gtk_menu_item_new_with_label(dirname);
+		g_signal_connect(G_OBJECT(menuitem),"activate",G_CALLBACK(dirmenu_activate_lcb),filebrowser);
+		g_free(dirname);
+		gtk_menu_append(GTK_MENU(menu), menuitem);
+		gtk_widget_show(menuitem);
+		g_free(tmplist->data);
+		tmplist = g_list_next(tmplist);
+	} 
+	g_list_free(new_history_list);
+
+	gtk_option_menu_set_history(GTK_OPTION_MENU(filebrowser->dirmenu), 0);
+}
+
+static void focus_follow_toggled_lcb(GtkToggleButton *togglebutton, Tbfwin *bfwin) {
+	main_v->props.filebrowser_focus_follow = togglebutton->active;
+}
+
 /**
  * filebrowsel_init:
  * @bfwin: #Tbfwin*
@@ -1797,10 +1904,14 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		filebrowser->curfilter = find_filter_by_name(main_v->props.last_filefilter);
 	}
 	
+	filebrowser->dirmenu = gtk_option_menu_new();
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(filebrowser->dirmenu), gtk_menu_new());
+/*	populate_dir_history(filebrowser,TRUE, NULL);*/
+	
 	filebrowser->store = gtk_tree_store_new (N_COLUMNS,GDK_TYPE_PIXBUF,G_TYPE_STRING);
 	filebrowser->tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(filebrowser->store));
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(filebrowser->tree), FALSE);	
-
+	/*gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(filebrowser->tree), TRUE);*/
 	{
 		GtkCellRenderer *renderer;
 		GtkTreeViewColumn *column;
@@ -1840,7 +1951,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		filebrowser->store2 = gtk_list_store_new (N_COLUMNS,GDK_TYPE_PIXBUF,G_TYPE_STRING);
 		filebrowser->tree2 = gtk_tree_view_new_with_model(GTK_TREE_MODEL(filebrowser->store2));
 		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(filebrowser->tree2), FALSE);
-
+		/*gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(filebrowser->tree2), TRUE);*/
 		renderer = gtk_cell_renderer_pixbuf_new();
 		column = gtk_tree_view_column_new ();
 		gtk_tree_view_column_pack_start (column, renderer, FALSE);
@@ -1864,6 +1975,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		gchar *buildfrom = NULL;
 		
 		filebrowser->showfulltree = checkbut_with_value(_("Show full tree"), FALSE);
+		filebrowser->focus_follow = checkbut_with_value(_("Follow focus"), main_v->props.filebrowser_focus_follow);
 		if (bfwin->project && bfwin->project->basedir && strlen(bfwin->project->basedir)>2) {
 			buildfrom = bfwin->project->basedir;
 		} else {
@@ -1883,6 +1995,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		path = build_tree_from_path(filebrowser, buildfrom);
 		if (filebrowser->last_opened_dir) g_free(filebrowser->last_opened_dir);
 		filebrowser->last_opened_dir = ending_slash(buildfrom);
+		populate_dir_history(filebrowser,FALSE, filebrowser->last_opened_dir);
 		if (path) {
 			filebrowser_expand_to_root(filebrowser,path);
 			gtk_tree_path_free(path);
@@ -1899,10 +2012,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		GtkWidget *vbox, *scrolwin;
 	/*	GtkAdjustment* adj;*/
 		vbox = gtk_vbox_new(FALSE, 0);
-		
-		filebrowser->dirmenu = gtk_option_menu_new();
-		gtk_option_menu_set_menu(GTK_OPTION_MENU(filebrowser->dirmenu), gtk_menu_new());
-		populate_dir_history(filebrowser,TRUE);
+
 		gtk_box_pack_start(GTK_BOX(vbox),filebrowser->dirmenu, FALSE, FALSE, 0);
 		
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
@@ -1918,7 +2028,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 			gtk_paned_add2(GTK_PANED(vpaned), scrolwin2);
 			gtk_widget_set_size_request(vpaned, main_v->props.left_panel_width, -1);
 			gtk_box_pack_start(GTK_BOX(vbox), vpaned, TRUE, TRUE, 0);
-			gtk_paned_set_position(GTK_PANED(vpaned), main_v->props.two_pane_filebrowser_height);
+			gtk_paned_set_position(GTK_PANED(vpaned), main_v->globses.two_pane_filebrowser_height);
 			g_signal_connect(G_OBJECT(vpaned),"notify::position",G_CALLBACK(filebrowser_two_pane_notify_position_lcb), NULL);
 		} else {
 			gtk_widget_set_size_request(scrolwin, main_v->props.left_panel_width, -1);
@@ -1930,7 +2040,9 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		gtk_adjustment_set_value(GTK_ADJUSTMENT(adj), GTK_ADJUSTMENT(adj)->lower);*/
 		
 		gtk_box_pack_start(GTK_BOX(vbox), filebrowser->showfulltree, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(vbox), filebrowser->focus_follow, FALSE, FALSE, 0);
 		g_signal_connect(G_OBJECT(filebrowser->showfulltree), "toggled", G_CALLBACK(showfulltree_toggled_lcb), bfwin);
+		g_signal_connect(G_OBJECT(filebrowser->focus_follow), "toggled", G_CALLBACK(focus_follow_toggled_lcb), bfwin);
 		if (filebrowser->basedir == NULL) {
 			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(filebrowser->showfulltree), TRUE);
 			gtk_widget_set_sensitive(GTK_WIDGET(filebrowser->showfulltree), FALSE);
@@ -1967,7 +2079,10 @@ void filebrowser_cleanup(Tbfwin *bfwin) {
 			gtk_list_store_clear(GTK_LIST_STORE(FILEBROWSER(bfwin->filebrowser)->store2));
 			g_object_unref(G_OBJECT(FILEBROWSER(bfwin->filebrowser)->store2));
 		}
-		if (FILEBROWSER(bfwin->filebrowser)->basedir) g_free(FILEBROWSER(bfwin->filebrowser)->basedir);
+		if (FILEBROWSER(bfwin->filebrowser)->basedir)
+			g_free(FILEBROWSER(bfwin->filebrowser)->basedir);
+		if (FILEBROWSER(bfwin->filebrowser)->last_opened_dir) 
+			g_free(FILEBROWSER(bfwin->filebrowser)->last_opened_dir);
 		
 		DEBUG_MSG("filebrowser_cleanup, free filebrowser\n");
 		g_free(bfwin->filebrowser);
