@@ -1,6 +1,6 @@
 /* Bluefish HTML Editor
  *
- * Copyright (C) 1999-2002 Olivier Sessink
+ * Copyright (C) 1999-2004 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "bf_lib.h"
 #include "pixmap.h"
 #include "gui.h" /* statusbar_message() */
+#include "char_table.h" /* xml_escape() */
 
 #ifdef WIN32
 #define DIRSTR "\\"
@@ -135,10 +136,9 @@ void window_close_by_data_cb(GtkWidget * widget, gpointer data) {
  */
 void setup_toggle_item(GtkItemFactory * ifactory, gchar * path, gint state) {
 	GtkWidget *toggle;
-
 	toggle = gtk_item_factory_get_widget(ifactory, path);
 	if (!toggle) {
-		DEBUG_MSG("warning, this menuitem (%s) does NOT exist!", path);
+		g_print("warning, menuitem \"%s\" does NOT exist! check the translation!\n", path);
 		return;
 	}
 	DEBUG_MSG("setting path %s to %d\n",path,state);
@@ -823,6 +823,15 @@ void bf_mnemonic_label_tad_with_alignment(const gchar *labeltext, GtkWidget *m_w
 	}
 }
 
+/**
+ * bf_label_with_markup:
+ * @labeltext: #const gchar* labeltext string
+ */
+GtkWidget *bf_label_with_markup(const gchar *labeltext) {
+	GtkWidget *label = gtk_label_new(NULL);
+	gtk_label_set_markup(GTK_LABEL(label), labeltext);
+	return label;
+}
 /**
  * bf_label_tad_with_markup:
  * @labeltext: #const gchar* label string
@@ -843,9 +852,7 @@ void bf_label_tad_with_markup(const gchar *labeltext, gfloat xalign, gfloat yali
 								GtkWidget *table, guint left_attach, guint right_attach, 
 								guint top_attach, guint bottom_attach) {
 	GtkWidget *label;
-	
-	label = gtk_label_new(NULL);
-	gtk_label_set_markup(GTK_LABEL(label), labeltext);
+	label = bf_label_with_markup(labeltext);
 	gtk_misc_set_alignment(GTK_MISC(label), xalign, yalign);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, left_attach, right_attach, top_attach, bottom_attach);	
 }
@@ -943,11 +950,17 @@ static void hig_dialog_backend (GtkDialog *dialog, gchar *primary, gchar *second
 		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
 		gtk_misc_set_alignment (GTK_MISC (image), 0, 0);
 	}
-
-	if(secondary) /* Creates label-content. */
-		message = g_strconcat(spanstart, primary, spanend, secondary, msgend, NULL);
-	else
-		message = g_strconcat(spanstart, primary, spanend, NULL);
+	
+	if(secondary) { /* Creates label-content. */
+		gchar *str1, *str2;
+		str1 = xml_escape(primary);
+		str2 = xml_escape(secondary);
+		message = g_strconcat(spanstart, str1, spanend, str2, msgend, NULL);
+	} else {
+		gchar *str1;
+		str1 = xml_escape(primary);
+		message = g_strconcat(spanstart, str1, spanend, NULL);
+	}
 			
 	label = gtk_label_new (message);
 	g_free(message);
@@ -1225,10 +1238,42 @@ typedef struct {
 } Tfilebut;
 
 static void file_but_clicked_lcb(GtkWidget * widget, Tfilebut *fb) {
-	gchar *tmpstring, *tmp2string, *setfile;
+	gchar *tmpstring=NULL, *tmp2string, *setfile;
 	DEBUG_MSG("file_but_clicked_lcb, started, which_entry=%p\n",fb->entry);
 	setfile = gtk_editable_get_chars(GTK_EDITABLE(GTK_ENTRY(fb->entry)),0,-1);
+	/* if setfile is empty we should probably use the current document basedir ? right? */
+	if (!setfile || strlen(setfile)==0) {
+		if (fb->bfwin->current_document->filename && strlen(fb->bfwin->current_document->filename)) {
+			if (setfile) g_free(setfile);
+			setfile = path_get_dirname_with_ending_slash(fb->bfwin->current_document->filename);
+		}
+	} else if (setfile && strchr(setfile, '/') == NULL && fb->bfwin->current_document->filename) {
+		/* if setfile is a relative name, we should try to make it a full path. relative names have 
+		no slashes in the name */
+		gchar *basedir, *oldsetfile;
+		oldsetfile = setfile;
+		basedir = path_get_dirname_with_ending_slash(fb->bfwin->current_document->filename);
+		setfile = create_full_path(oldsetfile, basedir);
+		g_free(oldsetfile);
+		g_free(basedir);
+	}
+	
+#ifdef HAVE_ATLEAST_GTK_2_4
+	{
+		GtkWidget *dialog;
+		dialog = file_chooser_dialog(NULL, _("Select File"), GTK_FILE_CHOOSER_ACTION_OPEN, setfile, FALSE, FALSE, NULL);
+		gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(gtk_widget_get_toplevel(fb->entry)));
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),setfile);
+		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+			tmp2string = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
+			tmpstring = create_full_path(tmp2string, NULL);
+			g_free(tmp2string);
+		}
+		gtk_widget_destroy(dialog);
+	}
+#else
 	tmpstring = return_file(setfile);
+#endif
 	g_free(setfile);
 	DEBUG_MSG("file_but_clicked_lcb, return_file returned %s\n",tmpstring);
 	if (tmpstring) {
@@ -1236,7 +1281,7 @@ static void file_but_clicked_lcb(GtkWidget * widget, Tfilebut *fb) {
 			if (fb->bfwin->current_document->filename != NULL) {
 				tmp2string = create_relative_link_to(fb->bfwin->current_document->filename, tmpstring);
 			} else {
-				tmp2string = g_path_get_basename(tmpstring);
+				tmp2string = g_strdup(tmpstring);
 			}
 			g_free(tmpstring);
 			tmpstring = tmp2string;
@@ -1283,6 +1328,110 @@ GtkWidget *file_but_new(GtkWidget * which_entry, gint full_pathname, Tbfwin *bfw
 /************************************************************************/
 /************    FILE SELECTION FUNCTIONS  ******************************/
 /************************************************************************/
+
+#ifdef HAVE_ATLEAST_GTK_2_4
+
+static void viewlocal_toggled_lcb(GtkToggleButton *togglebutton,GtkWidget *dialog) {
+	g_object_set(G_OBJECT(dialog), "show-hidden", togglebutton->active, NULL);
+}
+
+static void file_chooser_set_current_dir(GtkWidget *dialog, gchar *dir) {
+	if (dir) {
+		if (dir[0] == '/') gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog),dir);
+		else gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog),dir);
+	}
+}
+
+GtkWidget * file_chooser_dialog(Tbfwin *bfwin, gchar *title, GtkFileChooserAction action, 
+											gchar *set, gboolean localonly, gboolean multiple, const gchar *filter) {
+	GtkWidget *vbox, *hbox, *dialog, *viewlocal;
+	dialog = gtk_file_chooser_dialog_new_with_backend(title,bfwin ? GTK_WINDOW(bfwin->main_window) : NULL,
+			action,"gnome-vfs",
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			(action == GTK_FILE_CHOOSER_ACTION_SAVE) ? GTK_STOCK_SAVE : GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+	if (set && strlen(set)) {
+		DEBUG_MSG("file_chooser_dialog, set=%s,localonly=%d\n",set,localonly);
+		if (localonly || strchr(set,':')==NULL) {
+			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),set);
+		} else {
+			gtk_file_chooser_set_uri(GTK_FILE_CHOOSER(dialog),set);
+		}
+	} else if (!localonly && bfwin) { /* localonly is used for the project files */
+		if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
+			DEBUG_MSG("file_chooser_dialog, opendir=%s, savedir=%s\n",bfwin->session->opendir,bfwin->session->savedir);
+			if (bfwin->session->savedir) file_chooser_set_current_dir(dialog,bfwin->session->savedir);
+			else if (bfwin->session->opendir) file_chooser_set_current_dir(dialog,bfwin->session->opendir);
+		} else {
+			DEBUG_MSG("file_chooser_dialog, opendir=%s\n",bfwin->session->opendir);
+			file_chooser_set_current_dir(dialog,bfwin->session->opendir);
+		}
+	}
+#ifdef DEBUG
+	DEBUG_MSG("file_chooser_dialog, current_folder_uri=%s\n", gtk_file_chooser_get_current_folder_uri(GTK_FILE_CHOOSER(dialog)));
+#endif
+	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog),localonly);
+	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), multiple);
+/*	if (bfwin) {
+		GList *tmplist;
+		/ *tmplist = g_list_first(bfwin->session->recent_dirs);* /
+		tmplist = g_list_first(main_v->recent_directories);
+		DEBUG_MSG("file_chooser_dialog, session dir len=%d\n",g_list_length(bfwin->session->recent_dirs));
+		while (tmplist) {
+			GError *error=NULL;
+			gtk_file_chooser_add_shortcut_folder_uri(GTK_FILE_CHOOSER(dialog),(gchar*)tmplist->data,&error);
+			DEBUG_MSG("file_chooser_dialog, adding folder %s\n",(gchar*)tmplist->data);
+			if (error) {
+				DEBUG_MSG("file_chooser_dialog, ERROR adding dir %s: %s\n", (gchar*)tmplist->data, error->message);
+				g_error_free(error);
+			}
+			tmplist = g_list_next(tmplist);
+		}
+	}*/
+	vbox = gtk_vbox_new(FALSE, 5);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
+	viewlocal = boxed_checkbut_with_value(_("Show hidden"), 0, hbox);
+	g_signal_connect(G_OBJECT(viewlocal), "toggled", G_CALLBACK(viewlocal_toggled_lcb), dialog);
+	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog),vbox);
+	if (action == GTK_FILE_CHOOSER_ACTION_OPEN || action == GTK_FILE_CHOOSER_ACTION_SAVE){
+		GList *tmplist;
+		GtkFileFilter* ff;
+		ff = gtk_file_filter_new();
+		gtk_file_filter_set_name(ff,_("All files"));
+		gtk_file_filter_add_pattern(ff, "*");
+		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), ff);
+		if (filter == NULL)	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), ff);
+		tmplist = g_list_first(main_v->filetypelist);
+		while (tmplist) {
+			gchar **tmp;
+			Tfiletype *ft = (Tfiletype *)tmplist->data;
+			if ((strcmp(ft->type, "objectfile") != 0) && ft->extensions && *ft->extensions) {
+				ff = gtk_file_filter_new();
+				gtk_file_filter_set_name(ff,ft->type);
+				DEBUG_MSG("file_chooser_dialog, adding filter '%s'\n", ft->type);
+				tmp = ft->extensions;
+				while (*tmp) {
+					gchar *pattern;
+					pattern = g_strconcat("*", *tmp, NULL);
+					gtk_file_filter_add_pattern(ff, pattern);
+					DEBUG_MSG("file_chooser_dialog, adding pattern '%s' to '%s'\n", pattern, ft->type);
+					g_free(pattern);
+					tmp++;
+				}
+				gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), ff);
+				if (filter && strcmp(filter, ft->type) == 0)	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), ff);
+			}
+			tmplist = g_list_next(tmplist);
+		}
+	}
+	gtk_widget_show_all(vbox);
+	return dialog;
+}
+
+#else
 typedef struct {
 	gboolean select_dir;
 	gint multipleselect;
@@ -1295,7 +1444,7 @@ static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
 {
 	gchar *dirname, *selected_file;
 
-	selected_file = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs)));
+	selected_file = get_utf8filename_from_on_disk_encoding(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs)));
 
 	if (g_file_test(selected_file, G_FILE_TEST_IS_DIR)) {
 		DEBUG_MSG("fs_ok_clicked_lcb,file_is_dir said %s is a dir!!!!\n", selected_file);
@@ -1323,7 +1472,7 @@ static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
 		orig = filenames = gtk_file_selection_get_selections(GTK_FILE_SELECTION(fileselect->fs));
 		if (filenames) {
 			while (*filenames) {
-				fileselect->filenames_to_return = g_list_append(fileselect->filenames_to_return, g_strdup(*filenames));
+				fileselect->filenames_to_return = g_list_append(fileselect->filenames_to_return, get_utf8filename_from_on_disk_encoding(*filenames));
 				filenames++;
 			}
 			g_strfreev(orig);
@@ -1488,6 +1637,7 @@ gchar *return_dir(gchar *setdir, gchar *title) {
 	gtk_main();
 	return fileselect.filename_to_return;
 }
+#endif /* HAVE_ATLEAST_GTK_2_4 */
 
 /************************************************************************/
 
