@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * gui.c - the main GUI
  *
- * Copyright (C) 2002-2003 Olivier Sessink
+ * Copyright (C) 2002-2004 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -196,7 +196,11 @@ static void left_panel_notify_position_lcb(GObject *object,GParamSpec *pspec,gpo
 	g_object_get(object, pspec->name, &position, NULL);
 	DEBUG_MSG("left_panel_notify_position_lcb, new position=%d\n", position);
 	if (main_v->props.restore_dimensions) {
-		main_v->props.left_panel_width = position;
+		if (main_v->props.left_panel_left) {
+			main_v->props.left_panel_width = position;
+		} else {
+			main_v->props.left_panel_width = main_v->globses.main_window_w - position;
+		}
 	}
 }
 
@@ -235,13 +239,20 @@ void left_panel_rebuild(Tbfwin *bfwin) {
 		bmark_cleanup(bfwin);
 		DEBUG_MSG("left_panel_rebuild, re-init\n");
 		bfwin->leftpanel_notebook = left_panel_build(bfwin);
-		gtk_paned_add1(GTK_PANED(bfwin->hpane), bfwin->leftpanel_notebook);
+		if (main_v->props.left_panel_left) {
+			gtk_paned_add1(GTK_PANED(bfwin->hpane), bfwin->leftpanel_notebook);
+		} else {
+			gtk_paned_add2(GTK_PANED(bfwin->hpane), bfwin->leftpanel_notebook);
+		}
 		gtk_widget_show_all(bfwin->leftpanel_notebook);
 	}
 }
 
 void left_panel_show_hide_toggle(Tbfwin *bfwin,gboolean first_time, gboolean show, gboolean sync_menu) {
-	if (sync_menu) setup_toggle_item(gtk_item_factory_from_widget(bfwin->menubar), N_("/View/View Left Panel"), show);
+	if (sync_menu) {
+		DEBUG_MSG("left_panel_show_hide_toggle, trying to sync menu\n");
+		setup_toggle_item_from_widget(bfwin->menubar, "/View/View Sidebar", show);
+	}
 	if (!first_time && ((show && bfwin->hpane) || (!show && bfwin->hpane == NULL))) {
 		DEBUG_MSG("left_panel_show_hide_toggle, retrurning!!, show=%d, bfwin->hpane=%p, first_time=%d\n",show,bfwin->hpane,first_time);
 		return;
@@ -261,11 +272,17 @@ void left_panel_show_hide_toggle(Tbfwin *bfwin,gboolean first_time, gboolean sho
 	}
 	if (show) {
 		bfwin->hpane = gtk_hpaned_new();
-		gtk_paned_set_position(GTK_PANED(bfwin->hpane), main_v->props.left_panel_width);
+		if (main_v->props.left_panel_left) {
+			DEBUG_MSG("set paned position to %d (left)\n",main_v->props.left_panel_width);
+			gtk_paned_set_position(GTK_PANED(bfwin->hpane), main_v->props.left_panel_width);
+		} else {
+			DEBUG_MSG("set paned position to %d (right)\n",main_v->globses.main_window_w - main_v->props.left_panel_width);
+			gtk_paned_set_position(GTK_PANED(bfwin->hpane), main_v->globses.main_window_w - main_v->props.left_panel_width);
+		}
 		g_signal_connect(G_OBJECT(bfwin->hpane),"notify::position",G_CALLBACK(left_panel_notify_position_lcb), NULL);
 		bfwin->leftpanel_notebook = left_panel_build(bfwin);
-		gtk_paned_add1(GTK_PANED(bfwin->hpane), bfwin->leftpanel_notebook);
-		gtk_paned_add2(GTK_PANED(bfwin->hpane), bfwin->notebook_box);
+		gtk_paned_add1(GTK_PANED(bfwin->hpane), main_v->props.left_panel_left ? bfwin->leftpanel_notebook : bfwin->notebook_box);
+		gtk_paned_add2(GTK_PANED(bfwin->hpane), main_v->props.left_panel_left ? bfwin->notebook_box : bfwin->leftpanel_notebook);
 		gtk_box_pack_start(GTK_BOX(bfwin->middlebox), bfwin->hpane, TRUE, TRUE, 0);
 		gtk_widget_show(bfwin->hpane);
 	} else {
@@ -295,11 +312,19 @@ void gui_set_title(Tbfwin *bfwin, Tdocument *doc) {
 }
 
 void gui_apply_settings(Tbfwin *bfwin) {
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(bfwin->notebook),main_v->props.document_tabposition);
 	/* We don't want to set the tab position if the left panel is hidden */
 	if (bfwin->leftpanel_notebook) {
 		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(bfwin->leftpanel_notebook),main_v->props.leftpanel_tabposition);
+
+		/* check if the left panel needs to move over to the right */
+		if (main_v->props.left_panel_left && bfwin->leftpanel_notebook == GTK_PANED(bfwin->hpane)->child1) {
+			DEBUG_MSG("gui_apply_settings, left panel is on the right location\n");
+		} else {
+			left_panel_show_hide_toggle(bfwin,FALSE, FALSE, FALSE);
+			left_panel_show_hide_toggle(bfwin,FALSE, TRUE, FALSE);
+		}
 	}
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(bfwin->notebook),main_v->props.document_tabposition);
 }
 
 typedef struct {
@@ -671,12 +696,186 @@ static void html_toolbar_remove_from_quickbar_lcb(GtkMenuItem *menuitem, Ttoolba
 	}
 }
 
+/* return the position in the quickbar GList of a tool bar item */
+gint get_quickbar_item_position(Ttoolbaritem *tbitem) {
+	GList *tmplist, *winlist = g_list_first(main_v->bfwinlist);
+	gint pos = 0;
+	tmplist  = g_list_first(BFWIN(winlist->data)->toolbar_quickbar_children);
+
+	while (tmplist) {
+		Tquickbaritem *qbi;
+		qbi = tmplist->data;
+		if (qbi->tbitem == tbitem) {
+			DEBUG_MSG("get_quickbar_item_position, quickbar item is at %d\n", pos);
+			return pos;
+		}
+		pos += 1;
+		tmplist = g_list_next(tmplist);
+	}
+	DEBUG_MSG("get_quickbar_item_position, not found, return -1!\n");
+	return -1;
+}
+
+static gboolean html_toolbar_quickbar_item_button_press_lcb(GtkWidget *widget,GdkEventButton *bevent,Ttoolbaritem *tbitem);
+
+static void html_toolbar_quickbar_switch(Ttoolbaritem *tbitem, gboolean moveright) {
+	gint pos;
+
+	/* first we look for the current location of this item */	
+	pos = get_quickbar_item_position(tbitem);
+	DEBUG_MSG("html_toolbar_quickbar_switch, found item %s at pos=%d, config list length=%d\n",tbitem->ident,pos,g_list_length(main_v->globses.quickbar_items));
+	if (pos >= 0) {
+		GList *tmp1, *tmp2, *tmplist;
+		/* then we move this item to the new place in the config list */
+		tmp1 = g_list_nth(main_v->globses.quickbar_items, pos);
+		if (!tmp1) {
+			DEBUG_MSG("html_toolbar_quickbar_switch, nothing found in config list at pos=%d, RETURN\n",pos);
+			return;
+		}
+		tmp2 = (moveright) ? g_list_next(tmp1) : g_list_previous(tmp1);
+		if (!tmp2) {
+			DEBUG_MSG("html_toolbar_quickbar_switch, no tmp2 found, moveright=%d, list length=%d RETURN\n",moveright,g_list_length(main_v->globses.quickbar_items));
+			return;
+		}
+		list_switch_order(tmp1, tmp2);
+		DEBUG_MSG("html_toolbar_quickbar_switch, after list_switch_order, list length=%d\n",g_list_length(main_v->globses.quickbar_items));
+		
+		/* now switch the widgets in every window that has a quickbar */
+		tmplist = g_list_first(main_v->bfwinlist);
+		while (tmplist) {
+			Tbfwin *bfwin = BFWIN(tmplist->data);
+			DEBUG_MSG("html_toolbar_quickbar_switch, checking bfwin=%p\n",bfwin);
+			if (bfwin->toolbar_quickbar && bfwin->toolbar_quickbar_children) {
+				Tquickbaritem *qbi1;
+				DEBUG_MSG("retrieving quickbaritem from pos %d\n",pos);
+				tmp1 = g_list_nth(bfwin->toolbar_quickbar_children, pos);
+				tmp2 = (moveright) ? g_list_next(tmp1) : g_list_previous(tmp1);
+				qbi1 = (Tquickbaritem *)tmp1->data;
+				list_switch_order(tmp1, tmp2);
+				DEBUG_MSG("html_toolbar_quickbar_switch, quickbaritem %p has tbitem %s\n",qbi1, qbi1->tbitem->ident);
+				/* I don't know why the next bit of code doesn't work... bug in gtk? */
+				/*g_object_ref(G_OBJECT(qbi1->button));
+				DEBUG_MSG("html_toolbar_quickbar_switch, detaching widget!, button=%p, button->parent=%p, toolbar=%p\n",qbi1->button,qbi1->button->parent,bfwin->toolbar_quickbar);
+				gtk_container_remove(GTK_CONTAINER(bfwin->toolbar_quickbar),qbi1->button);
+				DEBUG_MSG("html_toolbar_quickbar_switch, attaching widget at pos %d\n",(moveright)?pos+1:pos-1);
+				gtk_toolbar_insert_widget(GTK_TOOLBAR(bfwin->toolbar_quickbar),qbi1->button,
+							_(qbi1->tbitem->tooltiptext),"",(moveright)?pos+1:pos-1);
+				g_object_unref(G_OBJECT(qbi1->button));
+				*/
+				/* this workaround bit of code removes (and thus destroys) the button and creates a new identical button on the new location */
+				DEBUG_MSG("html_toolbar_quickbar_switch, about to destroy button,(toolbar has %d children)\n",g_list_length(GTK_TOOLBAR(bfwin->toolbar_quickbar)->children));
+				gtk_container_remove(GTK_CONTAINER(bfwin->toolbar_quickbar),qbi1->button);
+				DEBUG_MSG("html_toolbar_quickbar_switch, inserting widget at pos %d (toolbar has %d children)\n",(moveright)?pos+1:pos-1, g_list_length(GTK_TOOLBAR(bfwin->toolbar_quickbar)->children));
+				qbi1->button = gtk_toolbar_insert_item(GTK_TOOLBAR(bfwin->toolbar_quickbar), NULL, _(qbi1->tbitem->tooltiptext),
+							"", new_pixmap(qbi1->tbitem->pixmaptype), G_CALLBACK(qbi1->tbitem->func), bfwin, (moveright)?pos+1:pos-1);
+				g_signal_connect(qbi1->button, "button-press-event", G_CALLBACK(html_toolbar_quickbar_item_button_press_lcb), tbitem);
+				gtk_widget_show(qbi1->button);
+			}
+			tmplist = g_list_next(tmplist);
+		}
+		
+	}
+}
+
+static void html_toolbar_quickbar_move_left_lcb(GtkMenuItem *menuitem, Ttoolbaritem *tbitem) {
+	html_toolbar_quickbar_switch(tbitem, FALSE);
+/*	GList *winlist = g_list_first(main_v->bfwinlist); 
+	DEBUG_MSG("moving tbitem %p left on quickbars\n", tbitem);
+	while (winlist) {
+		GList *tmplist;
+		gint pos;
+		Tbfwin *bfwin = BFWIN(winlist->data);
+		Tquickbaritem *qbi;
+
+		pos = get_quickbar_item_position(tbitem);
+		tmplist = g_list_nth(bfwin->toolbar_quickbar_children, pos);
+		qbi = tmplist->data;
+
+		if (qbi) {
+			if (pos) { / * pos > 0 * /
+				gtk_widget_destroy(qbi->button);
+			}
+			g_free(qbi);
+		}
+		
+		if (pos) {
+			GList *other;
+			qbi = g_new(Tquickbaritem,1);
+			qbi->button = gtk_toolbar_insert_item(GTK_TOOLBAR(bfwin->toolbar_quickbar), NULL, _(tbitem->tooltiptext),
+							"", new_pixmap(tbitem->pixmaptype), G_CALLBACK(tbitem->func), bfwin, pos-1);
+			g_signal_connect(qbi->button, "button-press-event", G_CALLBACK(html_toolbar_quickbar_item_button_press_lcb), tbitem);
+			gtk_widget_show(qbi->button);
+			qbi->tbitem = tbitem;
+			
+			other = g_list_previous(tmplist);
+			list_switch_order(tmplist, other);
+			
+			tmplist = g_list_nth(main_v->globses.quickbar_items, pos);
+			other = g_list_previous(tmplist);
+			list_switch_order(tmplist, other);
+			g_list_free(tmplist);
+			g_list_free(other);
+		}
+		winlist = g_list_next(winlist);
+	}
+	 we don't create this list here so we don't need to free it!!
+	g_list_free(winlist);*/
+}
+
+static void html_toolbar_quickbar_move_right_lcb(GtkMenuItem *menuitem, Ttoolbaritem *tbitem) {
+	html_toolbar_quickbar_switch(tbitem, TRUE);
+/*	GList *winlist = g_list_first(main_v->bfwinlist); 
+	DEBUG_MSG("moving tbitem %p right on quickbars\n", tbitem);
+	while (winlist) {
+		gint pos;
+		GList *tmplist, *other;
+		Tbfwin *bfwin = BFWIN(winlist->data);
+		Tquickbaritem *qbi;
+		pos = get_quickbar_item_position(tbitem);
+		tmplist = g_list_nth(bfwin->toolbar_quickbar_children, pos);
+		qbi = tmplist->data;
+
+		if (qbi) {
+			gtk_widget_hide(qbi->button);
+			gtk_widget_destroy(qbi->button);
+			g_free(qbi);
+		}
+		
+		qbi = g_new(Tquickbaritem,1);
+		qbi->button = gtk_toolbar_insert_item(GTK_TOOLBAR(bfwin->toolbar_quickbar), NULL, _(tbitem->tooltiptext),
+							"", new_pixmap(tbitem->pixmaptype), G_CALLBACK(tbitem->func), bfwin, pos+1);
+		g_signal_connect(qbi->button, "button-press-event", G_CALLBACK(html_toolbar_quickbar_item_button_press_lcb), tbitem);
+		gtk_widget_show(qbi->button);
+		qbi->tbitem = tbitem;
+		
+		other = g_list_next(tmplist);
+		list_switch_order(tmplist, other);
+			
+		tmplist = g_list_nth(main_v->globses.quickbar_items, pos);
+		other = g_list_next(tmplist);
+		list_switch_order(tmplist, other);
+		g_list_free(tmplist);
+		g_list_free(other);
+		
+		winlist = g_list_next(winlist);
+	}
+	/ * g_list_free(winlist); */
+}
+
 static gboolean html_toolbar_quickbar_item_button_press_lcb(GtkWidget *widget,GdkEventButton *bevent,Ttoolbaritem *tbitem) {
 	if (bevent->button == 3) {
 		GtkWidget *menu = gtk_menu_new ();
 		GtkWidget *menuitem = gtk_menu_item_new_with_label(_("Remove from quickbar"));
+		GtkWidget *itemleft = gtk_menu_item_new_with_label(_("Move Left"));
+		GtkWidget *itemright = gtk_menu_item_new_with_label(_("Move Right"));
+		GtkWidget *sep = gtk_separator_menu_item_new();
 		DEBUG_MSG("popup for tbitem %p\n", tbitem);
 		g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(html_toolbar_remove_from_quickbar_lcb), tbitem);
+		g_signal_connect(G_OBJECT(itemleft), "activate", G_CALLBACK(html_toolbar_quickbar_move_left_lcb), tbitem);
+		g_signal_connect(G_OBJECT(itemright), "activate", G_CALLBACK(html_toolbar_quickbar_move_right_lcb), tbitem);
+		gtk_menu_append(GTK_MENU(menu), itemleft);
+		gtk_menu_append(GTK_MENU(menu), itemright);
+		gtk_menu_append(GTK_MENU(menu), sep);
 		gtk_menu_append(GTK_MENU(menu), menuitem);
 		gtk_widget_show_all (menu);
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
@@ -747,7 +946,7 @@ static void html_toolbar_add_items_to_submenu(Tbfwin *bfwin, GtkWidget *html_too
 			gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 		} else {
 			menu_item = gtk_menu_item_new();
-			gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), menu_item, tbi[i].tooltiptext, NULL);
+			gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), menu_item, _(tbi[i].tooltiptext), NULL);
 			gtk_container_add(GTK_CONTAINER(menu_item), new_pixmap(tbi[i].pixmaptype));
 			g_signal_connect(GTK_OBJECT(menu_item), "activate", G_CALLBACK(tbi[i].func), bfwin);
 			g_signal_connect(menu_item, "button-press-event", G_CALLBACK(html_toolbar_item_button_press_lcb), &tbi[i]);
@@ -798,6 +997,7 @@ void make_html_toolbar(Tbfwin *bfwin) {
 					g_signal_connect(qbi->button, "button-press-event", G_CALLBACK(html_toolbar_quickbar_item_button_press_lcb), &tbi[i]);
 					qbi->tbitem = &tbi[i];
 					bfwin->toolbar_quickbar_children = g_list_append(bfwin->toolbar_quickbar_children, qbi);
+					DEBUG_MSG("make_html_toolbar, appended %s as quickbaritem %p to quickbar %p\n",qbi->tbitem->ident, qbi,bfwin->toolbar_quickbar);
 					break;
 				}
 			}
@@ -810,13 +1010,13 @@ void make_html_toolbar(Tbfwin *bfwin) {
 	gtk_toolbar_set_style(GTK_TOOLBAR(html_toolbar), GTK_TOOLBAR_ICONS);
 	html_toolbar_add_items(bfwin,html_toolbar, tbi, 0, 14);
 	html_toolbar_add_items_to_submenu(bfwin,html_toolbar, tbi, 26, 31, _("Heading"), 91);
-	html_toolbar_add_items(bfwin,html_toolbar, tbi, 75, 77);
+	html_toolbar_add_items(bfwin,html_toolbar, tbi, 76, 78);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), html_toolbar, gtk_label_new(_(" Standard bar ")));
 
 	html_toolbar = gtk_toolbar_new();
 	gtk_toolbar_set_style(GTK_TOOLBAR(html_toolbar), GTK_TOOLBAR_ICONS);
 	html_toolbar_add_items(bfwin,html_toolbar, tbi, 15, 25);
-	html_toolbar_add_items_to_submenu(bfwin,html_toolbar, tbi, 78, 85, _("Context formatting"), 103);
+	html_toolbar_add_items_to_submenu(bfwin,html_toolbar, tbi, 79, 86, _("Context formatting"), 103);
 	html_toolbar_add_items(bfwin,html_toolbar, tbi, 25, 31);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), html_toolbar, gtk_label_new(_(" Fonts ")));
 
@@ -871,7 +1071,7 @@ void make_main_toolbar(Tbfwin *bfwin) {
 
 	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_NEW, _("New"),
 								"", G_CALLBACK(file_new_cb), bfwin, -1);
-	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_OPEN, _("Open.."),
+	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_OPEN, _("Open..."),
 								"", G_CALLBACK(file_open_cb), bfwin, -1);
 	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_SAVE, _("Save"),
 								"", G_CALLBACK(file_save_cb), bfwin, -1);
@@ -887,9 +1087,9 @@ void make_main_toolbar(Tbfwin *bfwin) {
 	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_PASTE, _("Paste"),	
 								"", G_CALLBACK(edit_paste_cb), bfwin, -1);
 	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));								
-	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_FIND, _("Search..."),	
+	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_FIND, _("Find..."),	
 								"", G_CALLBACK(search_cb), bfwin, -1);
-	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_FIND_AND_REPLACE, _("Search and Replace..."),	
+	gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_FIND_AND_REPLACE, _("Replace..."),	
 								"", G_CALLBACK(replace_cb), bfwin, -1);
 	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));								
 	bfwin->toolbar_undo = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_UNDO, _("Undo"),	
@@ -925,20 +1125,21 @@ void gui_set_undo_redo_widgets(Tbfwin *bfwin, gboolean undo, gboolean redo) {
 		gtk_widget_set_sensitive(bfwin->toolbar_redo, redo);
 		gtk_widget_set_sensitive(bfwin->toolbar_undo, undo);
 	}
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(gtk_item_factory_from_widget(bfwin->menubar), N_("/Edit/Undo")), undo);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(gtk_item_factory_from_widget(bfwin->menubar), N_("/Edit/Undo All")), undo);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(gtk_item_factory_from_widget(bfwin->menubar), N_("/Edit/Redo")), redo);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(gtk_item_factory_from_widget(bfwin->menubar), N_("/Edit/Redo All")), redo);
+	
+	menuitem_set_sensitive(bfwin->menubar, "/Edit/Undo", undo);
+	menuitem_set_sensitive(bfwin->menubar, "/Edit/Undo All", undo);
+	menuitem_set_sensitive(bfwin->menubar, "/Edit/Redo", redo);
+	menuitem_set_sensitive(bfwin->menubar, "/Edit/Redo All", redo);
 }
 
 void gui_set_document_widgets(Tdocument *doc) {
 	GtkItemFactory *tmp1 = gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar);
-	setup_toggle_item(tmp1,N_("/Document/Highlight Syntax"), (doc->highlightstate && doc->hl->highlightlist != NULL));
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(tmp1,N_("/Document/Highlight Syntax")), (doc->hl->highlightlist != NULL));
+	setup_toggle_item(tmp1,("/Document/Highlight Syntax"), (doc->highlightstate && doc->hl->highlightlist != NULL));
+	/*gtk_widget_set_sensitive(gtk_item_factory_get_widget(tmp1,_("/Document/Highlight Syntax")), (doc->hl->highlightlist != NULL));*/
 	gui_set_undo_redo_widgets(doc->bfwin, doc_has_undo_list(doc), doc_has_redo_list(doc));
-	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),N_("/Document/Wrap"), doc->wrapstate);
-	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),N_("/Document/Line Numbers"), doc->linenumberstate);
-	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),N_("/Document/Auto Close HTML tags"), doc->autoclosingtag);
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),"/Document/Wrap", doc->wrapstate);
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),"/Document/Line Numbers", doc->linenumberstate);
+	setup_toggle_item(gtk_item_factory_from_widget(BFWIN(doc->bfwin)->menubar),"/Document/Auto Close HTML tags", doc->autoclosingtag);
 	menu_current_document_set_toggle_wo_activate(BFWIN(doc->bfwin),doc->hl, doc->encoding);
 }
 
@@ -1082,21 +1283,11 @@ gboolean main_window_delete_event_lcb(GtkWidget *widget,GdkEvent *event,Tbfwin *
 	 * type dialogs. */
 	DEBUG_MSG("main_window_delete_event_lcb, started\n");
 	if (bfwin->project) {
-		if (!project_save_and_close(bfwin)) {
-			gint retval;
-			gchar *options[] = {_("Do_n't Save"), _("_Cancel"), NULL};
-			retval = multi_warning_dialog(bfwin->main_window,_("Failed to save the current project"), NULL, 0, 1, options);
-			if (retval == 1) {
-				DEBUG_MSG("main_window_delete_event_lcb, cancel clicked, returning TRUE\n");
-				return TRUE;
-			}
-			DEBUG_MSG("main_window_delete_event_lcb, continue clicked, returning FALSE\n");
-			return FALSE;
-		}
+		return !project_save_and_close(bfwin);
 	} else {
 		if (bfwin->documentlist && test_docs_modified(bfwin->documentlist)) {
 			DEBUG_MSG("main_window_delete_event_lcb, we have changed documents!\n");
-			file_close_all_cb(NULL, bfwin);
+			bfwin_close_all_documents(bfwin, TRUE);
 			if (bfwin->documentlist && test_docs_modified(bfwin->documentlist)) {
 				DEBUG_MSG("main_window_delete_event_lcb, we STILL have changed documents!?!\n");
 				/* if there are still documents modified we should cancel the closing */
@@ -1130,8 +1321,11 @@ void gui_create_main(Tbfwin *bfwin, GList *filenames) {
 	gtk_widget_show(vbox);
 
 	/* first a menubar */
+	DEBUG_MSG("gui_create_main, starting menu_create_main\n");
 	menu_create_main(bfwin, vbox);
+	DEBUG_MSG("gui_create_main, starting recent_menu\n");
 	recent_menu_init(bfwin);
+	DEBUG_MSG("gui_create_main, starting external-encoding_menu\n");
 	external_menu_rebuild(bfwin);
 	encoding_menu_rebuild(bfwin);
 	snr2_init(bfwin);
@@ -1139,6 +1333,7 @@ void gui_create_main(Tbfwin *bfwin, GList *filenames) {
 	add_allwindows_entries_to_window(bfwin);*/
 	/* then the toolbars */
 	{
+		DEBUG_MSG("gui_create_main, creating handles for all menu/toolbars\n");
 		bfwin->main_toolbar_hb = gtk_handle_box_new();
 		gtk_box_pack_start(GTK_BOX(vbox), bfwin->main_toolbar_hb, FALSE, FALSE, 0);
 		bfwin->html_toolbar_hb = gtk_handle_box_new();
@@ -1384,9 +1579,6 @@ void go_to_line_win_cb(Tbfwin *bfwin,guint callback_action, GtkWidget *widget) {
 	gtk_box_pack_start(GTK_BOX(hbox), but1, FALSE, FALSE, 0);
 	gtk_window_set_default(GTK_WINDOW(tgl->win), but1);
 
-	g_signal_connect(G_OBJECT (tgl->entry), "activate", G_CALLBACK(tgl_enter_lcb),
-                            (gpointer *) tgl);
-
 	gtk_widget_show_all(tgl->win);
 }
 
@@ -1427,12 +1619,12 @@ GtkWidget *start_splash_screen() {
 	gtk_window_set_decorated(GTK_WINDOW(splashscreen.window), FALSE);
 	gtk_window_set_role(GTK_WINDOW(splashscreen.window), "splash");
 	gtk_window_set_resizable(GTK_WINDOW(splashscreen.window),FALSE);
-	gtk_widget_set_size_request(splashscreen.window, 340, 400);
 	color.red = 65535;
 	color.blue = 65535;
 	color.green = 65535;
 	gtk_widget_modify_bg(splashscreen.window, GTK_STATE_NORMAL,&color);
 
+
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(splashscreen.window), vbox);
 	gtk_widget_show(vbox);
@@ -1462,21 +1654,30 @@ GtkWidget *start_splash_screen() {
 #endif /* #ifndef NOSPLASH */
 
 void gui_set_html_toolbar_visible(Tbfwin *bfwin, gboolean visible, gboolean sync_menu) {
-	if (sync_menu) setup_toggle_item(gtk_item_factory_from_widget(bfwin->menubar), N_("/View/View HTML Toolbar"), visible);
+	if (sync_menu) {
+		DEBUG_MSG("gui_set_html_toolbar_visible, trying to sync menu\n");
+		setup_toggle_item_from_widget(bfwin->menubar, "/View/View HTML Toolbar", visible);
+	}
 	if (gtk_container_children(GTK_CONTAINER(bfwin->html_toolbar_hb)) == NULL) {
 		make_html_toolbar(bfwin);
 	}
 	widget_set_visible(bfwin->html_toolbar_hb,visible);
 }
 void gui_set_main_toolbar_visible(Tbfwin *bfwin, gboolean visible, gboolean sync_menu) {
-	if (sync_menu) setup_toggle_item(gtk_item_factory_from_widget(bfwin->menubar), N_("/View/View Main Toolbar"), visible);
+	if (sync_menu) {
+		DEBUG_MSG("gui_set_main_toolbar_visible, trying to sync menu\n");
+		setup_toggle_item_from_widget(bfwin->menubar, "/View/View Main Toolbar", visible);
+	}
 	if (gtk_container_children(GTK_CONTAINER(bfwin->main_toolbar_hb)) == NULL) {
 		make_main_toolbar(bfwin);
 	}
 	widget_set_visible(bfwin->main_toolbar_hb,visible);
 }
 void gui_set_custom_menu_visible(Tbfwin *bfwin, gboolean visible, gboolean sync_menu) {
-	if (sync_menu) setup_toggle_item(gtk_item_factory_from_widget(bfwin->menubar), N_("/View/View Custom Menu"), visible);
+	if (sync_menu) {
+		DEBUG_MSG("gui_set_custom_menu_visible, trying to sync menu\n");
+		setup_toggle_item_from_widget(bfwin->menubar, "/View/View Custom Menu", visible);
+	}
 	if (gtk_container_children(GTK_CONTAINER(bfwin->custom_menu_hb)) == NULL) {
 		make_cust_menubar(bfwin,bfwin->custom_menu_hb);
 	}
