@@ -19,7 +19,6 @@
  */
 #ifndef __GUI_H_
 #define __GUI_H_
-void gui_toggle_autoindent_cb(gpointer callback_data,guint action,GtkWidget *widget);
 void gui_set_html_toolbar_visible(Tbfwin *bfwin, gboolean visible, gboolean sync_menu);
 void gui_set_main_toolbar_visible(Tbfwin *bfwin, gboolean visible, gboolean sync_menu);
 void gui_set_custom_menu_visible(Tbfwin *bfwin, gboolean visible, gboolean sync_menu);
@@ -35,7 +34,7 @@ void left_panel_show_hide_toggle(Tbfwin *bfwin,gboolean first_time, gboolean sho
 void gui_set_title(Tbfwin *bfwin, Tdocument *doc);
 void gui_apply_settings(Tbfwin *bfwin);
 void gui_set_undo_redo_widgets(Tbfwin *bfwin,gboolean undo, gboolean redo);
-void gui_set_widgets(Tbfwin *bfwin,gboolean undo, gboolean redo, gboolean wrap, gboolean highlight, Tfiletype *hl, gchar *encoding, gboolean linenumbers);
+void gui_set_document_widgets(Tdocument *doc);
 
 void gui_notebook_bind_signals(Tbfwin *bfwin);
 void gui_notebook_unbind_signals(Tbfwin *bfwin);
