@@ -50,5 +50,5 @@ void frameset_dialog(Tbfwin *bfwin, Ttagpopup *data);
 void frame_dialog(Tbfwin *bfwin, Ttagpopup *data);
 void embed_dialog(Tbfwin *bfwin, Ttagpopup *data);
 void script_dialog(Tbfwin *bfwin, Ttagpopup *data);
-void link_dialog(Tbfwin *bfwin, Ttagpopup *data);
+void linkdialog_dialog(Tbfwin *bfwin, Ttagpopup *data);
 #endif							/* __HTML_H_ */
