@@ -18,13 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/*#define DEBUG*/
+/* #define DEBUG */
 
 #include <gtk/gtk.h>
 #include <string.h>  	/* strlen() */
 #include <stdlib.h>	/* strtol() */
 
 #include "bluefish.h"
+#include "bf_lib.h" /* string_is_color*/
 #include "gtk_easy.h"
 #include "bf_lib.h"  /* strip_any_whitespace() */
 #include "stringlist.h"
@@ -44,43 +45,31 @@ static GList *glist_with_html_tags(gint with_pseudo_classes) {
 	} else {
 		tmplist = g_list_append(tmplist, "a");
 	}
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "abbr");
-		tmplist = g_list_append(tmplist, "acronym");
-	}
+	tmplist = g_list_append(tmplist, "abbr");
+	tmplist = g_list_append(tmplist, "acronym");
 	tmplist = g_list_append(tmplist, "address");
 	tmplist = g_list_append(tmplist, "applet");
 	tmplist = g_list_append(tmplist, "area");
 	tmplist = g_list_append(tmplist, "b");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "bdo");
-	}
+	tmplist = g_list_append(tmplist, "bdo");
 	tmplist = g_list_append(tmplist, "blockquote");
 	tmplist = g_list_append(tmplist, "body");
 	tmplist = g_list_append(tmplist, "br");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "button");
-	}
+	tmplist = g_list_append(tmplist, "button");
 	tmplist = g_list_append(tmplist, "caption");
 	tmplist = g_list_append(tmplist, "cite");
 	tmplist = g_list_append(tmplist, "code");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "col");
-		tmplist = g_list_append(tmplist, "colgroup");
-	}
+	tmplist = g_list_append(tmplist, "col");
+	tmplist = g_list_append(tmplist, "colgroup");
 	tmplist = g_list_append(tmplist, "dd");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "del");
-	}
+	tmplist = g_list_append(tmplist, "del");
 	tmplist = g_list_append(tmplist, "dfn");
 	tmplist = g_list_append(tmplist, "div");
 	tmplist = g_list_append(tmplist, "dl");
 	tmplist = g_list_append(tmplist, "dt");
 	tmplist = g_list_append(tmplist, "em");
 	tmplist = g_list_append(tmplist, "embed");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "fieldset");
-	}
+	tmplist = g_list_append(tmplist, "fieldset");
 	tmplist = g_list_append(tmplist, "form");
 	tmplist = g_list_append(tmplist, "h1");
 	tmplist = g_list_append(tmplist, "h2");
@@ -90,38 +79,24 @@ static GList *glist_with_html_tags(gint with_pseudo_classes) {
 	tmplist = g_list_append(tmplist, "h6");
 	tmplist = g_list_append(tmplist, "hr");
 	tmplist = g_list_append(tmplist, "i");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "iframe");
-	}
+	tmplist = g_list_append(tmplist, "iframe");
 	tmplist = g_list_append(tmplist, "img");
 	tmplist = g_list_append(tmplist, "input");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "ins");
-	}
+	tmplist = g_list_append(tmplist, "ins");
 	tmplist = g_list_append(tmplist, "kbd");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "label");
-		tmplist = g_list_append(tmplist, "legend");
-	}
+	tmplist = g_list_append(tmplist, "label");
+	tmplist = g_list_append(tmplist, "legend");
 	tmplist = g_list_append(tmplist, "li");
 	tmplist = g_list_append(tmplist, "map");
-	if(main_v->props.allow_frames){
-		tmplist = g_list_append(tmplist, "noframes");
-	}
+	tmplist = g_list_append(tmplist, "noframes");
 	tmplist = g_list_append(tmplist, "noscript");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "object");
-	}
+	tmplist = g_list_append(tmplist, "object");
 	tmplist = g_list_append(tmplist, "ol");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "optgroup");
-	}
+	tmplist = g_list_append(tmplist, "optgroup");
 	tmplist = g_list_append(tmplist, "option");
 	tmplist = g_list_append(tmplist, "p");
 	tmplist = g_list_append(tmplist, "pre");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "q");
-	}
+	tmplist = g_list_append(tmplist, "q");
 	if(main_v->props.allow_ruby){
 		tmplist = g_list_append(tmplist, "rb");
 		tmplist = g_list_append(tmplist, "rp");
@@ -131,23 +106,15 @@ static GList *glist_with_html_tags(gint with_pseudo_classes) {
 	tmplist = g_list_append(tmplist, "samp");
 	tmplist = g_list_append(tmplist, "select");
 	tmplist = g_list_append(tmplist, "small");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "span");
-	}
+	tmplist = g_list_append(tmplist, "span");
 	tmplist = g_list_append(tmplist, "strong");
 	tmplist = g_list_append(tmplist, "table");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "tbody");
-	}
+	tmplist = g_list_append(tmplist, "tbody");
 	tmplist = g_list_append(tmplist, "td");
 	tmplist = g_list_append(tmplist, "textarea");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "tfoot");
-	}
+	tmplist = g_list_append(tmplist, "tfoot");
 	tmplist = g_list_append(tmplist, "th");
-	if(main_v->props.allow_h4){
-		tmplist = g_list_append(tmplist, "thead");
-	}
+	tmplist = g_list_append(tmplist, "thead");
 	tmplist = g_list_append(tmplist, "tr");
 	tmplist = g_list_append(tmplist, "tt");
 	tmplist = g_list_append(tmplist, "ul");
@@ -639,7 +606,7 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 
 	scrolwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_widget_set_usize(scrolwin, 400, 300);
+	gtk_widget_set_size_request(scrolwin, 400, 300);
 	gtk_box_pack_start(GTK_BOX(hbox), scrolwin, FALSE, FALSE, 0);
 	
 	if (diag->styletype == multistyle) {
@@ -990,63 +957,18 @@ typedef struct {
 	Tbfwin *bfwin;
 } Tcolsel;
 
-static gint string_is_color(const gchar *color) {
-	gint i;
-
-	if (!color) {
-		DEBUG_MSG("string_is_color, pointer NULL\n");
-		return 0;
-	}
-	if (strlen(color) != 7) {
-		DEBUG_MSG("string_is_color, strlen(%s) != 7\n", color);
-		return 0;
-	}
-	if (color[0] != '#') {
-		DEBUG_MSG("string_is_color, 0 in %s is not #\n", color);
-		return 0;
-	}
-	for (i = 1; i <7 ; i++) {
-		if ((color[i] > 102) 
-				|| (color[i] < 48) 
-				|| ((color[i] > 57) && (color[i] < 65))
-				|| ((color[i] > 70) && (color[i] < 97))) {
-			DEBUG_MSG("string_is_color, %d in %s is not from a color, it is %d\n", i, color, color[i]);
-			return 0;
-		}
-	}
-	DEBUG_MSG("string_is_color, %s is color\n", color);
-	return 1;
-
-}
-
-static gchar *gdouble_arr_to_hex(gdouble *color, gint websafe)
-{
+static gchar *GdkColor_to_hexstring(GdkColor *color, gboolean websafe) {
 	gchar *tmpstr;
-	unsigned int red_int;
-	unsigned int green_int;
-	unsigned int blue_int;
-	gdouble red;
-	gdouble green;
-	gdouble blue;
-	
-	red = color[0];
-	green = color[1];
-	blue = color[2];
 
-	if (websafe) {
-		red_int = 0x33*((unsigned int) (red * 255 / 0x33));
-		green_int = 0x33*((unsigned int) (green * 255/0x33));
-		blue_int = 0x33*((unsigned int) (blue * 255/0x33));
-	} else {
-		red_int = (unsigned int) (red * 255);
-		green_int = (unsigned int) (green * 255);
-		blue_int = (unsigned int) (blue * 255);
-	}
 	tmpstr = g_malloc(8*sizeof(char));
-	g_snprintf (tmpstr, 8,"#%.2X%.2X%.2X", red_int, green_int, blue_int);
+	if (websafe) {
+		g_snprintf (tmpstr, 8,"#%.2X%.2X%.2X", (0x33 * color->red/(256*0x33)), (0x33 * color->green/(256*0x33)), (0x33 * color->blue/(256*0x33)) );
+	} else {
+		g_snprintf (tmpstr, 8,"#%.2X%.2X%.2X", color->red/256, color->green/256, color->blue/256);
+	}
 	return tmpstr;
 }
-
+/*
 static gdouble *hex_to_gdouble_arr(const gchar *color) {
 	static gdouble tmpcol[4];
 	gchar tmpstr[8];
@@ -1073,7 +995,7 @@ static gdouble *hex_to_gdouble_arr(const gchar *color) {
 	tmpcol[3] = 0;
 	return tmpcol;
 }
- 
+*/ 
 static void colsel_destroy_lcb(GtkWidget *widget, Tcolsel *csd) {
 	DEBUG_MSG("colsel_destroy_lcb, started for csd=%p\n",csd);
 	g_free(csd->returnval);
@@ -1082,18 +1004,26 @@ static void colsel_destroy_lcb(GtkWidget *widget, Tcolsel *csd) {
 
 static void colsel_ok_clicked_lcb(GtkWidget *widget, Tcolsel *csd) { 
 	gchar *tmpstr;
+	GdkColor gcolor;
 	/* only on a OK click we do the setcolor thing */
-	tmpstr = gtk_editable_get_chars(GTK_EDITABLE(csd->hexentry), 0, -1);
+	gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(csd->csel), &gcolor);
+
+	tmpstr = GdkColor_to_hexstring(&gcolor, FALSE);
+	if (csd->bfwin) {
+		csd->bfwin->session->colorlist = add_to_stringlist(csd->bfwin->session->colorlist, tmpstr);
+	}
+	
 	if (csd->is_modal) {
 		g_free(csd->returnval);
 		csd->returnval = tmpstr;
 		gtk_main_quit();
 	} else {
-		DEBUG_MSG("colsel_destroy_lcb, NOT modal, entering the value %s\n",tmpstr);
 		if (string_is_color(tmpstr)) {
 			if (csd->startpos || csd->endpos) {
+				DEBUG_MSG("colsel_ok_clicked_lcb, replacing with %s\n", tmpstr);
 				doc_replace_text(csd->bfwin->current_document, tmpstr, csd->startpos, csd->endpos);
 			} else {
+				DEBUG_MSG("colsel_ok_clicked_lcb, inserting %s\n", tmpstr);
 				doc_insert_two_strings(csd->bfwin->current_document,tmpstr, NULL);
 			}
 		}
@@ -1106,7 +1036,7 @@ static void colsel_cancel_clicked_lcb(GtkWidget *widget, Tcolsel *csd) {
 	window_destroy(csd->win);
 }
 /* declaration needed to connect/disconnect callback */
-static void colsel_color_changed(GtkWidget *widget, Tcolsel *csd);
+/*static void colsel_color_changed(GtkWidget *widget, Tcolsel *csd);
 
 static void hexentry_color_changed(GtkWidget *widget, Tcolsel *csd) {
 	gdouble *color;
@@ -1123,9 +1053,9 @@ static void hexentry_color_changed(GtkWidget *widget, Tcolsel *csd) {
 		}
 	}
 	g_free(tmpstr);
-}
-
+}*/
 
+/*
 static void colsel_color_changed(GtkWidget *widget, Tcolsel *csd) {
 	gdouble color[4];
 	gchar *tmpstr;
@@ -1137,12 +1067,12 @@ static void colsel_color_changed(GtkWidget *widget, Tcolsel *csd) {
 	gtk_entry_set_text(GTK_ENTRY(csd->hexentry), tmpstr);
 	csd->hex_changed_id = gtk_signal_connect(GTK_OBJECT(csd->hexentry), "changed", G_CALLBACK(hexentry_color_changed), csd);
 	g_free(tmpstr);
-}
+}*/
 
 static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, gint startpos, gint endpos) {
 	Tcolsel *csd;
 	GtkWidget *vbox, *hbox, *but;
-	gdouble *color;
+	GdkColor gcolor;
 	const gchar *this_color=setcolor;
 
 	csd = g_malloc(sizeof(Tcolsel));
@@ -1158,16 +1088,36 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(csd->win), vbox);
 	csd->csel = gtk_color_selection_new();
+	gtk_color_selection_set_has_opacity_control(GTK_COLOR_SELECTION(csd->csel), FALSE);
+	
 	if (this_color) {
-		color = hex_to_gdouble_arr(this_color);
-		if (color) {
-			gtk_color_selection_set_color(GTK_COLOR_SELECTION(csd->csel), color);
+		if (gdk_color_parse(this_color, &gcolor)) {
+			gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(csd->csel), &gcolor);
 		} else {
 			this_color=NULL;
 		}
 	}
-	csd->csel_changed_id = gtk_signal_connect(GTK_OBJECT(csd->csel), "color-changed", G_CALLBACK(colsel_color_changed), csd);
+	if (bfwin && bfwin->session->colorlist) {
+		GtkSettings* gtksettings;
+/*		GdkColor *gcolorarr=NULL;
+		gint num=0;*/
+		gchar *strings = stringlist_to_string(bfwin->session->colorlist, ":");
+		strings[strlen(strings)-1] = '\0';
+		gtksettings = gtk_widget_get_settings(GTK_WIDGET(csd->csel));
+		g_object_set(G_OBJECT(gtksettings), "gtk-color-palette", strings, NULL); 
+/*		DEBUG_MSG("colsel_dialog, setting palette from %s\n", strings);
+		if (gtk_color_selection_palette_from_string(strings, &gcolorarr, &num)) {
+			DEBUG_MSG("num=%d, gcolorarr=%p\n",num,gcolorarr);
+		} else {
+			DEBUG_MSG("hmm, failed to parse our string :(\n");
+		} */
+		g_free(strings);
+	}
+	gtk_color_selection_set_has_palette(GTK_COLOR_SELECTION(csd->csel), TRUE);
 	gtk_box_pack_start(GTK_BOX(vbox), csd->csel, TRUE, TRUE, 0);
+/*	
+	csd->csel_changed_id = gtk_signal_connect(GTK_OBJECT(csd->csel), "color-changed", G_CALLBACK(colsel_color_changed), csd);
+	
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 12);
@@ -1175,7 +1125,7 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	csd->hexentry = boxed_entry_with_text(this_color, 7, hbox);
 	csd->hex_changed_id = gtk_signal_connect(GTK_OBJECT(csd->hexentry), "changed", G_CALLBACK(hexentry_color_changed), csd);
 	csd->websafe = boxed_checkbut_with_value(_("_websafe"), 0, hbox);
-
+*/
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_hseparator_new(), TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 12);
@@ -1191,10 +1141,15 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	gtk_window_set_default(GTK_WINDOW(csd->win), but);
 	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
 	gtk_widget_show_all(csd->win);
+
 	DEBUG_MSG("colsel_dialog, finished\n");
 	return csd;
 }
 
+void edit_color_dialog(Tdocument *doc, gchar *color, gint startpos, gint endpos) {
+	colsel_dialog(doc->bfwin, color, FALSE, startpos, endpos);
+}
+
 void sel_colour_cb(GtkWidget *widget, Tbfwin *bfwin) {
 	Tcolsel *csd;
 	gchar *tmpstr=NULL;
