@@ -58,22 +58,35 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag *imdg) {
 		filename = gtk_editable_get_chars(GTK_EDITABLE(imdg->dg->entry[0]), 0, -1);
 		thumbnailfilename = create_thumbnail_filename(filename);
 		{
+			gchar *fullthumbfilename, *basedir = NULL;
 			gint w,h;
 			GError *error=NULL;
 			GdkPixbuf *tmp_im;
+			
+			/* the filename and thumbnailfilename can be relative paths to the current document */
+			if (filename[0] != '/' && imdg->dg->doc->filename && strlen(imdg->dg->doc->filename)) {
+				basedir = path_get_dirname_with_ending_slash(imdg->dg->doc->filename);
+			} else if (filename[0] == '/') {
+				basedir = path_get_dirname_with_ending_slash(filename);
+			}
+			/* we should use the full path to create the thumbnail filename */
+			fullthumbfilename = create_full_path(thumbnailfilename, basedir);
+			g_free(basedir);
+			DEBUG_MSG("image_insert_dialogok_lcb, thumbnail will be stored at %s\n", fullthumbfilename);
 			w = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(imdg->dg->spin[0]));
 			h = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(imdg->dg->spin[1]));
 			tmp_im = gdk_pixbuf_scale_simple(imdg->pb, w, h, GDK_INTERP_BILINEAR);
 			if (strcmp(main_v->props.image_thumbnailtype, "jpeg")==0) {
-	 			gdk_pixbuf_save(tmp_im,thumbnailfilename,main_v->props.image_thumbnailtype,&error, "quality", "50",NULL);
+	 			gdk_pixbuf_save(tmp_im,fullthumbfilename,main_v->props.image_thumbnailtype,&error, "quality", "50",NULL);
 			} else {
-				gdk_pixbuf_save(tmp_im,thumbnailfilename,main_v->props.image_thumbnailtype,&error, NULL);
+				gdk_pixbuf_save(tmp_im,fullthumbfilename,main_v->props.image_thumbnailtype,&error, NULL);
 			}
 			g_object_unref (tmp_im);
 			if (error) {
 				g_print("ERROR while saving thumbnail: %s\n", error->message);
 				g_error_free(error);
 			}
+			g_free(fullthumbfilename);
 		}
 		thestring = g_strconcat(cap("<A HREF=\""),filename, cap("\"><IMG SRC=\""), thumbnailfilename, "\"", NULL);
 		g_free(filename);
@@ -83,18 +96,20 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag *imdg) {
 		thestring = insert_string_if_entry(imdg->dg->entry[0], cap("SRC"), thestring, NULL);
 	}
 
-	thestring = insert_integer_if_spin(imdg->dg->spin[0], cap("WIDTH"), thestring, imdg->dg->check[0]);
-	thestring = insert_integer_if_spin(imdg->dg->spin[1], cap("HEIGHT"), thestring, imdg->dg->check[1]);
-	thestring = insert_integer_if_spin(imdg->dg->spin[2], cap("BORDER"), thestring, NULL);
-	thestring = insert_integer_if_spin(imdg->dg->spin[3], cap("HSPACE"), thestring, NULL);
-	thestring = insert_integer_if_spin(imdg->dg->spin[4], cap("VSPACE"), thestring, NULL);
+	thestring = insert_integer_if_spin(imdg->dg->spin[0], cap("WIDTH"), thestring, GTK_TOGGLE_BUTTON(imdg->dg->check[0])->active,0);
+	thestring = insert_integer_if_spin(imdg->dg->spin[1], cap("HEIGHT"), thestring, GTK_TOGGLE_BUTTON(imdg->dg->check[1])->active,0);
+	if (!main_v->props.xhtml) {
+		thestring = insert_integer_if_spin(imdg->dg->spin[2], cap("BORDER"), thestring, FALSE, 1);
+	}
+	thestring = insert_integer_if_spin(imdg->dg->spin[3], cap("HSPACE"), thestring, FALSE,0);
+	thestring = insert_integer_if_spin(imdg->dg->spin[4], cap("VSPACE"), thestring, FALSE,0);
 	thestring = insert_string_if_entry(imdg->dg->entry[1], cap("NAME"), thestring, NULL);
 	thestring = insert_string_if_entry(imdg->dg->entry[2], cap("ALT"), thestring, "");
 	thestring = insert_string_if_entry(imdg->dg->entry[3], cap("USEMAP"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(imdg->dg->combo[0])->entry), cap("ALIGN"), thestring, NULL);
 	thestring = insert_string_if_entry(imdg->dg->entry[4], NULL, thestring, NULL);
 
-	finalstring = g_strdup_printf(main_v->props.xhtml == 1 ? "%s />" : "%s>", thestring);
+	finalstring = g_strconcat(thestring, (main_v->props.xhtml == 1) ? " />" : ">", NULL);
 	g_free(thestring);
 
 	if (imdg->dg->range.end == -1) {
@@ -139,9 +154,29 @@ static void image_filename_changed(GtkWidget * widget, Timage_diag *imdg) {
 	if (imdg->pb) {
 		g_object_unref(imdg->pb);
 	}
+	DEBUG_MSG("image_filename_changed: filename=%s\n",gtk_entry_get_text(GTK_ENTRY(imdg->dg->entry[0])));
 	
-	imdg->pb = gdk_pixbuf_new_from_file(gtk_entry_get_text(GTK_ENTRY(imdg->dg->entry[0])), NULL);
-	
+	/* the entry usually has a relative filename, so we should make it absolute 
+	using the basedir of the document */
+	{
+		const gchar *filename;
+		gchar *fullfilename = NULL;
+
+		filename = gtk_entry_get_text(GTK_ENTRY(imdg->dg->entry[0]));
+		/* we should use the full path to create the thumbnail filename */
+		if (filename[0] != '/' && imdg->dg->doc->filename && strlen(imdg->dg->doc->filename)) {
+			gchar *basedir = path_get_dirname_with_ending_slash(imdg->dg->doc->filename);
+			fullfilename = create_full_path(filename, basedir);
+			g_free(basedir);
+		} else if (filename[0] == '/') {
+			fullfilename = g_strdup(filename);
+		} else {
+			return;
+		}
+		DEBUG_MSG("image_filename_changed: fullfilename=%s, loading!\n",fullfilename);
+		imdg->pb = gdk_pixbuf_new_from_file(fullfilename, NULL);
+		g_free(fullfilename);
+	}
 	if (!imdg->pb) {
 		return;
 	}
@@ -232,7 +267,6 @@ void image_insert_dialog_backend(gchar *filename,Tbfwin *bfwin, Ttagpopup *data,
 		static gchar *tagitems[] = { "width", "height", "alt", "border", "src", "hspace", "vspace", "align", "name", "usemap", NULL };
 		fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, imdg->dg);
 	}
-
 	imdg->frame = gtk_frame_new(_("Preview"));
 /*	gtk_widget_set_usize(imdg->frame, -1, 50);*/
 	gtk_box_pack_start(GTK_BOX(imdg->dg->vbox), imdg->frame, TRUE, TRUE, 0);
@@ -305,11 +339,11 @@ void image_insert_dialog_backend(gchar *filename,Tbfwin *bfwin, Ttagpopup *data,
 	g_list_free(popuplist);
 	bf_mnemonic_label_tad_with_alignment(_("_Align:"), imdg->dg->combo[0], 0, 0.5, dgtable, 3, 4, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->combo[0], 4, 6, 1, 2);
-
-	imdg->dg->spin[2] = spinbut_with_value(tagvalues[3], 0, 500, 1.0, 5.0);
-	bf_mnemonic_label_tad_with_alignment(_("Borde_r:"), imdg->dg->spin[2], 0, 0.5, dgtable, 3, 4, 2, 3);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[2], 4, 6, 2, 3);
-
+	if (!main_v->props.xhtml) {
+		imdg->dg->spin[2] = spinbut_with_value(tagvalues[3], 0, 500, 1.0, 5.0);
+		bf_mnemonic_label_tad_with_alignment(_("Borde_r:"), imdg->dg->spin[2], 0, 0.5, dgtable, 3, 4, 2, 3);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[2], 4, 6, 2, 3);
+	}
 	if (filename || tagvalues[4]) {
 		g_signal_emit_by_name(G_OBJECT(imdg->dg->entry[0]), "changed");
 	}
@@ -346,7 +380,7 @@ static void multi_thumbnail_dialog_destroy(GtkWidget *wid, Tmuthudia *mtd) {
 }
 
 static void multi_thumbnail_ok_clicked(GtkWidget *widget, Tmuthudia *mtd) {
-	GList *files, *tmplist;
+	GList *files=NULL, *tmplist;
 	gchar *string2insert=g_strdup("");
 
 	gtk_widget_hide(mtd->win);
@@ -370,7 +404,26 @@ static void multi_thumbnail_ok_clicked(GtkWidget *widget, Tmuthudia *mtd) {
 			main_v->props.image_thumnailformatstring = tmp;
 		}
 	}
+#ifdef HAVE_ATLEAST_GTK_2_4
+	{
+		GtkWidget *dialog;
+		/*dialog = gtk_file_chooser_dialog_new (_("Select files for thumbnail creation"),NULL,
+				GTK_FILE_CHOOSER_ACTION_OPEN,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+				NULL);
+		gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);*/
+		dialog = file_chooser_dialog(mtd->bfwin, _("Select files for thumbnail creation"), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, TRUE, TRUE, "webimage");
+		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+			GSList *slist = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
+			files = glist_from_gslist(slist);
+			g_slist_free(slist);
+		}
+		gtk_widget_destroy (dialog);
+	}
+#else
 	files = return_files_w_title(NULL, _("Select files for thumbnail creation"));
+#endif
 	tmplist = g_list_first(files);
 	while (tmplist) {
 		GdkPixbuf *tmp_im1, *tmp_im2;
@@ -380,8 +433,9 @@ static void multi_thumbnail_ok_clicked(GtkWidget *widget, Tmuthudia *mtd) {
 	
 		if (mtd->bfwin->current_document->filename) {
 			relfilename = create_relative_link_to(mtd->bfwin->current_document->filename, filename);
+			DEBUG_MSG("create_relative_link_to, filename=%s relfilename=%s \n", filename, relfilename);
 		} else {
-			relfilename = g_path_get_basename(filename);
+			relfilename = g_strdup (filename);
 		}
 		thumbfilename = create_thumbnail_filename(relfilename);
 		g_free(relfilename);
@@ -459,7 +513,7 @@ static void multi_thumbnail_ok_clicked(GtkWidget *widget, Tmuthudia *mtd) {
 			}
 			g_free(table);
 			tmp = string2insert;
-			string2insert = g_strconcat(string2insert, str, NULL);
+			string2insert = g_strconcat(string2insert, str, "\n", NULL);
 			g_free(tmp);
 			g_free(str);
 		}
