@@ -105,7 +105,7 @@ void init_output_box(Tbfwin *bfwin, GtkWidget *vbox) {
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 		gtk_container_add(GTK_CONTAINER(scrolwin), ob->lview);
-		gtk_widget_set_usize(scrolwin, 150, 150);
+		gtk_widget_set_size_request(scrolwin, 150, 150);
 		gtk_box_pack_start(GTK_BOX(ob->hbox), scrolwin, TRUE, TRUE, 0);
 		g_signal_connect(G_OBJECT(ob->lview), "row-activated",G_CALLBACK(ob_lview_row_activated_lcb),ob);
 	}
@@ -116,7 +116,7 @@ void init_output_box(Tbfwin *bfwin, GtkWidget *vbox) {
 		gtk_widget_show(image);
 		gtk_container_add(GTK_CONTAINER(but), image);
 		gtk_container_set_border_width(GTK_CONTAINER(but), 0);
-		gtk_widget_set_usize(but, 16,16);
+		gtk_widget_set_size_request(but, 16,16);
 		g_signal_connect(G_OBJECT(but), "clicked", G_CALLBACK(output_box_close_clicked_lcb), ob);
 		gtk_box_pack_start(GTK_BOX(ob->hbox), vbox2, FALSE, FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 0);
@@ -192,7 +192,11 @@ static GList *run_command(Toutputbox *ob) {
 	tmpfile = create_secure_dir_return_filename();
 	command2 = g_strconcat(command1, " > ", tmpfile, " 2>&1", NULL);
 	DEBUG_MSG("run_command, should run %s now\n", command2);
-	change_dir(ob->bfwin->current_document->filename);
+	{
+		gchar *tmpstring = g_path_get_dirname(ob->bfwin->current_document->filename);
+		chdir(tmpstring);
+		g_free(tmpstring);
+	}
 	system(command2);
 	retlist = get_stringlist(tmpfile, NULL);
 	remove_secure_dir_and_filename(tmpfile);
