@@ -75,6 +75,7 @@ static void setup_bfwin_for_project(Tbfwin *bfwin) {
 	bfwin->session = bfwin->project->session;
 	bfwin->bookmarkstore = bfwin->project->bookmarkstore;
 	bmark_set_store(bfwin);
+	bmark_reload(bfwin);
 	filebrowser_set_basedir(bfwin, bfwin->project->basedir);
 	recent_menu_from_list(bfwin, bfwin->project->session->recent_files, FALSE);
 	set_project_menu_widgets(bfwin, TRUE);
@@ -95,6 +96,34 @@ static Tproject *create_new_project(Tbfwin *bfwin) {
 		DEBUG_MSG("create_new_project, new project, no bfwin\n");
 	}
 	prj->session = g_new0(Tsessionvars,1);
+	/* we should copy bookmarks from the files to this session */
+	if (bfwin && prj->files) {
+		GList *tmplist;
+		tmplist = g_list_first(bfwin->documentlist);
+		while (tmplist) {
+			bmark_clean_for_doc(DOCUMENT(tmplist->data));
+			tmplist = g_list_next(tmplist);
+		}
+	
+		tmplist = g_list_first(bfwin->session->bmarks);
+		while (tmplist) {
+			gchar **entry = (gchar**)tmplist->data;
+			if (count_array(entry) > 2) {
+				GList *tmplist2 = g_list_first(prj->files);
+				while (tmplist2) {
+					if (strcmp(tmplist2->data, entry[2])==0) {
+						/* move it out of the default session into this session */
+						bfwin->session->bmarks = g_list_remove_link(bfwin->session->bmarks,tmplist);
+						prj->session->bmarks = g_list_concat(prj->session->bmarks, tmplist);
+						/* no further filenames to check */
+						tmplist2 = g_list_last(tmplist2);
+					}
+					tmplist2 = g_list_next(tmplist2);
+				}
+			}
+			tmplist = g_list_next(tmplist);
+		}
+	}
 	if (prj->files) {
 		gint len;
 		gchar *somefile, *prefix;
@@ -138,14 +167,50 @@ gboolean project_save(Tbfwin *bfwin, gboolean save_as) {
 	
 	bfwin->project->session->searchlist = limit_stringlist(bfwin->project->session->searchlist, 10, TRUE);
 	bfwin->project->session->replacelist = limit_stringlist(bfwin->project->session->replacelist, 10, TRUE);
+	bfwin->project->session->urllist = limit_stringlist(bfwin->project->session->urllist, 10, TRUE);
 	bmark_store_all(bfwin);
 
 	if (save_as || bfwin->project->filename == NULL) {
-		gchar *filename = return_file_w_title(NULL, _("Enter Bluefish project filename"));
 		gint suflen,filen;
+		gchar *filename = NULL;
+#ifdef HAVE_ATLEAST_GTK_2_4
+		{
+			GtkWidget *dialog;
+/*			dialog = gtk_file_chooser_dialog_new(_("Enter Bluefish project filename"),NULL,
+					GTK_FILE_CHOOSER_ACTION_OPEN,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					NULL);*/
+			dialog = file_chooser_dialog(bfwin,_("Enter Bluefish project filename"),GTK_FILE_CHOOSER_ACTION_SAVE, NULL, TRUE, FALSE, "bfproject");
+			if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+				filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+			}
+			gtk_widget_destroy(dialog);
+		}
+#else
+		filename = return_file_w_title(NULL, _("Enter Bluefish project filename"));
+#endif
 		if (!filename) {
 			return FALSE;
 		}
+		if (save_as || bfwin->project->filename == NULL) {
+			gchar *ondiskencoding = get_filename_on_disk_encoding(filename);
+			if (g_file_test(ondiskencoding, G_FILE_TEST_EXISTS)) {
+				gchar *tmpstr;
+				gint retval;
+				gchar *options[] = {_("_Cancel"), _("_Overwrite"), NULL};
+				tmpstr = g_strdup_printf(_("A file named \"%s\" already exists."), filename);
+				retval = multi_warning_dialog(bfwin->main_window,tmpstr, 
+												_("Do you want to replace the existing file?"), 1, 0, options);
+				g_free(tmpstr);
+				if (retval == 0) {
+					g_free(filename);
+					g_free(ondiskencoding);
+					return FALSE;
+				}
+			}
+			g_free(ondiskencoding);
+		}
 		suflen = strlen(main_v->props.project_suffix);
 		filen = strlen(filename);
 		if (filen > suflen && strcmp(&filename[filen - suflen], main_v->props.project_suffix)==0) {
@@ -163,12 +228,10 @@ gboolean project_save(Tbfwin *bfwin, gboolean save_as) {
 }
 
 void set_project_menu_widgets(Tbfwin *bfwin, gboolean win_has_project) {
-	GtkItemFactory * ifactory;
-	ifactory = gtk_item_factory_from_widget(bfwin->menubar);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(ifactory, _("/Project/Save")), win_has_project);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(ifactory, _("/Project/Save as...")), win_has_project);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(ifactory, _("/Project/Save & close")), win_has_project);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(ifactory, _("/Project/Edit Project Options...")), win_has_project);
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Save", win_has_project);
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Save as...", win_has_project);
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Save & close", win_has_project);
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Edit Project Options...", win_has_project);
 }
 
 void project_open_from_file(Tbfwin *bfwin, gchar *fromfilename) {
@@ -229,7 +292,24 @@ void project_open_from_file(Tbfwin *bfwin, gchar *fromfilename) {
 
 static void project_open(Tbfwin *bfwin) {
 	/* first we ask for a filename */
-	gchar *filename = return_file_w_title(NULL, _("Select Bluefish project filename"));
+	gchar *filename = NULL;
+#ifdef HAVE_ATLEAST_GTK_2_4
+	{
+		GtkWidget *dialog;
+		/*dialog = gtk_file_chooser_dialog_new(_("Select Bluefish project filename"),NULL,
+				GTK_FILE_CHOOSER_ACTION_OPEN,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+				NULL);*/
+		dialog = file_chooser_dialog(bfwin, _("Select Bluefish project filename"), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, TRUE, FALSE, "bfproject");
+		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+			filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		}
+		gtk_widget_destroy(dialog);
+	}
+#else
+	filename = return_file_w_title(NULL, _("Select Bluefish project filename"));
+#endif
 	if (filename) {
 		DEBUG_MSG("project_open, for filename %s\n",filename);
 		project_open_from_file(bfwin,filename);
@@ -262,21 +342,61 @@ static void setup_bfwin_for_nonproject(Tbfwin *bfwin) {
 	set_project_menu_widgets(bfwin, FALSE);
 }
 
+/* 
+ * returns TRUE if the project is closed, 
+ * returns FALSE if something went wrong or was cancelled
+ */
 gboolean project_save_and_close(Tbfwin *bfwin) {
-	if (project_save(bfwin, FALSE)) {
-		DEBUG_MSG("project_save_and_close, save returned TRUE\n");
-		file_close_all_cb(NULL,bfwin);
-		if (test_only_empty_doc_left(bfwin->documentlist)) {
-			DEBUG_MSG("project_save_and_close, all documents are closed\n");
-			add_to_recent_list(bfwin,bfwin->project->filename, TRUE, TRUE);
-			project_destroy(bfwin->project);
-			setup_bfwin_for_nonproject(bfwin);
-			DEBUG_MSG("project_save_and_close, returning TRUE\n");
-			return TRUE;
+	gboolean dont_save = FALSE;
+	while (!bfwin->project->filename) {
+		gchar *text;
+		gint retval;
+		gchar *buttons[] = {_("Do_n't save"), GTK_STOCK_CANCEL, GTK_STOCK_SAVE, NULL};
+		if (dont_save) {
+			break;
 		}
+		DEBUG_MSG("project_save_and_close, project not named, getting action\n");
+		/* dialog */
+
+		text = g_strdup(_("Do you want to save the project?"));
+		retval = multi_query_dialog(bfwin->main_window, text, 
+			_("If you don't save your changes they will be lost."), 2, 1, buttons);
+		switch (retval) {
+		case 0:
+			/* don't save proj. save files, though */
+			DEBUG_MSG("project_save_and_close, don't save project, but save files\n");
+			dont_save = TRUE;
+		break;
+		case 1:
+			/* cancel */
+			DEBUG_MSG("project_save_and_close, not closing window any more");
+			return FALSE;
+		break;
+		case 2:
+			DEBUG_MSG("project_save_and_close, bringing up save project dialog\n");
+			dont_save = project_save(bfwin, FALSE);
+		break;
+		default:
+		break;
+		}		
 	}
-	DEBUG_MSG("project_save_and_close, returning FALSE\n");
-	return FALSE;
+	/* test if we should save */
+	if (!dont_save) {
+		if (!project_save(bfwin, FALSE)) {
+			DEBUG_MSG("project_save failed, returning\n");
+			return FALSE;
+		}
+		add_to_recent_list(bfwin,bfwin->project->filename, TRUE, TRUE);
+	}
+	bfwin_close_all_documents(bfwin, TRUE);
+	if (!test_only_empty_doc_left(bfwin->documentlist)) {
+		DEBUG_MSG("closing all documents failed, returning\n");
+		return FALSE;
+	}
+	project_destroy(bfwin->project);
+	setup_bfwin_for_nonproject(bfwin);
+	DEBUG_MSG("project_save_and_close, returning TRUE\n");
+	return TRUE;
 }
 
 typedef enum {
