@@ -22,10 +22,12 @@
 #define __RCFILE_H_
 void rcfile_parse_main(void);
 void rcfile_parse_highlighting(void);
-void rcfile_parse_custom_menu(void);
+void rcfile_parse_custom_menu(gboolean full_reset, gboolean load_new);
 void rcfile_check_directory(void);
 void rcfile_save_configfile_menu_cb(gpointer callback_data,guint action,GtkWidget *widget);
 void rcfile_save_all(void);
 gboolean rcfile_parse_project(Tproject *project, gchar *filename);
 gboolean rcfile_save_project(Tproject *project, gchar *filename);
+gboolean rcfile_save_global_session(void);
+gboolean rcfile_parse_global_session(void);
 #endif /* __RCFILE_H_ */
