@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) 2003-2004 Olivier Sessink
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 #include <gtk/gtk.h>
 #include <string.h> /* strcasecmp() */
 
@@ -21,16 +38,17 @@ typedef struct {
 
 typedef struct {
 	Tdocument *doc;
-	gint tag_so;
-	gint tag_eo;
-	gboolean found_tag;
-} Trecent_tag;
+	gint so;
+	gint eo;
+	gboolean found;
+} Tevent_substring;
 
 /********************************/
 /* a global var for this module */
 /********************************/
 
-Trecent_tag rec_tag;
+Tevent_substring rec_tag;
+Tevent_substring rec_color;
 
 /*****************/
 /* the functions */
@@ -38,7 +56,6 @@ Trecent_tag rec_tag;
 
 static gboolean iter_char_search_lcb(gunichar ch,Tin_html_tag *iht){
 /* TODO: a </tag> should not count as editable tag!!! */
-
 	if (ch==iht->findchar && iht->prevchar != iht->ignore_if_prevchar) {
 		return TRUE;
 	}
@@ -46,75 +63,103 @@ static gboolean iter_char_search_lcb(gunichar ch,Tin_html_tag *iht){
 	return FALSE;
 }
 
-/* TODO: a </tag> should not count as editable tag!!! */
-gboolean doc_bevent_in_html_tag(Tdocument *doc, GdkEventButton *bevent) {
-	gint xpos, ypos;
-	GtkTextWindowType wintype;
-	GtkTextIter iter;
+static gboolean locate_color(Tdocument *doc, const GtkTextIter *iter) {
+	Tin_html_tag iht;
+	gboolean retval=FALSE;
+	GtkTextIter leftiter=*iter, rightiter, maxiter = *iter;
+	DEBUG_MSG("locate_color, started\n");
+	rec_color.found = FALSE;
+	gtk_text_iter_backward_chars(&maxiter, 8);
+	/* first we look to the left for a #, and we look back at max. 8 chars (7 would be good enough) */
+	iht.findchar = '#';
+	iht.prevchar = '\n';
+	iht.ignore_if_prevchar = '\0';
+	if (gtk_text_iter_backward_find_char(&leftiter,
+					(GtkTextCharPredicate)iter_char_search_lcb,&iht,&maxiter)) 
+					{
+		gchar *text;
+		rightiter = leftiter;
+		gtk_text_iter_forward_chars(&rightiter, 7);
+		text = gtk_text_buffer_get_text(doc->buffer,&leftiter,&rightiter,FALSE);
+		DEBUG_MSG("locate_color,is '%s' a color?\n",text);
+		if (text) {
+			retval = string_is_color(text);
+			if (retval) {
+				rec_color.so = gtk_text_iter_get_offset(&leftiter);
+				rec_color.eo = gtk_text_iter_get_offset(&rightiter);
+				rec_color.found = TRUE;
+				rec_color.doc = doc;
+				DEBUG_MSG("found color from so=%d to eo=%d\n",rec_color.so, rec_color.eo);
+			}
+			g_free(text);
+		}
+	}
+	return retval;
+}
 
-	wintype = gtk_text_view_get_window_type(GTK_TEXT_VIEW(doc->view), doc->view->window);
-	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(doc->view), wintype,bevent->x, bevent->y,
-					  &xpos, &ypos);
+static gboolean locate_current_tag(Tdocument *doc, const GtkTextIter *iter) {
+	GtkTextIter gtiter, ltiter;
+	gboolean ltfound, gtfound;
+	Tin_html_tag iht;
+	gtiter = ltiter = *iter;
 
-	rec_tag.tag_so = rec_tag.tag_eo = -1;
+	rec_tag.found = FALSE;
+
+	/* backward search for tag start */
+	iht.findchar = '>';
+	iht.prevchar = '\n';
+	iht.ignore_if_prevchar = 0;
+	gtfound = gtk_text_iter_backward_find_char(&gtiter,(GtkTextCharPredicate)iter_char_search_lcb,
+                                          &iht,NULL);
+	/* perhaps we should limit the search fto 50 charcters or so */
+	iht.findchar = '<';
+	iht.prevchar = '\n';
+	iht.ignore_if_prevchar = '?';
+	ltfound = gtk_text_iter_backward_find_char(&ltiter,(GtkTextCharPredicate)iter_char_search_lcb,
+                                          &iht,NULL);
+	rec_tag.so = rec_tag.eo = -1;
 	rec_tag.doc = doc;
-	
-	DEBUG_MSG("doc_bevent_in_html_tag, xpos=%d, ypos=%d\n", xpos, ypos);
-	gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(doc->view), &iter, xpos, ypos);
-	{
-		GtkTextIter gtiter, ltiter;
-		gboolean ltfound, gtfound;
-		Tin_html_tag iht;
-		gtiter = ltiter = iter;
+	if ((ltfound && gtfound && gtk_text_iter_compare(&ltiter,&gtiter) > 0)
+			|| (ltfound && !gtfound)) {
+		rec_tag.so = gtk_text_iter_get_offset(&ltiter);
+		DEBUG_MSG("a tag is started on the left side at %d\n",rec_tag.so);
+	} else {
+		DEBUG_MSG("no tag start found on the left side\n");
+	}
 
-		/* backward search for tag start */
+	if (rec_tag.so >=0) {
+		/* forward search for end tag */
 		iht.findchar = 62;/* 62 = > */
 		iht.prevchar = 10; /* \n */
-		iht.ignore_if_prevchar = 0;
-		gtfound = gtk_text_iter_backward_find_char(&gtiter,(GtkTextCharPredicate)iter_char_search_lcb,
-                                             &iht,NULL);
+		iht.ignore_if_prevchar = 63 /* ? */;
+		gtfound = gtk_text_iter_forward_find_char(&gtiter,(GtkTextCharPredicate)iter_char_search_lcb,
+                                          &iht,NULL);
 		/* perhaps we should limit the search fto 50 charcters or so */
 		iht.findchar = 60;/* 60 = < */
 		iht.prevchar = 10; /* \n */
-		iht.ignore_if_prevchar = 63 /* ? */;
-		ltfound = gtk_text_iter_backward_find_char(&ltiter,(GtkTextCharPredicate)iter_char_search_lcb,
-                                             &iht,NULL);
+		iht.ignore_if_prevchar = 0;
+		ltfound = gtk_text_iter_forward_find_char(&ltiter,(GtkTextCharPredicate)iter_char_search_lcb,
+                                          &iht,NULL);
 		if ((ltfound && gtfound && gtk_text_iter_compare(&ltiter,&gtiter) > 0)
-				|| (ltfound && !gtfound)) {
-			rec_tag.tag_so = gtk_text_iter_get_offset(&ltiter);
-			DEBUG_MSG("a tag is started on the left side at %d\n",rec_tag.tag_so);
+			|| (gtfound && !ltfound)) {
+			rec_tag.eo = gtk_text_iter_get_offset(&gtiter)+1;
+			DEBUG_MSG("a tag is ended on the right side at %d\n",rec_tag.eo);
+			rec_tag.found = TRUE;
+			return TRUE;
 		} else {
-			DEBUG_MSG("no tag start found on the left side\n");
-		}
-
-		if (rec_tag.tag_so >=0) {
-			/* forward search for end tag */
-			iht.findchar = 62;/* 62 = > */
-			iht.prevchar = 10; /* \n */
-			iht.ignore_if_prevchar = 63 /* ? */;
-			gtfound = gtk_text_iter_forward_find_char(&gtiter,(GtkTextCharPredicate)iter_char_search_lcb,
-                                             &iht,NULL);
-			/* perhaps we should limit the search fto 50 charcters or so */
-			iht.findchar = 60;/* 60 = < */
-			iht.prevchar = 10; /* \n */
-			iht.ignore_if_prevchar = 0;
-			ltfound = gtk_text_iter_forward_find_char(&ltiter,(GtkTextCharPredicate)iter_char_search_lcb,
-                                             &iht,NULL);
-			if ((ltfound && gtfound && gtk_text_iter_compare(&ltiter,&gtiter) > 0)
-				|| (gtfound && !ltfound)) {
-				rec_tag.tag_eo = gtk_text_iter_get_offset(&gtiter)+1;
-				DEBUG_MSG("a tag is ended on the right side at %d\n",rec_tag.tag_eo);
-				rec_tag.found_tag = TRUE;
-				return TRUE;
-			} else {
-				DEBUG_MSG("no tag end found on the right side\n");
-			}
+			DEBUG_MSG("no tag end found on the right side\n");
 		}
 	}
 	return FALSE;
 }
 
-static void input_tag_splitter(gpointer nothing, gpointer data)
+/* TODO: a </tag> should not count as editable tag!!! */
+void rpopup_bevent_in_html_code(Tdocument *doc, GtkTextIter *iter) {
+	locate_current_tag(doc, iter);
+	locate_color(doc, iter);
+}
+/*
+static void input_tag_splitter(Tbfwin *bfwin, gpointer data)
 {
 	Ttagpopup *tag_popup;
 	GList *tmplist;
@@ -122,7 +167,7 @@ static void input_tag_splitter(gpointer nothing, gpointer data)
 	gchar *itemvalue = NULL;
 
 	tag_popup = (Ttagpopup *) data;
-	DEBUG_MSG("input_tag_splitter, started\n");
+	DEBUG_MSG("input_tag_splitter, started for tag_popup=%p\n",tag_popup);
 
 	tmplist = g_list_first(tag_popup->taglist);
 	while (tmplist) {
@@ -142,26 +187,26 @@ static void input_tag_splitter(gpointer nothing, gpointer data)
 		g_strdown(itemvalue);
 		DEBUG_MSG("input_tag_splitter, itemvalue=%s, counter=%d\n", itemvalue, counter);
 		if (strcmp(itemvalue, "submit") == 0) {
-			buttondialog_dialog(NULL, data);
+			buttondialog_dialog(bfwin, data);
 		} else if (strcmp(itemvalue, "reset") == 0) {
-			buttondialog_dialog(NULL, data);
+			buttondialog_dialog(bfwin, data);
 		} else if (strcmp(itemvalue, "button") == 0) {
-			buttondialog_dialog(NULL, data);
+			buttondialog_dialog(bfwin, data);
 		} else if (strcmp(itemvalue, "text") == 0) {
-			textdialog_dialog(NULL, data);
+			textdialog_dialog(bfwin, data);
 		} else if (strcmp(itemvalue, "hidden") == 0) {
-			hiddendialog_dialog(NULL, data);
+			hiddendialog_dialog(bfwin, data);
 		} else if (strcmp(itemvalue, "radio") == 0) {
-			radiodialog_dialog(NULL, data);
+			radiodialog_dialog(bfwin, data);
 		} else if (strcmp(itemvalue, "checkbox") == 0) {
-			checkdialog_dialog(NULL, data);
+			checkdialog_dialog(bfwin, data);
 		}
 		g_free(itemvalue);
 	} else {
 		DEBUG_MSG("input_tag_splitter, not found ?\n");
 	}
 }
-
+*/
 
 typedef struct {
 	gchar *tag;
@@ -202,7 +247,10 @@ static void parse_tagstring(Tbfwin *bfwin, gchar * tagstring, gint pos, gint end
 		{"select",G_CALLBACK(selectdialog_dialog)},
 		{"option",G_CALLBACK(optiondialog_dialog)},
 		{"meta",G_CALLBACK(meta_dialog)},
-		{"img",G_CALLBACK(image_insert_dialog)}
+		{"img",G_CALLBACK(image_insert_dialog)},
+		{"input",G_CALLBACK(inputdialog_rpopup)},
+		{"button",G_CALLBACK(buttondialog_dialog)},
+		{"link",G_CALLBACK(linkdialog_dialog)}
 	};
 
 	DEBUG_MSG("parse_tagstring, started, tagstring=%s\n", tagstring);
@@ -335,10 +383,10 @@ static void parse_tagstring(Tbfwin *bfwin, gchar * tagstring, gint pos, gint end
 	if (strcmp(tmpstring, "link") == 0) {
 		link_cb(NULL, tag_popup);
 	} else*/
-	if (strcmp(tmpstring, "input") == 0) {
+/*	if (strcmp(tmpstring, "input") == 0) {
 		DEBUG_MSG("parse_tagstring, identified as INPUT tag, splitting tag!\n");
-		input_tag_splitter(NULL, tag_popup);
-	}
+		input_tag_splitter(bfwin, tag_popup);
+	}*/
 	tmplist = g_list_first(tmplist);
 	while (tmplist) {
 		g_free(((Ttagitem *) tmplist->data)->item);
@@ -351,17 +399,48 @@ static void parse_tagstring(Tbfwin *bfwin, gchar * tagstring, gint pos, gint end
 	g_free(tmpstring);
 }
 
+gboolean rpopup_doc_located_tag(Tdocument *doc) {
+	return (rec_tag.doc == doc && rec_tag.found);
+}
+
+gboolean rpopup_doc_located_color(Tdocument *doc) {
+	return (rec_color.doc == doc && rec_color.found);
+}
 
 void rpopup_edit_tag_cb(GtkMenuItem *menuitem,Tdocument *doc) {
-	if (rec_tag.doc == doc && rec_tag.found_tag) {
+	if (rpopup_doc_located_tag(doc)) {
 		gchar *text;
-		DEBUG_MSG("rpopup_edit_tag_cb, get text from %d to %d\n", rec_tag.tag_so, rec_tag.tag_eo);
+		DEBUG_MSG("rpopup_edit_tag_cb, get text from %d to %d\n", rec_tag.so, rec_tag.eo);
 		/* we do not need the < and > chars so we cut those */
-		text = doc_get_chars(doc, rec_tag.tag_so+1, rec_tag.tag_eo - 1);
+		text = doc_get_chars(doc, rec_tag.so+1, rec_tag.eo - 1);
 		DEBUG_MSG("rpopup_edit_tag_cb, about to parse %s\n", text);
-		parse_tagstring(doc->bfwin, text, rec_tag.tag_so, rec_tag.tag_eo);
-		g_free(text);
+		if (text) {
+			parse_tagstring(doc->bfwin, text, rec_tag.so, rec_tag.eo);
+			g_free(text);
+		}
 	} else {
 		DEBUG_MSG("rpopup_edit_tag_cb, no tag search known!!\n");
 	}
 }
+
+void rpopup_edit_color_cb(GtkMenuItem *menuitem,Tdocument *doc) {
+	if (rpopup_doc_located_color(doc)) {
+		gchar *text;
+		text = doc_get_chars(doc, rec_color.so, rec_color.eo);
+		if (text) {
+			DEBUG_MSG("rpopup_edit_color_cb, using color '%s'\n",text);
+			edit_color_dialog(doc, text, rec_color.so, rec_color.eo);
+			g_free(text);
+		}
+	}
+}
+
+void edit_tag_under_cursor_cb(Tbfwin *bfwin) {
+	GtkTextIter iter;
+	Tdocument *doc = bfwin->current_document;
+	GtkTextMark* imark = gtk_text_buffer_get_insert(doc->buffer);
+	gtk_text_buffer_get_iter_at_mark(doc->buffer,&iter,imark);
+	if (locate_current_tag(doc, &iter)) {
+		rpopup_edit_tag_cb(NULL, doc);
+	}
+}
