@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snr2.c - rewrite of search 'n replace functions
  *
- * Copyright (C) 2000-2002 Olivier Sessink
+ * Copyright (C) 2000-2004 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,19 +20,19 @@
 /*
  * THIS IS THE BACKEND FLOW DIAGRAM
  *
- *                         snr2_run
- *         ________________/ | \  \___________________________
- *        /        |         |  \                   \  \      \
- *       /         |         |  replace_prompt_all  |   \      \
- *      /          |         |         /           /     \      \
- *   search_all    |   _____replace_prompt_doc    /       \    replace_all
- *     \           |  /          /               /         \     /
- *      \   doc_show_result     /  replace_doc_once   replace_doc_multiple
- *       \                   __/                  \         /
- *     search_doc           /                   replace_backend
- *           \             / ___________________________/
- *            \           / /
- *             search_backend
+ *                              snr2_run
+ *              ________________/ | \  \___________________________
+ *             /      / |         |  \                   \  \      \
+ *  search_bookmark  |  |         |  replace_prompt_all  |   \      \
+ *     |            /   |         |         /           /     \      \
+ *     |   search_all   |   _____replace_prompt_doc    /       \    replace_all
+ *     |     |          |  /          /               /         \     /
+ *     |     \   doc_show_result     /  replace_doc_once   replace_doc_multiple
+ *     |      \                   __/                  \         /
+ *     |  search_doc             /                   replace_backend
+ *      \________ \             /  ___________________________/
+ *               \ \           /  /
+ *                 search_backend
  */
 /*****************************************************/
 /* #define DEBUG */
@@ -52,6 +52,7 @@
 #include <gdk/gdkkeysyms.h> /* GDK_Return */
 
 #include "bluefish.h"
+#include "bookmark.h" /* bmark_add_extern() */
 #include "bf_lib.h"
 #include "undo_redo.h" /* doc_unre_new_group */
 #include "document.h"			/* doc_replace_text() */
@@ -61,18 +62,6 @@
 #include "highlight.h" /* doc_highlight_full() */
 #include "stringlist.h" /* add_to_history_stringlist */
 
-/* Note:
-    With multibyte text support a byte-offset
-    of a string can not be used as a character-offset of GtkText.
-    The member "start" and "end" are character-offset.
-    The member "bstart" and "bend" are byte-offset in the source
-    buffer.
-    These values are set in search_backend().
-    In single-byte text mode,
-        start == bstart, end == bend .
-
-    Iwasa, updated 2002 by Olivier
-*/
 /* Updates, May 2003, by Ruben Dorta */
 
 typedef enum { string, uppercase, lowercase } Treplace_types;
@@ -97,6 +86,7 @@ typedef struct {
 	gint prompt_before_replace;
 	gint is_case_sens;
 	gint replace_once;
+	gint bookmark_results;
 	Treplace_types replacetype_option;
 	Tmatch_types matchtype_option;
 	Tplace_types placetype_option;
@@ -123,6 +113,7 @@ typedef struct {
 	GtkWidget *replacetype_option;
 	GtkWidget *matchtype_option;
 	GtkWidget *placetype_option;
+	GtkWidget *bookmark_results;
 } Tsnr2_win;
 #define LASTSNR2(var) ((Tlast_snr2 *)(var))
 /***********************************************************/
@@ -134,7 +125,7 @@ void snr2_run(Tbfwin *bfwin,Tdocument *doc);
 void snr2_init(Tbfwin *bfwin) {
 	Tlast_snr2 *lsnr2 = g_new0(Tlast_snr2,1);
 	lsnr2->bfwin = bfwin;
-	LASTSNR2(bfwin->snr2) = lsnr2;
+	bfwin->snr2 = lsnr2;
 }
 
 static void reset_last_snr2(Tbfwin *bfwin) {
@@ -156,6 +147,7 @@ static void reset_last_snr2(Tbfwin *bfwin) {
  * @matchtype: see #Tmatch_types
  * @is_case_sens: If the search is case sensitive, #gint
  * @buf: #gchar* to the document buffer
+ * @byte_offset: #guint where in the buffer the search should start, in bytes, not characters
  * @want_submatches: #gint
  * 
  * Performs an actual search in a supplied buffer (#gchar*, aka string).
@@ -163,7 +155,7 @@ static void reset_last_snr2(Tbfwin *bfwin) {
  *
  * Return value: #Tsearch_result, contains both character and byte offsets, for wide-char-compatibility. Note values for start/end are set to -1 on error.
  **/
-Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types matchtype, gint is_case_sens, gchar *buf, gboolean want_submatches) {
+Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types matchtype, gint is_case_sens, gchar *buf, guint byte_offset, gboolean want_submatches) {
 	Tsearch_result returnvalue;
 	int (*f) ();
 	gint buflen, patlen, match, i;
@@ -178,7 +170,7 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 		DEBUG_MSG("search_backend, search_pattern or buf is NULL\n");
 		return returnvalue;
 	}
-	
+	DEBUG_MSG("search_backend, starting for byte_offset=%u\n",byte_offset);
 	if (matchtype == match_posix) {
 		/* regex part start */
 		regex_t reg_pat;
@@ -202,11 +194,14 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 		nmatch = (want_submatches) ? reg_pat.re_nsub+1 : 1;
 		DEBUG_MSG("search_backend, expr. contains %d sub search_patterns\n", reg_pat.re_nsub );
 		pmatch = g_malloc(nmatch*sizeof(regmatch_t));
-		retval = regexec(&reg_pat, buf, nmatch, pmatch, 0);
+		retval = regexec(&reg_pat, buf+byte_offset, nmatch, pmatch, 0);
 		DEBUG_MSG("search_backend, regexec retval=%d\n", retval);
 		if (retval != 0) {
-			pmatch[0].rm_so = -1;
-			pmatch[0].rm_eo = -1;
+			returnvalue.bstart = -1;
+			returnvalue.bend = -1;
+		} else {
+			returnvalue.bstart = pmatch[0].rm_so + byte_offset;
+			returnvalue.bend = pmatch[0].rm_eo + byte_offset;
 		}
 #ifdef DEBUG
 		{	int i;
@@ -215,8 +210,6 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 			}
 		}
 #endif
-		returnvalue.bstart = pmatch[0].rm_so;
-		returnvalue.bend = pmatch[0].rm_eo;
 		regfree(&reg_pat);
 		if (want_submatches) {
 			returnvalue.pmatch = pmatch;
@@ -241,10 +234,10 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 			g_free(errstring);
 			return returnvalue;/* error compiling the search_pattern, returning the default result set,which is the 'nothing found' set */
 		}
-		retval = pcre_exec(pcre_c,NULL,buf,strlen(buf),0,0,ovector,30);
+		retval = pcre_exec(pcre_c,NULL,buf+byte_offset,strlen(buf+byte_offset),0,0,ovector,30);
 		if (retval > 0) {
-			returnvalue.bstart = ovector[0];
-			returnvalue.bend = ovector[1];
+			returnvalue.bstart = ovector[0] + byte_offset;
+			returnvalue.bend = ovector[1] + byte_offset;
 		} else {
 			returnvalue.bstart = -1;
 			returnvalue.bend = -1;
@@ -256,8 +249,8 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 			DEBUG_MSG("search_backend, nmatch=%d, retval=%d\n", nmatch, retval);
 			pmatch = g_malloc((nmatch+1)*sizeof(regmatch_t));
 			for (i=0;i<=nmatch;i++) { /* nmatch==1 means 1 subsearch_pattern, so 2 search_patterns in total*/
-				pmatch[i].rm_so = ovector[i*2];
-				pmatch[i].rm_eo = ovector[i*2+1];
+				pmatch[i].rm_so = ovector[i*2] + byte_offset;
+				pmatch[i].rm_eo = ovector[i*2+1] + byte_offset;
 			}
 			returnvalue.pmatch = pmatch;
 			returnvalue.nmatch = retval;
@@ -282,7 +275,7 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 		buflen = strlen(buf);
 		patlen = strlen(search_pattern);
 		
-		for (i = 0; i <= (buflen - patlen); i++) {
+		for (i = byte_offset; i <= (buflen - patlen); i++) {
 			match = f(&buf[i], search_pattern, patlen);
 			if (match == 0) {
 				returnvalue.bstart = i;
@@ -293,9 +286,9 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 		/* non regex part end */	
 	}
 	
-	/* for multibyte characters */
+	/* if we have a valid result, we now calculate the character offsets for this result */
 	if (returnvalue.bstart >= 0 && returnvalue.bend >= 0) {
-		utf8_offset_cache_reset();
+		/* utf8_offset_cache_reset(); */
 		if (returnvalue.bstart >= 0) {
 			returnvalue.start = utf8_byteoffset_to_charsoffset_cached(buf, returnvalue.bstart);
 		}
@@ -316,7 +309,7 @@ Tsearch_result search_backend(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types
 		returnvalue.bend = -1;
 	}
 
-	DEBUG_MSG("search_backend, returning result.start=%d, result.end=%d\n", returnvalue.start, returnvalue.end);
+	DEBUG_MSG("search_backend, returning result.start=%d, result.end=%d, bstart=%d, bend=%d\n", returnvalue.start, returnvalue.end, returnvalue.bstart, returnvalue.bend);
 	return returnvalue;
 }
 
@@ -341,6 +334,7 @@ Tsearch_result search_doc(Tbfwin *bfwin,Tdocument *document, gchar *search_patte
 	Tsearch_result result;
 	
 	DEBUG_MSG("search_doc, started on document %p, startpos=%d\n", document, startpos);
+	utf8_offset_cache_reset();
 	fulltext = doc_get_chars(document, startpos, -1);
 	DEBUG_MSG("search_doc, fulltext=%p, search_pattern=%p\n", fulltext, search_pattern);
 	if (unescape) {
@@ -348,7 +342,7 @@ Tsearch_result search_doc(Tbfwin *bfwin,Tdocument *document, gchar *search_patte
 	} else {
 		realpat = search_pattern;
 	}
-	result = search_backend(bfwin,realpat, matchtype, is_case_sens, fulltext, FALSE);
+	result = search_backend(bfwin,realpat, matchtype, is_case_sens, fulltext, 0, FALSE);
 	if (unescape) {
 		g_free(realpat);
 	}
@@ -408,7 +402,6 @@ void doc_show_result(Tdocument *document, gint start, gint end) {
 Tsearch_all_result search_all(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matchtype, gint is_case_sens, gboolean unescape) {
 	GList *tmplist;
 	Tsearch_all_result result_all;
-	Tsearch_result result;
 
 	DEBUG_MSG("search_all, started\n");
 	result_all.start = -1;
@@ -422,6 +415,8 @@ Tsearch_all_result search_all(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types
 		tmplist = g_list_first(bfwin->documentlist);
 	}
 	while (tmplist) {
+		Tsearch_result result;
+
 		result = search_doc(bfwin,(Tdocument *)tmplist->data, search_pattern, matchtype, is_case_sens, LASTSNR2(bfwin->snr2)->result.end, unescape);
 		if (result.end > 0) {
 			result_all.start = result.start;
@@ -445,12 +440,12 @@ static gchar *reg_replace(gchar *replace_pattern, gint offset, Tsearch_result re
 	Tconvert_table * tct;
 	gchar *retval;
 	gint i, size;
+	DEBUG_MSG("reg_replace, started for pattern='%s',standardescape=%d\n",replace_pattern,standardescape);
 	size = (result.nmatch <= 10) ? (result.nmatch == 0 ) ? 0 : result.nmatch -1 : 10;
 	tct = new_convert_table(size, standardescape);
 	for (i=0;i<size;i++) {
 		tct[i].my_int = i+48;
 		tct[i].my_char = doc_get_chars(doc, offset+result.pmatch[i+1].rm_so, offset+result.pmatch[i+1].rm_eo);
-		DEBUG_MSG("new_reg_replace, i=%d, my_int=%c, my_char=%s\n",i,tct[i].my_int,tct[i].my_char);
 	}
 	retval = expand_string(replace_pattern, '\\', tct);
 	free_convert_table(tct);
@@ -524,6 +519,7 @@ static gchar *reg_replace(gchar *replace_pattern, gint offset, Tsearch_result re
 
 /**
  * replace_backend:
+ * @bfwin: #Tbfwin with the window from this/these documents
  * @search_pattern: #gchar* to search pattern
  * @matchtype: See #Tmatch_types
  * @is_case_sens: #gint
@@ -540,7 +536,7 @@ static gchar *reg_replace(gchar *replace_pattern, gint offset, Tsearch_result re
  * Return value: #Tsearch_result
  **/
 Tsearch_result replace_backend(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matchtype, gint is_case_sens
-			, gchar *buf, gchar *replace_pattern, Tdocument *doc, gint offset, Treplace_types replacetype
+			, gchar *buf, guint byte_offset, gchar *replace_pattern, Tdocument *doc, gint offset, Treplace_types replacetype
 			, gint *replacelen, gboolean unescape) {
 /* the offset in this function is the difference between the buffer and the text widget because of previous replace 
 actions, so the first char in buf is actually number offset in the text widget */
@@ -550,11 +546,13 @@ actions, so the first char in buf is actually number offset in the text widget *
 	
 	if (unescape) {
 		realpat = unescape_string(search_pattern,FALSE);
+		DEBUG_MSG("replace_backend, realpat='%s'\n",realpat);
 	} else {
 		realpat = search_pattern;
 	}
-	result = search_backend(bfwin,realpat, matchtype, is_case_sens, buf, (matchtype != match_normal));
+	result = search_backend(bfwin,realpat, matchtype, is_case_sens, buf, byte_offset, (matchtype != match_normal));
 	if (unescape) {
+		DEBUG_MSG("replace_backend, free-ing realpat\n");
 		g_free(realpat);
 	}
 	DEBUG_MSG("replace_backend, offset=%d, result.start=%d, result.end=%d\n", offset, result.start, result.end);
@@ -563,6 +561,7 @@ actions, so the first char in buf is actually number offset in the text widget *
 		case string:
 			if (matchtype == match_normal) {
 				if (unescape) {
+					DEBUG_MSG("replace_backend, replace_pattern='%s'\n",replace_pattern);
 					tmpstr = unescape_string(replace_pattern, FALSE);
 				} else {
 					tmpstr = g_strdup(replace_pattern);
@@ -581,7 +580,7 @@ actions, so the first char in buf is actually number offset in the text widget *
 			g_strdown(tmpstr);
 		break;
 		}
-		DEBUG_MSG("replace_backend, len=%d, offset=%d, start=%d, end=%d, document=%p\n", result.end - result.start, offset, result.start + offset, result.end + offset, doc);
+		DEBUG_MSG("replace_backend, len=%d, offset=%d, start=%d, end=%d, document=%p, tmpstr='%s'\n", result.end - result.start, offset, result.start + offset, result.end + offset, doc,tmpstr);
 		doc_replace_text_backend(doc, tmpstr, result.start + offset, result.end + offset);
 		if (*replacelen == -1) {
 			*replacelen = g_utf8_strlen(tmpstr, -1);
@@ -623,7 +622,7 @@ Tsearch_result replace_doc_once(Tbfwin *bfwin,gchar *search_pattern, Tmatch_type
 
 	doc_unre_new_group(doc);
 	fulltext = doc_get_chars(doc, startpos, endpos);
-	result = replace_backend(bfwin,search_pattern, matchtype, is_case_sens, fulltext, replace_pattern, doc, startpos, replacetype, &replacelen, unescape);
+	result = replace_backend(bfwin,search_pattern, matchtype, is_case_sens, fulltext, 0, replace_pattern, doc, startpos, replacetype, &replacelen, unescape);
 	if ( result.end > 0) {
 		LASTSNR2(bfwin->snr2)->result.start = result.start + startpos;
 		LASTSNR2(bfwin->snr2)->result.end = result.end + startpos;
@@ -664,7 +663,7 @@ void replace_doc_multiple(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matc
 	gchar *fulltext, *realpats, *realpatr;
 	gboolean realunesc;
 	Tsearch_result result;
-	gint in_buf_offset=0;
+	gint buf_byte_offset=0;
 	gint buf_text_offset=startpos;
 	gint replacelen; /* replacelen -1 means there is no replacelen known yet */
 	doc_unre_new_group(doc);
@@ -676,6 +675,7 @@ void replace_doc_multiple(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matc
 		if (unescape) {
 			realpats = unescape_string(search_pattern, FALSE);
 			realpatr = unescape_string(replace_pattern, FALSE);
+			DEBUG_MSG("replace_doc_multiple, unescaped patterns, realpats='%s', realpatr='%s'\n",realpats, realpatr);
 		} else {
 			realpats = search_pattern;
 			realpatr = replace_pattern;
@@ -689,38 +689,29 @@ void replace_doc_multiple(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matc
 		realunesc = unescape;
 	}
 	fulltext = doc_get_chars(doc, startpos, endpos);
-	result = replace_backend(bfwin,realpats, matchtype, is_case_sens, fulltext, realpatr, doc, buf_text_offset, replacetype, &replacelen, realunesc);
+	result = replace_backend(bfwin,realpats, matchtype, is_case_sens, fulltext, 0, realpatr, doc, buf_text_offset, replacetype, &replacelen, realunesc);
 	while (result.end > 0) {
 		if (replacetype == string) {
 			buf_text_offset += replacelen - (result.end - result.start);
 		}
 		if (LASTSNR2(bfwin->snr2)->overlapping_search) {
-/*			if (GTK_TEXT(doc->textbox)->use_wchar) {
-				onechar = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos + result.start, startpos + result.start + 1);
-				if (onechar) {
-					in_buf_offset += result.bstart + strlen(onechar);
-				} else {
-					in_buf_offset += result.bstart + 1;
-				}
-				g_free(onechar);
-			} else {*/
-				in_buf_offset += result.bstart + 1;
-/*			}*/
-			buf_text_offset += result.start + 1;
+			buf_byte_offset = result.bstart + 1;
+			/* buf_text_offset += result.start + 1; */
 		} else {
-			in_buf_offset += result.bend;
-			buf_text_offset += result.end;
+			buf_byte_offset = result.bend;
+			/* buf_text_offset += result.end; */
 		}
-		DEBUG_MSG("replace_doc_multiple, after first search, buf_text_offset=%d, in_buf_offset=%d\n", buf_text_offset, in_buf_offset);
+		DEBUG_MSG("replace_doc_multiple, after first search, buf_text_offset=%d, buf_byte_offset=%d\n", buf_text_offset, buf_byte_offset);
 		if (matchtype != match_normal && replacetype == string) {
 			/* all regex replaces can have different replace lengths, so they have to be re-calculated */
 			replacelen = -1;
 		}
-		result = replace_backend(bfwin,realpats, matchtype, is_case_sens, &fulltext[in_buf_offset], realpatr, doc, buf_text_offset, replacetype, &replacelen, realunesc);
+		result = replace_backend(bfwin,realpats, matchtype, is_case_sens, fulltext, buf_byte_offset, realpatr, doc, buf_text_offset, replacetype, &replacelen, realunesc);
 
-		DEBUG_MSG("replace_doc_multiple, 1- buf_text_offset=%d, in_buf_offset=%d, result.start=%d, result.end=%d\n", buf_text_offset, in_buf_offset, result.start, result.end);
+		DEBUG_MSG("replace_doc_multiple, 1- buf_text_offset=%d, buf_byte_offset=%d, result.start=%d, result.end=%d\n", buf_text_offset, buf_byte_offset, result.start, result.end);
 	}
 	if (unescape && (matchtype == match_normal || replacetype != string)) {
+		DEBUG_MSG("replace_doc_multiple, free-ing realpats and realpatr\n");
 		g_free(realpats);
 		g_free(realpatr);
 	}
@@ -737,6 +728,7 @@ void replace_doc_multiple(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matc
 
 /**
  * replace_all:
+ * @bfwin: #Tbfwin* the window with the documents to act upon
  * @search_pattern: #gchar* to search pattern
  * @matchtype: see #Tmatch_types
  * @is_case_sens: #gint
@@ -775,45 +767,51 @@ static void replace_prompt_dialog_ok_lcb(GtkWidget *widget, Tbfwin *bfwin) {
 	window_close_by_widget_cb(widget, NULL);
 	
 	doc_get_selection(bfwin->current_document, &sel_start_pos, &sel_end_pos);
-		if ((sel_start_pos == LASTSNR2(bfwin->snr2)->result.start) &&
-					(sel_end_pos == LASTSNR2(bfwin->snr2)->result.end)) {
-
-			if (LASTSNR2(bfwin->snr2)->replacetype_option==string) {
-				tmpstr = g_strdup(LASTSNR2(bfwin->snr2)->replace_pattern);
-				/* if it was a regex replace we need to do the sub-search_pattern matching */
-				tmpstr = reg_replace(tmpstr, 0, LASTSNR2(bfwin->snr2)->result, bfwin->current_document, LASTSNR2(bfwin->snr2)->unescape);
-				
-			} else if (LASTSNR2(bfwin->snr2)->replacetype_option==uppercase) {
-				tmpstr = doc_get_chars(bfwin->current_document, LASTSNR2(bfwin->snr2)->result.start ,LASTSNR2(bfwin->snr2)->result.end);
-				g_strup(tmpstr);
-			} else {
-				tmpstr = doc_get_chars(bfwin->current_document, LASTSNR2(bfwin->snr2)->result.start ,LASTSNR2(bfwin->snr2)->result.end);
-				g_strdown(tmpstr);
-			}
-			/* avoid new highlighting at this stage, so call the backend directly instead of the frontend function
-			this because the highlighting interferes with the selection
-			the better solution is to have the highlighting handle the selection better, 
-			the problem starts in document.c in get_positions() because the selection is not saved there
-			I don't know why the selection is gray, but that's basically the reason why it doesn't save the selection
-			 */
-			doc_unre_new_group(bfwin->current_document);
-			doc_replace_text_backend(bfwin->current_document, tmpstr, LASTSNR2(bfwin->snr2)->result.start,LASTSNR2(bfwin->snr2)->result.end);
-			doc_unre_new_group(bfwin->current_document);
-			doc_set_modified(bfwin->current_document, 1);
+	if ((sel_start_pos == LASTSNR2(bfwin->snr2)->result.start) &&
+				(sel_end_pos == LASTSNR2(bfwin->snr2)->result.end)) {
+		gint lenadded;
+		if (LASTSNR2(bfwin->snr2)->replacetype_option==string) {
+			tmpstr = g_strdup(LASTSNR2(bfwin->snr2)->replace_pattern);
+			/* if it was a regex replace we need to do the sub-search_pattern matching */
+			tmpstr = reg_replace(tmpstr, 0, LASTSNR2(bfwin->snr2)->result, bfwin->current_document, LASTSNR2(bfwin->snr2)->unescape);
 			
-			g_free(tmpstr);
-			if (LASTSNR2(bfwin->snr2)->result.pmatch) {
-				g_free(LASTSNR2(bfwin->snr2)->result.pmatch);
-				LASTSNR2(bfwin->snr2)->result.pmatch = NULL;
-			}
-			if (!LASTSNR2(bfwin->snr2)->replace_once) {
-				snr2_run(bfwin,NULL);
-			}
+		} else if (LASTSNR2(bfwin->snr2)->replacetype_option==uppercase) {
+			tmpstr = doc_get_chars(bfwin->current_document, LASTSNR2(bfwin->snr2)->result.start ,LASTSNR2(bfwin->snr2)->result.end);
+			g_strup(tmpstr);
+		} else {
+			tmpstr = doc_get_chars(bfwin->current_document, LASTSNR2(bfwin->snr2)->result.start ,LASTSNR2(bfwin->snr2)->result.end);
+			g_strdown(tmpstr);
 		}
+		/* avoid new highlighting at this stage, so call the backend directly instead of the frontend function
+		this because the highlighting interferes with the selection
+		the better solution is to have the highlighting handle the selection better, 
+		the problem starts in document.c in get_positions() because the selection is not saved there
+		I don't know why the selection is gray, but that's basically the reason why it doesn't save the selection
+		 */
+
+		doc_unre_new_group(bfwin->current_document);
+		doc_replace_text_backend(bfwin->current_document, tmpstr, LASTSNR2(bfwin->snr2)->result.start,LASTSNR2(bfwin->snr2)->result.end);
+		doc_unre_new_group(bfwin->current_document);
+		doc_set_modified(bfwin->current_document, 1);
+
+		lenadded = strlen(tmpstr) - (LASTSNR2(bfwin->snr2)->result.end - LASTSNR2(bfwin->snr2)->result.start);
+		DEBUG_MSG("lenadded=%d (streln=%d, end-start=%d)\n",lenadded,strlen(tmpstr),(LASTSNR2(bfwin->snr2)->result.end - LASTSNR2(bfwin->snr2)->result.start));
+		g_free(tmpstr);
+		if (LASTSNR2(bfwin->snr2)->result.pmatch) {
+			g_free(LASTSNR2(bfwin->snr2)->result.pmatch);
+			LASTSNR2(bfwin->snr2)->result.pmatch = NULL;
+		}
+		if (!LASTSNR2(bfwin->snr2)->replace_once) {
+			if (!LASTSNR2(bfwin->snr2)->overlapping_search && lenadded > 0) {
+				LASTSNR2(bfwin->snr2)->result.end += lenadded;
+			}
+			snr2_run(bfwin,NULL);
+		}
+	}
 #ifdef DEBUG
-		 else {
-			g_print("replace_prompt_dialog_ok_lcb, selection != result, not replacing!!\n");
-		}
+	 else {
+		g_print("replace_prompt_dialog_ok_lcb, selection != result, not replacing!!\n");
+	}
 #endif /* DEBUG */
 }
 
@@ -916,23 +914,25 @@ gint replace_prompt_doc(Tbfwin *bfwin, gchar *search_pattern, Tmatch_types match
 	gchar *fulltext, *realpat;
 	Tsearch_result result;
 
+	DEBUG_MSG("replace_prompt_doc, doc=%p, startpos=%d, endpos=%d\n",doc,startpos,endpos);
 	if (LASTSNR2(bfwin->snr2)->result.pmatch) {
 		g_free(LASTSNR2(bfwin->snr2)->result.pmatch);
 		LASTSNR2(bfwin->snr2)->result.pmatch = NULL;
 	}
 	fulltext = doc_get_chars(doc, startpos, endpos);
+	utf8_offset_cache_reset();
 	if (unescape) {
 		realpat = unescape_string(search_pattern, FALSE);
 	} else {
 		realpat = search_pattern;
 	}
-	result = search_backend(bfwin,realpat, matchtype, is_case_sens, fulltext, 1);
+	result = search_backend(bfwin,realpat, matchtype, is_case_sens, fulltext,0, 1);
 	if (unescape) {
 		g_free(realpat);
 	}
 	LASTSNR2(bfwin->snr2)->doc = doc;
 	g_free(fulltext);
-	DEBUG_MSG("replace_prompt_doc, result.end=%d\n", result.end);
+	DEBUG_MSG("replace_prompt_doc, doc=%p, result.end=%d\n", doc, result.end);
 	if (result.end > 0) {
 		gint i;
 		LASTSNR2(bfwin->snr2)->result.start = result.start + startpos;
@@ -975,11 +975,13 @@ void replace_prompt_all(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matcht
 		tmplist = g_list_first(bfwin->documentlist);
 		tmpdoc = (Tdocument *)tmplist->data;
 	}
-	retvalue = replace_prompt_doc(bfwin,search_pattern, matchtype, is_case_sens, 0, -1, replace_pattern, tmpdoc, unescape);
+	DEBUG_MSG("replace_prompt_all, starting with tmpdoc=%p at position %d\n",tmpdoc,LASTSNR2(bfwin->snr2)->result.end);
+	retvalue = replace_prompt_doc(bfwin,search_pattern, matchtype, is_case_sens, (LASTSNR2(bfwin->snr2)->result.end < 0)?0:LASTSNR2(bfwin->snr2)->result.end, -1, replace_pattern, tmpdoc, unescape);
 	while (retvalue == 0) {
 		tmplist = g_list_find(bfwin->documentlist, LASTSNR2(bfwin->snr2)->doc);
 		tmplist = g_list_next(tmplist);
 		if (tmplist) {
+			DEBUG_MSG("replace_prompt_all, next document is %p\n",tmplist->data);
 			retvalue = replace_prompt_doc(bfwin,search_pattern, matchtype, is_case_sens, 0, -1, replace_pattern, (Tdocument *)tmplist->data, unescape);
 		} else {
 			retvalue = 1;
@@ -987,6 +989,58 @@ void replace_prompt_all(Tbfwin *bfwin,gchar *search_pattern, Tmatch_types matcht
 	}
 }
 
+static void search_doc_bookmark_backend(Tbfwin *bfwin,Tdocument *document, gchar *search_pattern, Tmatch_types matchtype, gint is_case_sens, gint startpos, gboolean unescape) {
+	gchar *fulltext, *realpat;
+	gint buf_byte_offset=0;
+	Tsearch_result result;
+	fulltext = doc_get_chars(document, 0, -1);
+	utf8_offset_cache_reset();
+	if (unescape) {
+		realpat = unescape_string(search_pattern, FALSE);
+	} else {
+		realpat = search_pattern;
+	}
+	result = search_backend(bfwin,realpat, matchtype, is_case_sens, fulltext, 0, FALSE);
+	while (result.end > 0) {
+		gchar *text = doc_get_chars(document, result.start, result.end);
+		DEBUG_MSG("search_bookmark, adding bookmark '%s' at %d\n", text, result.start);
+		bmark_add_extern(document, result.start, NULL, text, !main_v->props.bookmarks_default_store);
+		g_free(text);
+		if (LASTSNR2(bfwin->snr2)->overlapping_search) {
+			buf_byte_offset = result.bstart + 1;
+		} else {
+			buf_byte_offset = result.bend;
+		}
+		result = search_backend(bfwin,realpat, matchtype, is_case_sens, fulltext, buf_byte_offset, FALSE);
+	}
+	if (unescape) {
+		g_free(realpat);
+	}
+	g_free(fulltext);
+}
+
+/**
+ * search_bookmark:
+ * @bfwin: #Tbfwin *
+ * @startat: #gint
+ *
+ * will search, and bookmark all matches
+ * 
+ */
+static void search_bookmark(Tbfwin *bfwin, gint startat) {
+	DEBUG_MSG("search_bookmark, started\n");
+	if (LASTSNR2(bfwin->snr2)->placetype_option==opened_files) {
+		GList *tmplist = g_list_first(bfwin->documentlist);
+		while (tmplist) {
+			search_doc_bookmark_backend(bfwin,DOCUMENT(tmplist->data), LASTSNR2(bfwin->snr2)->search_pattern, LASTSNR2(bfwin->snr2)->matchtype_option, LASTSNR2(bfwin->snr2)->is_case_sens, 0, LASTSNR2(bfwin->snr2)->unescape);
+			tmplist = g_list_next(tmplist);
+		}
+	} else {
+		search_doc_bookmark_backend(bfwin,DOCUMENT(bfwin->current_document), LASTSNR2(bfwin->snr2)->search_pattern, LASTSNR2(bfwin->snr2)->matchtype_option, LASTSNR2(bfwin->snr2)->is_case_sens, startat, LASTSNR2(bfwin->snr2)->unescape);
+	}
+	DEBUG_MSG("search_bookmark, done\n");
+}
+
 /*****************************************************/
 
 /**
@@ -1063,21 +1117,25 @@ void snr2_run(Tbfwin *bfwin, Tdocument *doc) {
 			}		
 		}
 	} else { /* find, not replace */
-		if (LASTSNR2(bfwin->snr2)->placetype_option==opened_files) {
-			DEBUG_MSG("snr2dialog_ok_lcb, search = all\n");
-			result_all = search_all(bfwin,LASTSNR2(bfwin->snr2)->search_pattern, LASTSNR2(bfwin->snr2)->matchtype_option, LASTSNR2(bfwin->snr2)->is_case_sens, LASTSNR2(bfwin->snr2)->unescape);
-			DEBUG_MSG("snr2dialog_ok_lcb, result_all.doc=%p\n",result_all.doc);
-			if (result_all.end > 0) {
-				doc_show_result(result_all.doc, result_all.start, result_all.end);
-			} else {
-				info_dialog(bfwin->main_window,_("Search: no match found"), NULL);
-			}
+		if (LASTSNR2(bfwin->snr2)->bookmark_results) {
+			search_bookmark(bfwin, startpos);
 		} else {
-			result = search_doc(bfwin,doc, LASTSNR2(bfwin->snr2)->search_pattern, LASTSNR2(bfwin->snr2)->matchtype_option, LASTSNR2(bfwin->snr2)->is_case_sens, startpos, LASTSNR2(bfwin->snr2)->unescape);
-			if (result.end > 0) {
-				doc_show_result(doc, result.start, result.end);	
+			if (LASTSNR2(bfwin->snr2)->placetype_option==opened_files) {
+				DEBUG_MSG("snr2dialog_ok_lcb, search = all\n");
+				result_all = search_all(bfwin,LASTSNR2(bfwin->snr2)->search_pattern, LASTSNR2(bfwin->snr2)->matchtype_option, LASTSNR2(bfwin->snr2)->is_case_sens, LASTSNR2(bfwin->snr2)->unescape);
+				DEBUG_MSG("snr2dialog_ok_lcb, result_all.doc=%p\n",result_all.doc);
+				if (result_all.end > 0) {
+					doc_show_result(result_all.doc, result_all.start, result_all.end);
+				} else {
+					info_dialog(bfwin->main_window,_("Search: no match found"), NULL);
+				}
 			} else {
-				info_dialog(bfwin->main_window,_("Search: no match found"), NULL);
+				result = search_doc(bfwin,doc, LASTSNR2(bfwin->snr2)->search_pattern, LASTSNR2(bfwin->snr2)->matchtype_option, LASTSNR2(bfwin->snr2)->is_case_sens, startpos, LASTSNR2(bfwin->snr2)->unescape);
+				if (result.end > 0) {
+					doc_show_result(doc, result.start, result.end);
+				} else {
+					info_dialog(bfwin->main_window,_("Search: no match found"), NULL);
+				}
 			}
 		}
 	}
@@ -1117,14 +1175,14 @@ void snr2_run_extern_replace(Tdocument *doc, gchar *search_pattern, gint region,
 	search_pattern_bck = LASTSNR2(bfwin->snr2)->search_pattern;
 	replace_pattern_bck = LASTSNR2(bfwin->snr2)->replace_pattern;
 	last_snr2_bck = *LASTSNR2(bfwin->snr2);
-	DEBUG_MSG("last_snr2_bck.search_pattern=%p, replace_pattern=%p\n"
+	DEBUG_MSG("snr2..extern..: last_snr2_bck.search_pattern=%p, replace_pattern=%p\n"
 		,last_snr2_bck.search_pattern, last_snr2_bck.replace_pattern);
 
 	if (!search_pattern || !replace_pattern || !strlen(search_pattern)) {
 		DEBUG_MSG("snr2_run_extern, returning, non-valid arguments\n");
 		return;
 	}
-	DEBUG_MSG("snr2..extern..: doc=%p, search_pattern=%s, region=%d, matchtype=%d, is_case_sens=%d, replace_pattern=%s, store_as_last=%d\n"
+	DEBUG_MSG("snr2..extern..: doc=%p, search_pattern='%s', region=%d, matchtype=%d, is_case_sens=%d, replace_pattern=%s, store_as_last=%d\n"
 			,doc,search_pattern,region,matchtype,is_case_sens,replace_pattern,store_as_last_snr2);
 	LASTSNR2(bfwin->snr2)->search_pattern = g_strdup(search_pattern);
 	LASTSNR2(bfwin->snr2)->placetype_option = region;
@@ -1134,8 +1192,10 @@ void snr2_run_extern_replace(Tdocument *doc, gchar *search_pattern, gint region,
 	LASTSNR2(bfwin->snr2)->replace_pattern = g_strdup(replace_pattern);
  	LASTSNR2(bfwin->snr2)->prompt_before_replace = 0;
  	LASTSNR2(bfwin->snr2)->replace_once = 0;
+ 	LASTSNR2(bfwin->snr2)->unescape = 0;
 	LASTSNR2(bfwin->snr2)->matchtype_option = matchtype;
  	LASTSNR2(bfwin->snr2)->replacetype_option = string;
+ 	LASTSNR2(bfwin->snr2)->bookmark_results = 0;
 	snr2_run(BFWIN(doc->bfwin),doc);
 	if (store_as_last_snr2) {
 		DEBUG_MSG("free-ing old patterns at %p and %p\n",search_pattern_bck,replace_pattern_bck);
@@ -1201,7 +1261,7 @@ static void snr2dialog_ok_lcb(GtkWidget *widget, Tsnr2_win *data) {
 	LASTSNR2(bfwin->snr2)->search_pattern = gtk_text_buffer_get_text(buf,&itstart,&itend, FALSE);*/
 	LASTSNR2(bfwin->snr2)->search_pattern = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(data->search_combo)->entry),0,-1);
 	
-	data->bfwin->session->searchlist = add_to_history_stringlist(data->bfwin->session->searchlist,LASTSNR2(bfwin->snr2)->search_pattern,TRUE);
+	data->bfwin->session->searchlist = add_to_history_stringlist(data->bfwin->session->searchlist,LASTSNR2(bfwin->snr2)->search_pattern,TRUE,TRUE);
 	
 	LASTSNR2(bfwin->snr2)->unescape = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->unescape));
  	LASTSNR2(bfwin->snr2)->is_case_sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->is_case_sens));
@@ -1214,12 +1274,13 @@ static void snr2dialog_ok_lcb(GtkWidget *widget, Tsnr2_win *data) {
 		LASTSNR2(bfwin->snr2)->replace = 1;
 		LASTSNR2(bfwin->snr2)->replace_pattern = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(data->replace_combo)->entry),0,-1);
 		
-		data->bfwin->session->replacelist = add_to_history_stringlist(data->bfwin->session->replacelist,LASTSNR2(bfwin->snr2)->replace_pattern,TRUE);
+		data->bfwin->session->replacelist = add_to_history_stringlist(data->bfwin->session->replacelist,LASTSNR2(bfwin->snr2)->replace_pattern,TRUE,TRUE);
 		
 	 	LASTSNR2(bfwin->snr2)->prompt_before_replace = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->prompt_before_replace));
 	 	LASTSNR2(bfwin->snr2)->replace_once = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->replace_once));
 	} else {
 		LASTSNR2(bfwin->snr2)->replace = 0;
+		LASTSNR2(bfwin->snr2)->bookmark_results = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->bookmark_results));
 	}
 	window_destroy(data->window);
 	g_free(data);
@@ -1271,7 +1332,7 @@ static void snr2dialog(Tbfwin *bfwin, gint is_replace, gint is_new_search) {
 		tmptext = _("Replace");
 		snr2win->replace = 1;
 	} else {
-		tmptext = _("Search");
+		tmptext = _("Find");
 		snr2win->replace = 0;
 	}
 	if (is_new_search) {
@@ -1297,7 +1358,7 @@ static void snr2dialog(Tbfwin *bfwin, gint is_replace, gint is_new_search) {
 	gtk_table_set_row_spacings (GTK_TABLE (table), 12);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
-	snr2win->search_label = gtk_label_new_with_mnemonic(_("Search _for: "));
+	snr2win->search_label = gtk_label_new_with_mnemonic(_("_Search for: "));
 	gtk_table_attach (GTK_TABLE (table), snr2win->search_label, 0, 1, 0, 1,
 					(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 2, 0);
 	gtk_label_set_justify (GTK_LABEL (snr2win->search_label), GTK_JUSTIFY_LEFT);
@@ -1310,7 +1371,7 @@ static void snr2dialog(Tbfwin *bfwin, gint is_replace, gint is_new_search) {
 	/*g_signal_connect(G_OBJECT(snr2win->search_entry), "key_press_event", G_CALLBACK(search_entry_key_press_event_lcb), snr2win);*/
 
 	if (is_replace) {
-		snr2win->replace_label = gtk_label_new_with_mnemonic(_("Replace wit_h: "));
+		snr2win->replace_label = gtk_label_new_with_mnemonic(_("_Replace with: "));
 		gtk_table_attach (GTK_TABLE (table), snr2win->replace_label, 0, 1, 1, 2,
 						(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 2, 0);
 		gtk_label_set_justify (GTK_LABEL (snr2win->replace_label), GTK_JUSTIFY_LEFT);
@@ -1377,12 +1438,14 @@ static void snr2dialog(Tbfwin *bfwin, gint is_replace, gint is_new_search) {
 			g_signal_connect(G_OBJECT((GtkWidget *) snr2win->replacetype_option), "changed", G_CALLBACK(replacetype_changed_lcb), snr2win);
 		}
 	}
-	snr2win->unescape = boxed_checkbut_with_value(_("Patterns contain backslash escape sequences (\\n, \\t)"), LASTSNR2(bfwin->snr2)->unescape, vbox);
-	snr2win->is_case_sens = boxed_checkbut_with_value(_("Match ca_se"), LASTSNR2(bfwin->snr2)->is_case_sens, vbox);
+	snr2win->unescape = boxed_checkbut_with_value(_("_Patterns contain backslash escape sequences (\\n, \\t)"), LASTSNR2(bfwin->snr2)->unescape, vbox);
+	snr2win->is_case_sens = boxed_checkbut_with_value(_("_Match case"), LASTSNR2(bfwin->snr2)->is_case_sens, vbox);
 	snr2win->overlapping_search = boxed_checkbut_with_value(_("O_verlap searches"), LASTSNR2(bfwin->snr2)->overlapping_search, vbox);
 	if (is_replace) {
 		snr2win->prompt_before_replace = boxed_checkbut_with_value(_("Prompt _before replace"), LASTSNR2(bfwin->snr2)->prompt_before_replace, vbox);
-		snr2win->replace_once = boxed_checkbut_with_value(_("Rep_lace once"), LASTSNR2(bfwin->snr2)->replace_once, vbox);
+		snr2win->replace_once = boxed_checkbut_with_value(_("Replace o_nce"), LASTSNR2(bfwin->snr2)->replace_once, vbox);
+	} else {
+		snr2win->bookmark_results = boxed_checkbut_with_value(_("Bookmark results"), LASTSNR2(bfwin->snr2)->bookmark_results, vbox);
 	}
 
 	hbox = gtk_hseparator_new ();
@@ -1399,9 +1462,9 @@ static void snr2dialog(Tbfwin *bfwin, gint is_replace, gint is_new_search) {
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
 	gtk_window_set_default(GTK_WINDOW(snr2win->window), button);
 	
-	gtk_label_set_mnemonic_widget(GTK_LABEL(snr2win->search_label), GTK_WIDGET(snr2win->search_combo));
+	gtk_label_set_mnemonic_widget(GTK_LABEL(snr2win->search_label), GTK_COMBO(snr2win->search_combo)->entry);
 	if(is_replace) {
-		gtk_label_set_mnemonic_widget(GTK_LABEL(snr2win->replace_label), GTK_WIDGET(snr2win->replace_combo));
+		gtk_label_set_mnemonic_widget(GTK_LABEL(snr2win->replace_label), GTK_COMBO(snr2win->replace_combo)->entry);
 	}
 	gtk_widget_grab_focus(snr2win->search_combo);
 	gtk_widget_show_all(vbox);
@@ -1592,6 +1655,7 @@ void update_filenames_in_file(Tdocument *doc, gchar *oldfilename, gchar *newfile
 	}
 
 	fulltext = doc_get_chars(doc, 0, -1);
+	utf8_offset_cache_reset();
 	result = search_backend(search_pattern, matchtype, is_case_sens, fulltext, 0);
 	while (result.end > 0) {
 		if (doc_has_newfilename) {
@@ -1626,19 +1690,32 @@ void update_encoding_meta_in_file(Tdocument *doc, gchar *encoding) {
 		gchar *search_pattern, *fulltext;
 		Tsearch_result result;
 		/* first find if there is a meta encoding tag already */
-		search_pattern = "<meta[ \t\n]http-equiv[ \t\n]*=[ \t\n]*\"content-type\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"text/html;[ \t\n]*charset=[a-z0-9-]+\"[ \t\n]*>";
+		search_pattern = "<meta[ \t\n]http-equiv[ \t\n]*=[ \t\n]*\"content-type\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"([^;]*);[ \t\n]*charset=[a-z0-9-]*\"[ \t\n]*(/?)>";
 		fulltext = doc_get_chars(doc, 0, -1);
-		result = search_backend(bfwin,search_pattern, match_posix, 0, fulltext, 0);
+		utf8_offset_cache_reset();
+		result = search_backend(bfwin,search_pattern, match_posix, 0, fulltext, 0, 1);
 		if (result.end > 0) {
-			gchar *replacestring = g_strconcat("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=",encoding,"\">", NULL);
+			gchar *replacestring, *type, *xhtmlend;
+			DEBUG_MSG("update_encoding_meta_in_file, we have a match, nmatch=%d\n",result.nmatch);
+			if (result.nmatch > 2) {
+				type = g_strndup(fulltext+result.pmatch[1].rm_so, result.pmatch[1].rm_eo - result.pmatch[1].rm_so);
+				xhtmlend = g_strndup(fulltext+result.pmatch[2].rm_so, result.pmatch[2].rm_eo - result.pmatch[2].rm_so);
+				DEBUG_MSG("update_encoding_meta_in_file, type=%s (bstart=%d, bend=%d, so[1]=%d, eo[1]=%d)\n",type,result.bstart,result.bend,result.pmatch[1].rm_so,result.pmatch[1].rm_eo);
+			} else {
+				type = g_strdup("text/html");
+				xhtmlend = g_strdup( main_v->props.xhtml ? "/" : "");
+			}
+			replacestring = g_strconcat("<meta http-equiv=\"Content-Type\" content=\"",type,"; charset=",encoding,"\" ",xhtmlend,">", NULL);
 			DEBUG_MSG("update_encoding_meta_in_file, 1: we have a match\n");
 			doc_replace_text(doc, replacestring, result.start, result.end);
 			g_free(replacestring);
+			g_free(type);
+			g_free(xhtmlend);
 		} else {
 			DEBUG_MSG("update_encoding_meta_in_file, 1: NO match\n");
 			/* now search for <head>, we can append it to this tag */
 			search_pattern = "<head>";
-			result = search_backend(bfwin,search_pattern, match_posix, 0, fulltext, 0);
+			result = search_backend(bfwin,search_pattern, match_posix, 0, fulltext, 0, 0);
 			if (result.end > 0) {
 				gchar *replacestring = g_strconcat("<head>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=",encoding,"\">", NULL);
 				DEBUG_MSG("update_encoding_meta_in_file, 2: we have a match\n");
