@@ -31,7 +31,7 @@ gint count_array(gchar **array);
 gchar *array_to_string(gchar **array);
 gchar **string_to_array(gchar *string);
 gchar **array_from_arglist(const gchar *string1, ...);
-
+GList *list_from_arglist(gboolean allocate_strings, ...);
 GList *duplicate_stringlist(GList *list, gint dup_data);
 gint free_stringlist(GList * which_list);
 GList *get_list(const gchar * filename, GList * which_list, gboolean is_arraylist);
@@ -46,11 +46,17 @@ GList *duplicate_arraylist(GList *arraylist);
 /* removes a string from a stringlist if it is the same */
 GList *remove_from_stringlist(GList *which_list, const gchar * string);
 GList *limit_stringlist(GList *which_list, gint num_entries, gboolean keep_end);
-GList *add_to_history_stringlist(GList *which_list, const gchar *string, gboolean move_to_end);
+GList *add_to_history_stringlist(GList *which_list, const gchar *string, gboolean recent_on_top, gboolean move_if_exists);
 /* adds a string to a stringlist if it is not yet in there */
 GList *add_to_stringlist(GList * which_list, const gchar * string);
 gchar *stringlist_to_string(GList *stringlist, gchar *delimiter);
-gboolean arraylist_value_exists(GList *arraylist, gchar **value, gint testlevel);
+
+gint array_n_strings_identical(gchar **array1, gchar **array2, gboolean case_sensitive, gint testlevel);
+GList *arraylist_delete_identical(GList *arraylist, gchar **compare, gint testlevel, gboolean case_sensitive);
+GList *arraylist_append_identical_from_list(GList *thelist, GList *source, gchar **compare, gint testlevel, gboolean case_sensitive);
+GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcefilename, gchar **compare, gint testlevel, gboolean case_sensitive);
+
+gboolean arraylist_value_exists(GList *arraylist, gchar **value, gint testlevel, gboolean case_sensitive);
 GList *arraylist_load_new_identifiers_from_list(GList *mylist, GList *deflist, gint uniquelevel);
 GList *arraylist_load_new_identifiers_from_file(GList *mylist, const gchar *fromfilename, gint uniquelevel);
 #endif							/* __STRINGLIST_H_ */
