@@ -2,6 +2,7 @@
  * wizards.h - wizard callback prototypes
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
+ * Copyright (C) 1999-2002 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +22,7 @@
 #ifndef __WIZARDS_H_
 #define __WIZARDS_H_
 
-void tablewizard(GtkWidget *widget, gpointer data);
-void framewizard(GtkWidget * widget, gpointer data);
-
+void tablewizard_dialog(Tbfwin *bfwin);
+void framewizard_dialog(Tbfwin *bfwin);
 
 #endif							/* __WIZARDS_H_ */
